
#********************************************************************************************************************
#
#                              SOURCE Script to create Control.dat that will be called by BMD2_Extract.exe
#       Then, executes Extract_BMD2.exe to read model BMD2 file, extract model segments and parameters, and write out extracted *.txt
#  
#      *************NOTE: In the Excel Lookup, only select Pcodes and Segments that appear in the model *.BMD2.*************** 
#          The Extract_BMD2.exe may extract spurious values when a Pcode/Segment is selected in the Excel Lookup, 
#                            but it is **NOT** included in the model BMD2 output or monitoring data *.txt
#
#                               Uses arguments in Excel lookup *.xlsx to populate Control.Dat
#                                                              
#              The additional SOURCE scripts will then read back in the extracted *.txt for interpolation/processing/calibration
#                               
#                         Designed to run as a SOURCE code through Master calibration script   
#                               
#                     Extract_BMD2.exe and required .dll files needs to be in the 'Scripts' subfolder
#              Model output *.BMD2 needs to be in the Path_Model_In Filepath (filename is provided in Excel lookup)
#             Extracted Model *.txt will be written to the Path_Model_In filepath and named based on Excel Lookup
#             
#             
#        ********This version surrounds the WASP segment name in quotes to allow for spaces in the Segment name**********
#        ********This normalizes relative paths because FORTRAN and R interpret relative path notation differently**********
#             
#                                                           
#********************************************************************************************************************

#********************************************************************************************************************
#                 Extract arguments to be used in Control.Dat file that will be called by BMD2_Extract.exe
#********************************************************************************************************************

Path_Model_In_EXE <- paste(normalizePath(Path_Model_In), RawBMD_Filename, sep = "/") %>%                              ## Establishes filepath to BMD2 file; normalizes filepath to allow relative vs absolute paths (FORTRAN interprets relative differently than R)
  str_replace_all(., "/", "\\\\") %>%                                                                                 ## Replaces forward slashes with backward slashes
  shQuote()                                                                                                           ## Surrounds object with double quotes so extractor can use filepaths with spaces

Path_Extract <- paste(normalizePath(Path_Model_In), Model_Extracted_Filename, sep = "/") %>%                          ## Establishes filepath where extracted *.txt file will be written; normalizes filepath to allow relative vs absolute paths (FORTRAN interprets relative differently than R)
  str_replace_all(., "/", "\\\\") %>%                                                                                 ## Replaces forward slashes with backward slashes
  shQuote()                                                                                                           ## Surrounds object with double quotes so extractor can use filepaths with spaces

File_Format <- paste("\t", "\t", 2, sep = "")                                                                         ## File format of extracted file (2 = .txt, 1 = .csv)

Names_Seg <- Stat_Lookup %>%                                                                                          ## Generates calibration segments
  drop_na(., Calibrate) %>%                                                                                           ## Drops any segment that is not selected
  distinct(., Segment_ID, Segment_Name) %>%                                                                            ## Extracts names of model segments to extract/calibrate
  dplyr::transmute(Names_Mrgd = paste0(.$Segment_ID, " \t ", "\"", .$Segment_Name, "\"" )) %>%                         ## Concatenates unique combinations WASP_ID and Segment_Name columns
  as_vector(.)                                                                                                        ## Converts to character vectro

Numb_Seg <- paste("\t", "\t", length(Names_Seg), sep = " ")                                                           ## Numb of calibration segments argument

Names_Var <- PCode_Lookup %>%                                                                                         ## Generates calibration Pcodes
  drop_na(., Calibrate) %>%                                                                                           ## Drops any Pcode not selected for calibration
  distinct(., Anal_Name) %>%                                                                                          ## Extracts analyte names based on Model parameter name
  dplyr::transmute(Vars_Mrgd = paste("\"", Anal_Name, "\"", sep = "")) %>%                                            ## Forces R to add quotations (needed for Control.dat)
  as_vector(.)                                                                                                        ## Converts to character vector

Numb_Var <- paste("\t", length(Names_Var), sep = " ")                                                                 ## Number of calibration Pcodes


#********************************************************************************************************************
#                 Creates and saves Control.Dat file that will be called by BMD2_Extract.exe
#********************************************************************************************************************

Dat_File <- c(Path_Model_In_EXE, Path_Extract, File_Format, Numb_Seg, Names_Seg, Numb_Var, Names_Var)                          ## Creates Dat File based on arguments

write.table(Dat_File, file = paste(Path_Scripts, "Control.dat", sep = "/"),                                ## Writes out Control.dat file
            row.names = FALSE, col.names = FALSE, sep = "\t", quote = FALSE, qmethod = "double" )                     ## Drops quotes, except for Pcodes

#********************************************************************************************************************
#                      Creates .exe command that links to filepath of BMD2_Extract.exe
#********************************************************************************************************************

Shell_Command <- paste("cd ", str_remove(Path_Scripts, "/Scripts"), "/Scripts && BMD2_Extract.exe",  sep = "" )                              ## Creates command and filepath for .EXE execution

#********************************************************************************************************************
#                            Uses Shell to execute BMD2_Extract.exe that will open BMD2 file 
#        Extract the model segments and Pcodes and then writes out a *.txt file based on name provided in Excel lookup
##********************************************************************************************************************

shell(Shell_Command)                                                                                                 ## Executes .EXE based on args in Control.dat

#*********************************************************************************************************************
#       QA Check: Reads in Variables_in_BMD2.out to verify that selected Pcodes in BMD2 file
#           This .OUT file includes all variables exported to the .BMD2 and can be used to validate that a selected variable was exported to the .BMD2
#                  If any selected variables are missing, then the scripts will throw an error and stop
##********************************************************************************************************************

BMD2_Variables <- readLines(paste(Path_Scripts, "Variables_in_BMD2.out", sep = "/"), skipNul = TRUE) %>%             ## Imports the variables_in_BMD2.out
  data.frame("Index_Anal_Name" = .) %>%                                                                              ## Sets column name
  mutate(Index_Anal_Name = str_trim(Index_Anal_Name)) %>%                                                            ## Trims white space
  separate(Index_Anal_Name, c("Row_Index", "Anal_Name"), "^\\S*\\K\\s+") %>%                                         ## Splits into two columns at first space  
  mutate(Anal_Name = str_replace_all(Anal_Name, "\\s", "\\."))                                                       ## Converts spaces into periods

Pcode_Lookup_Variables <- PCode_Lookup %>%                                                                           ## Generates calibration Pcodes
  drop_na(., Calibrate) %>%                                                                                          ## Drops any Pcode not selected for calibration
  distinct(., Anal_Name) %>%                                                                                         ## Extracts distinct list of analytes
  mutate(Anal_Name = str_replace_all(Anal_Name, "\\s", "\\."))                                                       ## Converts spaces into periods


#*********************************************************************************************************************
#                   QA Check: Stops Calibration scripts if selected variable not in Variables_in_BMD.out file
#                   
##********************************************************************************************************************

if (length(setdiff(Pcode_Lookup_Variables$Anal_Name, BMD2_Variables$Anal_Name)) != 0){
  stop(paste("The following analyte(s) was selected for calibration, but was missing from the .BMD2 input data:", paste(setdiff(Pcode_Lookup_Variables$Anal_Name, BMD2_Variables$Anal_Name), collapse = ", "), "\nVerify that the selected analyte(s) was exported to the BMD2 Filename:", RawBMD_Filename))      ## Error / ends script if Measured DF is missing a required column
} else {
  print(paste("All analytes selected for calibration were found within the *.BMD2 file:", RawBMD_Filename))
}

