


#********************************************************************************************************************
#
#                              SOURCE Script to extract and import LSPC .OUT files
#                              
#                         Requires IF statement to run either LSPC or BMD2 extractor
#      
#       Will export a formatted .txt file that matches the format of the file exported by the FORTRAN BMD2 extractor, allowing subsequent scripts to be used
#       
#       
#       Currently, scripts have not been set-up to apply unit conversions to Precip, Shear Stress, or Depth
#                                                           
#                   Units of flow and nutrients will be converted based on PERO, RO, and CMS vs CFS
#********************************************************************************************************************


#********************************************************************************************************************
#                 Extract arguments that will be used to import/process selected LSPC .OUT files
#********************************************************************************************************************

Subbasin_Vec <- Stat_Lookup %>%                                                                                                                            ## Create vector of .OUT files to import
  drop_na(Calibrate) %>%                                                                                                                                   ## Drop any subbasin that isn't selected for calibration
  .$Segment_ID %>%
  unique() 


Out_Files <- paste0(Path_Model_In, "/", Subbasin_Vec, ".out")                                                                                                    ## Sets filepath and filename of .OUT files                       

#********************************************************************************************************************
#                 Extract header row that will be used to name columns
#          Only imports header from the first subbasin .OUT file in each filepath
#********************************************************************************************************************

Raw_Header <- read_lines(Out_Files[[1]])[grepl('^TT', read_lines(Out_Files[[1]]))] %>%                                  ## Reads in header banner of first calibration .OUT file
  str_trim(., side = c("both")) %>%                                                                                     ## Drops white space
  as.data.frame(stringsAsFactors = FALSE)  %>%                                                                          ## Converts DF
  .[which(. == "TT Label"):nrow(.),] %>%                                                                                ## Extracts all labels after row indicated TT Lable
  as.data.frame(stringsAsFactors = FALSE)  %>%                                                                          ## Converts DF again
  setNames(., "Orig_Name") %>%                                                                                          ## Sets column name
  mutate(., Orig_Name = str_remove(Orig_Name, "^TT ")) %>%                                                              ## Drops TT from dataframe
  mutate(Pcode =  gsub( " .*$", "", Orig_Name)) %>%                                                                     ## Extracts the Pcode from the original analyte name
  mutate(Anal_Name = str_remove(Orig_Name, Pcode)) %>%                                                                  ## Extracts analyte name by removing Pcode
  mutate(Anal_Name = gsub("(.*)\\(.*","\\1",Anal_Name)) %>%                                                             ## Drops the units at the end of the Anal_Name (after the parantheses)
  mutate_all(., str_trim, side = c("both")) %>%                                                                         ## Drops whitespace
  mutate(Pcode_temp = gsub( " .*$", "", Anal_Name)) %>%                                                                 ## Extracts additional Pcode names for CONC1
  mutate(Pcode = case_when(Pcode %in% c("CONC1", "CONC2", "CONC3") ~ paste(Pcode, Pcode_temp, sep = "_"),               ## Appends Pcode for CONC1, CONC2, and CONC2
                           TRUE ~ Pcode)) %>%
  select(-Pcode_temp) %>%                                                                                               ## Drops column
  dplyr::filter(., str_detect(Pcode, "TT|Label|Date|WATER", negate = TRUE))                                             ## Drops rows with these character strings, leaving only the column headings
 

#********************************************************************************************************************
#               Import PCode_Lookkup
#********************************************************************************************************************

PCode_Lookup <- PCode_Lookup  %>%                                                                                       ## Imports Parameter Crosswalk, but skips columns beyond Col 4
  dplyr::mutate(Calibrate = toupper(Calibrate)) %>%                                                                     ## Casts to uppercase                                             
  dplyr::mutate(Anal_Name = str_trim(Anal_Name, side = "both")) %>%                                                     ## Strips white space
  mutate(LSPC_Code = Raw_Header$Pcode[match(Anal_Name, Raw_Header$Anal_Name)]) %>%                                      ## Index match to map PCode and Analyte Name
  dplyr::mutate(Anal_Name = gsub(" ", ".", Anal_Name)) %>%                                                              ## Changes spaces into periods to coincide with column name format
  mutate(., Anal_Name = str_replace_all(Anal_Name, "[\\(\\)\\/\\\\]", ".")) %>%                                         ## Changes slashes and parentheses to periods
  {. ->> Flow_Units} %>%                                                                                                ## Exports intermediate DF 
  drop_na(., Calibrate)                                                                                                 ## Drops outfiles not selected for calibration

#********************************************************************************************************************
#                                Import the individual .OUT files and merge
#                                
#                       Uses REGEX to extract the subbasin ID from the .OUT filename and cbinds it to DF
#                       
#                 This avoids subbasin IDs that may occur from the Segment ID that is reported in the TT column of the .OUT files
#                 
#                                  Also, allows hyphenated Subbasin IDs to be used
#********************************************************************************************************************

Raw_LSPC <- map_df(.x = Out_Files, ~cbind(Subbasin = as.character(str_extract(.x, "[^/]*.out$") %>% str_remove(., ".out")), fread(.x, stringsAsFactors = FALSE)))   %>%                         ## Reads in all of the .OUT files selected for calibration, and uses grep to cbind the subbasin filename 
  setNames(., c("Segment_ID", "Segment_ID_Internal", "Year", "Month", "Day", "Hour", "Min", "Sec", Raw_Header$Pcode))  %>%                                                                      ## Sets column names
  select(-Segment_ID_Internal)                                                                                                                                                                  ## Drops the internal subbasin ID


#********************************************************************************************************************
#               Extracts the flow units for the calibration data
#           Used to convert LSPC flows into same flow units as the calibration data
#********************************************************************************************************************


Flow_Units <- str_to_upper(PCode_Lookup$Units[grep("SURFACE.OUTFLOW|INTERFLOW.OUTFLOW|GROUNDWATER.OUTFLOW|TOTAL.OUTFLOW|TOTAL.RATE.OF.OUTFLOW", str_to_upper(PCode_Lookup$Anal_Name))]) %>%                     ## Extracts flow units based on Analyte Names
  unique(.) %>%
  .[1]

if(is.na(Flow_Units)) { Flow_Units <- "CFS"}                                                                            ## If no flow unit is provided, defaults to CFS


#********************************************************************************************************************
#                               Creates Custom Function to convert flow and loads/concentrations
#                               
#          Back-calculates nutrient concentrations for surface runoff using PERO and loads
#                       
#                            CFS to CMS conversion factor = 0.028316847
#                       
#                     acre-in /day to CFS conversion factor = 0.0420138889
#                     
#                     acre-in/day to CMS conversion factor =  0.00118970086354167 
#                     
#                                lbs/day to kilogram/day = 0.4535924
#                       
#                                    1 day = 86400 seconds
#                                    
#                                    
#       ************NOTE: Precipitation, Shear Stress, Depth are not converted.  They are reported as-is from .OUT file********************
#                   
#********************************************************************************************************************


if(Flow_Units == "CMS") {                                                                                                                  ## Based on FLOW units, sets flow conversion factor
  Flow_Conv_Fact <- 0.028316847
} else {
  Flow_Conv_Fact <- 1
}


CONVERT_LOAD <- function(DF, PCODE) {                                                                                                                            ## Creates custom function for converting PERO loads to concentration
  temp <- mutate(DF, !!PCODE := case_when(!!PCODE %in% c("POQUAL1", "POQUAL2", "POQUAL3") ~ (!!sym(PCODE) * 0.4535924) / (PERO * 0.00118970086354167 * 86.4),
                                          TRUE ~ as.numeric(NA)))
  return(temp)
}


CONVERT_FLOW <- function(DF, PCODE) {                                                                                                                             ## Creates custom function for converting PERO and RO flows to selected flow units
  temp <- mutate(DF, !!PCODE := case_when(!!PCODE %in% c("PERO", "SURO", "IFWO", "AGWO") ~ !!sym(PCODE) * 0.0420138889 * Flow_Conv_Fact,
                                  !!PCODE %in% c("RO") ~ !!sym(PCODE) * Flow_Conv_Fact,
                                  TRUE ~ as.numeric(NA)), )
  return(temp)
}


#********************************************************************************************************************
#               FOR Loop to Apply Unit Conversions
#********************************************************************************************************************


for (i in c("POQUAL1", "POQUAL2", "POQUAL3")) {                                                                               ## Calls load conversion function to convert PERO loads to concentration
  Raw_LSPC <- Raw_LSPC %>%
    CONVERT_LOAD(., i)
}


for (i in c("PERO", "SURO", "IFWO", "AGWO", "RO")) {                                                                          ## Calls flow converstion function to convert PERO and RO flows to selected units
  Raw_LSPC <- Raw_LSPC %>%
    CONVERT_FLOW(., i)
  
}


#********************************************************************************************************************
#               Extracts and reformats data
#********************************************************************************************************************


LSPC_Extract <- Raw_LSPC %>% 
  mutate(Date_Time = make_datetime(year = Year, month = Month, day = Day, hour = Hour, min = Min, sec = 0, tz = Model_TZ)) %>%        ## Creates Date_Time w/ TZ set
  mutate(Station_ID = Stat_Lookup$Segment_Name[match(Segment_ID, Stat_Lookup$Segment_ID)]) %>%                                        ## Adds Segment_Name based on Station Lookup
  select(Date_Time, Station_ID, PCode_Lookup$LSPC_Code) %>%                                                                           ## Selects columns
  setnames(., old = PCode_Lookup$LSPC_Code, new = PCode_Lookup$Anal_Name) %>%                                                         ## Sets column names
  mutate(Date_Time = as.character(format(Date_Time, "%m/%d/%Y %H:%M:%S")))                                                            ## Converts date to a character to match FORTRAN extractor
  #mutate(Date_Time = as.character(Date_Time, format = "%m/%d/%Y %H:%M:%S"))                                                          ## Converts date to a character to match FORTRAN extractor

#********************************************************************************************************************
#                 Write out a formatted .txt file that mimics the format of the FORTRAN WASP BMD2 extractor 
#********************************************************************************************************************

fwrite(LSPC_Extract, file = paste(Path_Model_In, Model_Extracted_Filename, sep = "/"), sep = "\t", na = "",                                                   ## Saves paired Calib_WASP DF
       col.names = TRUE, row.names = FALSE, dateTimeAs = "write.csv")

rm(Raw_LSPC)

