
#********************************************************************************************************************
#
#             Subscript for using R4 LOADEST-WASP utility to evaluate simulated vs. observed loads
#                    
#                    
#             Scripts currently designed only to process and compare output from WASP .BMD2 files
#                       
#                       
#                       FLOW is assumed to be in CMS and concentrations in mg/L
#                       
#                  Requires FLOW, and TN and/or TP be to be simulated and extracted
#                  
#                  
#               To be compared, a station must have flow records and include a USGS flow gage and ID
#               
#               The LOADEST-WASP.exe requires a single station ID based on USGSXXXXXXXXX naming scheme
#               
#               
#         Scripts will also generate time series plots overlaying WASP loads vs. Measured Loads
#         
#         NOTE: Time series plots are based on simply LOAD = FLOW x CONC x 86.4, which is different from the LOADEST estimates
#                        that use a best-fitting linear regression or polynomial equation.
#                        
#                        Time series figures will differ from LOADEST estimates
#                  
#
#********************************************************************************************************************

#********************************************************************************************************************
#                             Set Filepath for Location of LOADEST-WASP.exe
#                                     Trailing slash is excluded
#                                      
#********************************************************************************************************************

Filepath_LOADEST <- "C:/LOADEST-WASP"                                                                                                  ## Sets filepath to the location of the .exe

#********************************************************************************************************************
#                                   Create LOADEST subfolder if missing
#                                      
#********************************************************************************************************************

if(dir.exists(paste0(str_remove(Path_Data_Out, "/Proc_Out"), "/LOADEST")) != TRUE) {                                              ## Verifies if subfolder already exists
  dir.create(paste0(str_remove(Path_Data_Out, "/Proc_Out"), "/LOADEST"))                                                          ## When subfolder is missing, creates the subfolder
  print(paste(paste0(str_remove(Path_Data_Out, "/Proc_Out"), "/LOADEST"), "was missing and was created by the R Scripts"))        ## Prints message when subfolder was created
}

#********************************************************************************************************************
#                                   Create Plot Subfolder in LOADEST subfolder if missing
#                                      
#********************************************************************************************************************

if(dir.exists(paste0(str_remove(Path_Data_Out, "/Proc_Out"), "/LOADEST/Plots")) != TRUE) {                                              ## Verifies if subfolder already exists
  dir.create(paste0(str_remove(Path_Data_Out, "/Proc_Out"), "/LOADEST/Plots"))                                                          ## When subfolder is missing, creates the subfolder
  print(paste(paste0(str_remove(Path_Data_Out, "/Proc_Out"), "/LOADEST/Plots"), "was missing and was created by the R Scripts"))        ## Prints message when subfolder was created
}

#********************************************************************************************************************
#             Start of IF statement to extract FLOW, TN, and TP time series data and reformat for LOADEST-WASP.exe
#                                      
#********************************************************************************************************************

#********************************************************************************************************************
#             Verifies that WASP data are being compared and that flow, and either TN/TP were simulated
#             LOADEST-WASP scripts currently only designed to run using WASP Output and requires either TN/TP
#             
#             At a given location, LOADEST-WASP.exe requires at least 12 measured records for a given analyte
#                                      
#********************************************************************************************************************

if(Model_Type != "WASP") {
  
  stop("LOADEST analysis designed only to compare WASP simulated values; Model Type must be set to WASP with FLOW in CMS")                                             ## Error: Verifies that WASP model type was selected for processing
  
} else {
  
  if(!length(c(grep("Flow.Out.of.Segment", PCode_Lookup$Anal_Name), grep("Total.Nitrogen|Total.Phosphorus", PCode_Lookup$Anal_Name))) >= 2) {
    
    stop("LOADEST analysis requires simulated flows, as well as Total Nitrogen and/or Total Phosphorus; Verify that Flow were processed, as well as TN and/or TP")    ## Error: Verifies that FLOW is included in the processed data, as well as either TN and/or TP
    
  } else {
    
    Nutrient_Load_Vec <- PCode_Lookup$PCode[which(PCode_Lookup$Anal_Name %in% c("Total.Nitrogen", "Total.Phosphorus"))]                                               ## Extracts the custom pcodes used in the Pcode lookup for TN/TP
    
    print(paste("LOADEST analysis will be applied to", length(Nutrient_Load_Vec), "analyte(s):", paste(Nutrient_Load_Vec, collapse = ", ")))                          ## Prints message of which analytes are to be compared
    
    #********************************************************************************************************************
    #                          Extract the individual stations to allow me to extract the USGS gage ID
    #               
    #               This will automatically extract only those stations with FLOW records and nutrient analytes
    #               
    #           When multiple Station_IDs are used in the same model segment, it will split the merged station ID into multiple IDs and
    #                           then assign a standard Station ID based on the USGS gage number
    #                           
    #                       
    #               The .exe requires the STATION identifier to be in the format of USGSXXXXXXXX, with no spaces or dashes
    #                                      
    #********************************************************************************************************************
    
    Numb_Stations <- Station_Order_Lookup %>%                                                                             ## Finds the maximum number of merged stations that comprise a Station_ID_Mrgd element                                                                                   
      mutate(Station_Count = str_count(Station_ID_Mrgd, "_") + 1) %>%                                                     ## Counts the number of underscore, which is the deliminator for merged stations (Adds 1 to account for additional station)
      summarize(Count = max(Station_Count))                                                                               ## Finds maximum number of individual stations across all merged stations
    
    Station_Load_Vec <- paste("Station", seq(1, Numb_Stations$Count, by = 1), sep = "_")                                  ## Creates vector of Station IDs that will be used as column names in the next line of code
    
    #********************************************************************************************************************
    #                     Split merged station IDs into their individual stations
    #                     
    #********************************************************************************************************************
   
     Station_Load_Lookup <- NestData_Multi %>%
      dplyr::filter(PCode == PCode_Lookup$PCode[which(PCode_Lookup$Anal_Name == "Flow.Out.of.Segment")]) %>%              ## Extracts only those stations that had flow records
      select(Meas_Filt) %>%                                                                                               ## Select and unnests measured DF
      unnest(Meas_Filt) %>% 
      distinct(Station_ID_Mrgd, Segment_ID) %>%                                                                           ## Extract unique combinations of Segment IDs and Station IDs
      separate(Station_ID_Mrgd, into = Station_Load_Vec, remove = FALSE, sep = "_") %>%                                   ## Split merged Station IDs into their individual elements based on underscore
      gather(key = "key", value = "Station", -c(Station_ID_Mrgd, Segment_ID)) %>%                                         ## Gathers stations into long format 
      dplyr::filter(grepl("^USGS", Station)) %>%                                                                          ## Keeps only those station IDs that start with USGS
      mutate(Station = str_remove_all(Station, "-")) %>%                                                                  ## Removes dashes to fit format required by .exe
      select(-key)                                                                                                        ## Drops original key 
    
    #********************************************************************************************************************
    #                     Preprocesses the Observed DF that is extracted from the Nested DF
    #                     
    #********************************************************************************************************************
    
    Obs_DF <- NestData_Multi %>%                                                                                                        ## Selects and unnests measured DF from nested DF
      select(Meas_Filt) %>%
      unnest(Meas_Filt) %>%
      dplyr::mutate(PCode = case_when(PCode == PCode_Lookup$PCode[match("Flow.Out.of.Segment", PCode_Lookup$Anal_Name)] ~ "FLOW",       ## Applies Case_when to convert custom PCodes to standard Pcodes for easier reference below
                                      PCode == PCode_Lookup$PCode[match("Total.Nitrogen", PCode_Lookup$Anal_Name)] ~ "TN",
                                      PCode == PCode_Lookup$PCode[match("Total.Phosphorus", PCode_Lookup$Anal_Name)] ~ "TP",
                                      TRUE ~ NA_character_)) %>%
      dplyr::filter(PCode %in% c("FLOW", "TN", "TP")) %>%                                                                               ## Keeps only FLOW, TN, TP records
      select(Segment_ID, Date_Time, PCode, Result_Obs) %>%                                                                              ## Selects columns
      group_by(Segment_ID, PCode, Year = year(Date_Time), Month = month(Date_Time), Day = day(Date_Time)) %>%                           ## Groups DF by Segment_ID, pcode, and date
      summarize(Result_Avg = mean(Result_Obs)) %>%                                                                                      ## Calculates daily average 
      ungroup(.) %>%                                                                                                                    ## Ungroups DF
      mutate(Date_Time = paste(Month, Day, Year, sep = "/")) %>%                                                                        ## Converts year, month, day columns into a character string.  NOTE: This is not a date element
      select(Segment_ID, PCode, Date_Time, Result_Avg) %>%                                                                              ## Selects columns
      spread(key = PCode, value = Result_Avg) %>%                                                                                       ## Casts DF wide
      {if(!"Total.Nitrogen" %in% unique(PCode_Columns)) dplyr::mutate(., TN = as.numeric(NA)) else (dplyr::mutate(., TN = TN))} %>%     ## If TN is missing from DF, creates a blank column 
      {if(!"Total.Phosphorus" %in% unique(PCode_Columns)) dplyr::mutate(., TP = as.numeric(NA)) else (dplyr::mutate(., TP = TP))} %>%   ## IF TP is missing from DF, creates a blank column
      mutate(STATION = Station_Load_Lookup$Station[match(Segment_ID, Station_Load_Lookup$Segment_ID)]) %>%                              ## Remaps the WASP Segment ID to the USGS gage number from above
      mutate(TN_KGD = case_when(!is.na(TN) ~ FLOW * TN * 86.4,                                                                          ## Calculates load in KGD, assumes FLOW in CMS and CONC in mg/L
                                TRUE ~ as.numeric(NA)),
             TP_KGD = case_when(!is.na(TP) ~ FLOW * TP * 86.4,
                                TRUE ~ as.numeric(NA))) %>%
      rename(DATETIME = Date_Time) %>%                                                                                                  ## Renames Date_Time column
      {. ->> Obs_Load} %>%                                                                                                              ## Creates DF of observed loads
      drop_na(FLOW) %>%                                                                                                                 ## Drops any record that did not report flow
      dplyr::filter(FLOW > 0) %>%                                                                                                       ## Keeps only records with positive flow because zero flow will crash LOADEST
      select(STATION, DATETIME, FLOW, TN, TP)                                                                                           ## Selects columns
    
    #********************************************************************************************************************
    #                     Preprocesses the Simulated DF that is extracted from the Nested DF
    #                                      
    #********************************************************************************************************************
    
    Sim_DF <- NestData_Multi %>%                                                                                                        ## Selects and unnests simulated DF from nested data
      select(Model_Filt) %>%
      unnest(Model_Filt) %>%
      dplyr::mutate(PCode = case_when(PCode == PCode_Lookup$PCode[match("Flow.Out.of.Segment", PCode_Lookup$Anal_Name)] ~ "FLOW",       ## Applies Case_when to convert custom PCodes to standard Pcodes for easier reference below 
                                      PCode == PCode_Lookup$PCode[match("Total.Nitrogen", PCode_Lookup$Anal_Name)] ~ "TN",
                                      PCode == PCode_Lookup$PCode[match("Total.Phosphorus", PCode_Lookup$Anal_Name)] ~ "TP",
                                      TRUE ~ NA_character_)) %>%
      dplyr::filter(PCode %in% c("FLOW", "TN", "TP")) %>%                                                                               ## Keeps only FLOW, TN, TP records
      select(Segment_ID, Date_Time, PCode, Result_Sim_Full) %>%                                                                         ## Selects columns
      group_by(Segment_ID, PCode, Year = year(Date_Time), Month = month(Date_Time), Day = day(Date_Time)) %>%                           ## Groups DF by Segment_ID, pcode, and date
      summarize(Result_Avg = mean(Result_Sim_Full)) %>%                                                                                 ## Calculates daily average 
      ungroup(.) %>%                                                                                                                    ## Ungroups DF
      mutate(Date_Time = paste(Month, Day, Year, sep = "/")) %>%                                                                        ## Converts year, month, day columns into a character string.  NOTE: This is not a date element
      select(Segment_ID, PCode, Date_Time, Result_Avg) %>%                                                                              ## Selects columns
      spread(key = PCode, value = Result_Avg) %>%                                                                                       ## Casts DF wide
      {if(!"Total.Nitrogen" %in% unique(PCode_Columns)) dplyr::mutate(., TN = as.numeric(NA)) else (dplyr::mutate(., TN = TN))} %>%     ## If TN is missing from DF, creates a blank column 
      {if(!"Total.Phosphorus" %in% unique(PCode_Columns)) dplyr::mutate(., TP = as.numeric(NA)) else (dplyr::mutate(., TP = TP))} %>%   ## IF TP is missing from DF, creates a blank column
      mutate(STATION = Station_Load_Lookup$Station[match(Segment_ID, Station_Load_Lookup$Segment_ID)]) %>%                              ## Remaps the WASP Segment ID to the USGS gage number from above
      mutate(TN_KGD = case_when(!is.na(TN) ~ FLOW * TN * 86.4,                                                                          ## Calculates load in KGD, assumes FLOW in CMS and CONC in mg/L
                                TRUE ~ as.numeric(NA)),
             TP_KGD = case_when(!is.na(TP) ~ FLOW * TP * 86.4,
                                TRUE ~ as.numeric(NA))) %>%
      rename(DATETIME = Date_Time) %>%                                                                                                  ## Renames Date_Time column
      {. ->> Sim_Load} %>%                                                                                                              ## Creates DF of simulated loads
      drop_na(FLOW) %>%                                                                                                                 ## Drops any record that did not report flow
      dplyr::filter(FLOW > 0) %>%                                                                                                       ## Keeps only records with positive flow because zero flow will crash LOADEST                                                                                                                 ## Drops any record that did not report flow
      select(STATION, DATETIME, FLOW, TN, TP)                                                                                           ## Selects columns 
    
    #********************************************************************************************************************
    #                     FOR loop to export .csv for each analyte
    #               Separate .csv file created for Observed and Simulated data, as well as TN & TP
    #           These files are then read into the LOADEST-WASP.exe for analysis
    #           
    #                LOADEST-WASP requires at least 12 records to run
    #                                      
    #********************************************************************************************************************
    
    for (i in c("TN", "TP")) {
      
      col_vars <- c("STATION", "DATETIME", "FLOW", i)                                                                                   ## Set vector of columns to select
      
      na_vars <- c(i)                                                                                                                   ## Set vector of columns that are used for drop_na
      
      Obs_temp <- Obs_DF %>%                                                                                                            ## Selects columns
        select(all_of(col_vars)) %>%                                                                                                    ## Selects columns
        drop_na(all_of(na_vars)) %>%                                                                                                    ## Drops records with missing concentrations
        group_by(STATION) %>%                                                                                                           ## Groups DF by STATION
        dplyr::filter(n() >= 12)                                                                                                        ## Excludes any STATION that doesn't have at least 12 records
      
      Sim_temp <- Sim_DF %>% 
        select(all_of(col_vars)) %>%                                                                                                    ## Selects columns
        drop_na(all_of(na_vars)) %>%                                                                                                    ## Drops records with missing concentrations
        dplyr::filter(STATION %in% unique(Obs_temp$STATION))                                                                            ## Excludes any STATION that didn't have 12 measured records
      
      if(nrow(Obs_temp) > 1) {
        
        print(paste(i, "observed and simulated data were exported as .csv files for LOADEST analysis and included the following station(s): ", paste(unique(Obs_temp$STATION), collapse = ", ")))                                          ## Prints statement of which analytes and gages had data
       
        write.table(Obs_temp %>% select(all_of(col_vars)) %>% drop_na(all_of(na_vars)), paste0(str_remove(Path_Data_Out, "/Proc_Out"), "/LOADEST/OBS_", Model_Run_Code, "_", i, ".csv"), sep = ",", row.names = FALSE, quote = FALSE)     ## Exports TN OBS
        
        write.table(Sim_temp %>% select(all_of(col_vars)) %>% drop_na(all_of(na_vars)), paste0(str_remove(Path_Data_Out, "/Proc_Out"), "/LOADEST/SIM_",  Model_Run_Code, "_", i, ".csv"), sep = ",", row.names = FALSE, quote = FALSE)     ## Exports TP OBS
        
      }
    }
    
    #********************************************************************************************************************
    #                               Uses SHELL to call the LOADEST-WASP.exe
    #             This will open the LOADEST-WASP dialogue box to allow the user to point toward the OBS.csv and SIM.csv files
    #             
    #                      TN & TP are saved as separate .csv files to avoid data gaps
    #                      
    #********************************************************************************************************************
    
    shell(paste(Filepath_LOADEST, "LOADEST-WASP.exe", sep = "/"), wait = FALSE)                                                                                                                                                                           ## Executes .EXE which will then open dialogue box
    
  }
}

#********************************************************************************************************************
#                              Extracts vector of Segment IDs to be LOADEST plotted
#********************************************************************************************************************

Load_Segments <- Obs_Load %>%
  select(Segment_ID, TN_KGD, TP_KGD) %>%                                ## Selects only Load columns
  gather(key = "Pcode", value = "Result", -Segment_ID) %>%              ## Casts DF long
  drop_na(Result) %>%                                                   ## Drops missing values
  .["Segment_ID"] %>%                                                   ## Extracts vector of segments with LOADEST values
  unique(.) 

#********************************************************************************************************************
#                               Custom function to generate time series plots
#                               
#                     Overlays measured Load vs simulated Load in KGD
#                     
#           A separate panel will be generated for FLOW, TN Concentration, TP Concentration, TN Load, and TP Load
#             
#             Time series plots can help assess when WASP simulated loads are overestimated vs. LOADEST loads
#             
#                      NOTE: These load values are based on LOAD = FLOW x CONC x 86.4
#   The LOADEST estimated values will use more complex regression equations to estimate LOAD as FLOW changes
#                 Values in these figures may not match those estimated by LOADEST
#********************************************************************************************************************

PLOT_LOAD <- function(MEAS_DF, MODEL_DF, SEGMENT) {                                
  
FootNote <- paste("Simulated values: Solid line; Measured values: Blue dots", "\n Date Plotted: ", Sys.Date(), sep = "")                      ## Creates footnote legend
  
Obs_Plot <- MEAS_DF %>%
  dplyr::filter(Segment_ID == SEGMENT) %>%                                                                                                    ## Filters Segment ID                                                                                
  gather(key = "PCode", value = "Result", -DATETIME, -STATION, -Segment_ID) %>%                                                               ## Casts DF long
  mutate(DATETIME = mdy(DATETIME)) %>%                                                                                                        ## Converts dates
  mutate(PCode = case_when(PCode == "FLOW" ~ "FLOW_CMS",                                                                                      ## Remaps PCodes to include units
                           PCode == "TP" ~ "TP_MGL",
                           PCode == "TN" ~ "TN_MGL",
                           PCode == "TN_KGD" ~ "TN_KGD",
                           PCode == "TP_KGD" ~ "TP_KGD",
                           TRUE ~ NA_character_))

Sim_Plot <- MODEL_DF %>%     
  dplyr::filter(Segment_ID == SEGMENT) %>%                                                                                                    ## Filters Segment ID  
  gather(key = "PCode", value = "Result", -DATETIME, -STATION, -Segment_ID) %>%                                                               ## Casts DF long
  mutate(DATETIME = mdy(DATETIME)) %>%                                                                                                        ## Converts dates
  mutate(PCode = case_when(PCode == "FLOW" ~ "FLOW_CMS",                                                                                      ## Remaps PCodes to include units
                           PCode == "TP" ~ "TP_MGL",
                           PCode == "TN" ~ "TN_MGL",
                           PCode == "TN_KGD" ~ "TN_KGD",
                           PCode == "TP_KGD" ~ "TP_KGD",
                           TRUE ~ NA_character_))

ggplot() +                                                                                                                                    ## Calls up ggplot
  geom_line(data = Sim_Plot, aes(x = DATETIME, y = Result), linetype = 1, color = "black",  linewidth = 0.5) +                                     ## Creates time series of simulated values
  geom_point(data = Obs_Plot, aes(x = DATETIME, y = Result), color = "dodgerblue", size = 1.75, alpha = 0.8 ) +                               ## Adds the measured LOADEST observations to the time series
  theme_bw() +                                                                                                             
  scale_x_date(date_breaks = "2 years", date_labels = "%Y") +                                                                                 ## Sets the breakpoints on the x-axis
  labs(title = paste0("Station: ", Obs_Plot$STATION, "\nSegment ID: ", Obs_Plot$Segment_ID),                                                  ## Adds labels
       y = "Result",
       x = "Date",
       caption = FootNote) +
  theme(
    text = element_text(size = 8),                                                                                                            ## Resizes the text
    plot.title= element_text(size = 12, hjust = 0.5, color = 'blue'),                                                                         ## Justifies and changes font size of title
    plot.caption = element_text(hjust = 0.5, size = 8),
    panel.background = element_rect(fill = "white", color = "black"),                                                                         ## Change the panel background and creates a black border
    legend.position = "bottom",
    plot.margin = margin(unit(c(1, 1, 1, 5), "cm")),
    legend.margin = margin(unit(c(-10, 0, -1, 0), "cm")))  +                                                                                  ## Adjusts margins to avoid overlap
  facet_wrap(vars(PCode), nrow = 5, ncol = 1, scales = "free_y")                                                                              ## Creates multipanel plot

 ggsave(paste0(str_remove(Path_Data_Out, "/Proc_Out"), "/LOADEST/Plots/LOAD_", unique(Obs_Plot$Segment_ID), "_", unique(Obs_Plot$STATION), ".jpg"), dpi = 300, width = 10, height = 8, units = "in")             ## Saves figure
}


#********************************************************************************************************************
#                            Calls Plot_Load function to generate static time series plots
#********************************************************************************************************************

walk(.x = Load_Segments$Segment_ID, ~PLOT_LOAD(Obs_Load, Sim_Load, .x))


