
#********************************************************************************************************************
#
#                      Master Script to QA Input Data, Process and Interpolate Data, and Generate Plots
#
#                  Input datafile based on raw WASP .BMD2 or LSPC .OUT output and calibration data
#                              
#                         Includes IF statements to allow import of either LSPC or WASP data, exporting figures as HTML
#                         
#             *************NOTE: When run in .BAT mode, PANDOC.exe is manually set due to a bug in saveWidget()*************
#                         Requires PANDOC.exe to be installed with RStudio or separately, and a filepath provided in .BAT file
#                           
#                              Can be Run Interactively or Using .BAT File in Command Line
#                              
#
#                   This script calls SOURCE scripts using map and custom functions without FOR LOOPS
#                              
#
#                               Uses calibration lookup table v7.1 to set script arguments
#                                 
#                                 Developed by John Davis; EPA Region 4 Modeling Group
#                                                      
#********************************************************************************************************************


#********************************************************************************************************************
#                                          ARGS statement for running scripts in .BAT mode
#                           .BAT file requires only a single argument (working directory of calibration lookup table)
#********************************************************************************************************************

args <- commandArgs(trailingOnly = TRUE)                                                                                                ## When run in .BAT, tells R to extract arguments from command line

#********************************************************************************************************************
#                                    Load Required Libraries for R when run in .BAT mode
#********************************************************************************************************************

if (length(args) != 0) {                                                                      ## IF statement to only load packages when running in .BAT mode
  
  library(rlang)               # for quosure extraction
  library(pastecs)             # for summary stats
  library(car)                 # for Levene Test
  library(gridExtra)           # for plot layouts
  library(data.table)          # for fread data import
  library(Rmisc)               # for geomean confidence interval
  library(lubridate)           # for working with dates
  library(tidyverse)           # for data reformatting
  library(ggplot2)             # for creating graphs
  library(ggforce)             # for facet wrap paginate
  library(scales)              # for access breaks/formatting
  library(hydroGOF)            # for timeseries Goodness of Fit stats
  library(gtable)              # for conversion of DF into figure object
  library(ggpubr)              # for multipanel figures
  library(magrittr)            # for piping functions
  library(openxlsx)            # for writing Excel files
  library(ggmap)               # for mapping functions
  library(sf)                  # for importing shapefiles
  library(ggspatial)           # for mapping functions
  library(viridis)             # for color palette
  library(RColorBrewer)        # for color palette
  library(plotly)	             # for interactive plots
  library(rmarkdown)           # for HTML conversion
  library(leaflet)             # for interactive HTML plots
  library(leafpop)             # for interactive HTML plots
  library(htmlwidgets)         # for saving HTML widgets
  library(withr)               # for temporary env changes
  
}

#********************************************************************************************************************
#                                       Global Values to be Applied to All Scripts
#                             
#                             For interactive mode, set based on Controller tab in Excel Lookup table
#                             
#               For .BAT mode, set working directory based on .BAT arguments and other arguments based on Excel Lookup table
#               
#                     This updated version has changed how filepaths are created and entered into the lookup table.
#                    The filepaths are linked to the location of the FILES and NOT to the location of the SUBFOLDER
#         
#********************************************************************************************************************

#********************************************************************************************************************
#                                                  Interactive Mode
#                   
#                   IF Statement to Set Work Directory, Plot Type, and Geomean Month Filter based on Lookup Table
#                                                  
#********************************************************************************************************************

if(length(args) == 0){                                                                                                                                                    ## Sets when no ARGS passed from .BAT
  if(length(setdiff(c("Calib_Lookup.xlsx"),   list.files(path = "./Lookup/", pattern = "Lookup.xlsx$"))) != 0) {                                                          ## Validates that Lookup tables properly named       
    stop(paste("Lookup table not correctly named or in wrong filepath.  Filepath = ", getwd(), "/Lookup/",                                                                ## Error / ends script
               "\n Lookup Filename = Calib_Lookup.xlsx", sep= ""))                                                           
  } else {
    if(length(setdiff(c("Controller"), getSheetNames(file = "./Lookup/Calib_Lookup.xlsx"))) != 0) {                                                                       ## Validates that lookup table has the Controller tab
      stop("Calib Lookup Excel spreadsheet located, but the Controller tab is missing or incorrectly named")                                                              ## Error / ends script
    } else {
      #********************************************************************************************************************
      #                                     Import Controller
      #                                     
      #********************************************************************************************************************
      
      Controls <- openxlsx::read.xlsx(xlsxFile = "./Lookup/Calib_Lookup.xlsx", sheet = "Controller", colNames = TRUE, rows = c(1:9))                                     ## Imports controller tab, but skips rows beyond Row 8
      
      #********************************************************************************************************************
      #                                     Import Model Type Switch
      #                                     
      #********************************************************************************************************************
      
      Model <- openxlsx::read.xlsx(xlsxFile = "./Lookup/Calib_Lookup.xlsx", sheet = "Controller", colNames = TRUE, rows = c(11,12), na.strings = c("", " ", NA))        ## Imports controller tab, and sets Model_Type argument; If no argument provided sets to NA
      
      if(is.null(Model)) {
        Model <- data.frame(Model_Type = "Model_Type", Selected_Model = "WASP", stringsAsFactors = FALSE)                                                            ## If Model Type field is missing, defaults to WASP model
      }
      
      Model_Type <- Model$Selected_Model %>%                                                                                                                       ## Sets Model_Type argument
        str_trim(., side = "both") %>%                                                                                                                             ## Strips white space
        str_to_upper(.)                                                                                                                                            ## Casts to upper case
      
      if(is.na(Model_Type)) {                                                                                                                                      ## If no model is selected, assumes data PREPROCESSED
        Model_Type <- "PREPROCESSED"
      }
      
      if(!Model_Type %in% c("WASP", "LSPC", "EFDC", "PREPROCESSED", as.character(NA))) { stop("Model Type has to be set to either: WASP, LSPC, EFDC, PREPROCESSED, or left blank") }
      
      print(paste0(Model_Type, " was the selected model type on the Lookup Controller Tab"))                                                                      ## Reports the model type selected to process
      
      #Stat_Lookup <- openxlsx::read.xlsx(xlsxFile = "./Lookup/Calib_Lookup.xlsx", sheet = "Station_Lookup", colNames =  TRUE, cols = c(1:8))                  ## Imports Station Crosswalk, but skips columns beyond Col 8
      #  rename(Segment_ID = WASP_Segment,
      #         Segment_Name = WASP_Name)  
      
      
      #********************************************************************************************************************
      #                                     IF Statement to import either LSPC or WASP Station Lookup
      #                                  Assumes Station_Lookup tab to be used when model data PREPROCESSED
      #                                  
      #                       QA CHECK: First verifies that the required Worksheet tabs are in the Excel lookup file
      #********************************************************************************************************************
      
      if (Model_Type %in% c("EFDC", "WASP", "PREPROCESSED")) {
        
        if(length(setdiff(c("Controller", "Station_Lookup", "PCode_Lookup"), getSheetNames(file = "./Lookup/Calib_Lookup.xlsx"))) != 0) {
          stop(paste("Calib Lookup Excel spreadsheet located, but the following worksheet(s) are missing or incorrectly named: ",                                          ## Verifies that WASP Station and PCode Lookup table is in Excel file
                     paste(setdiff(c("Controller", "Station_Lookup", "PCode_Lookup"), getSheetNames(file = "./Lookup/Calib_Lookup.xlsx")), collapse = "; ")))
        } else {
          Stat_Lookup <- openxlsx::read.xlsx(xlsxFile = "./Lookup/Calib_Lookup.xlsx", sheet = "Station_Lookup", colNames =  TRUE, cols = c(1:8)) %>%                       ## Imports Station Crosswalk, but skips columns beyond Col 8
            rename(Segment_ID = WASP_Segment,                                                                                                                              ## Renames columns to standard names
                   Segment_Name = WASP_Name)
          
          PCode_Lookup <- openxlsx::read.xlsx(xlsxFile = "./Lookup/Calib_Lookup.xlsx", sheet = "PCode_Lookup", colNames = TRUE, cols = c(1:4))                             ## Imports Parameter Crosswalk, but skips columns beyond Col 4
          
        }
      } else {
        
        if(length(setdiff(c("Controller", "LSPC_Station", "LSPC_PCode"), getSheetNames(file = "./Lookup/Calib_Lookup.xlsx"))) != 0) {
          stop(paste("Calib Lookup Excel spreadsheet located, but the following worksheet(s) are missing or incorrectly named: ", 
                     paste(setdiff(c("Controller", "LSPC_Station", "LSPC_PCode"), getSheetNames(file = "./Lookup/Calib_Lookup.xlsx")), collapse = "; ")))
        } else {
          
          Stat_Lookup <- openxlsx::read.xlsx(xlsxFile = "./Lookup/Calib_Lookup.xlsx", sheet = "LSPC_Station", colNames =  TRUE, cols = c(1:8)) %>%                          ## Imports LSPC Station Crosswalk, but skips columns beyond Col 8
            rename(Segment_ID = LSPC_Subbasin,                                                                                                                              ## Renames columns to standard names
                   Segment_Name = LSPC_Name)
          
          PCode_Lookup <- openxlsx::read.xlsx(xlsxFile = "./Lookup/Calib_Lookup.xlsx", sheet = "LSPC_PCode", colNames = TRUE, cols = c(1:4))                                ## Imports LSPC Pcode Crosswalk
        }
      }
      
      if(!is.numeric(as.numeric(Controls$Filter_Argument[Controls$Filter_Settings == "Geomean_First_Month"])) | !is.numeric(as.numeric(Controls$Filter_Argument[Controls$Filter_Settings == "Geomean_Last_Month"]))) {                                                                   ## Validates geomean month filters are numeric 
        stop("On the controller tab, the Geomean_First_Month and Geomean_Last_Month need to be numeric integers between 1 and 12, inclusive")                                                                              ## Error / ends script
      } else {
        
        if(nchar(Controls$Argument[Controls$Calibration_Settings == "Model_Run_Code"]) > 10) {
          print("On the controller tab, the Model Run Code was over 10 characters in length.  Overly long Model Run Codes may cause problems when saving files.")         ## Prints warning when an overly long Model Run Code is used, as this can cause filepath saving issues
        }
        
        # Sets model settings -----------------------------------------------------
        
        Settings <- Controls %>%                                                                                                                                         ## Extracts project settings subtable
          as.data.frame() %>%
          select(Calibration_Settings, Argument) %>%
          drop_na(Calibration_Settings) %>%
          mutate(Argument = case_when(is.na(.$Argument) & Calibration_Settings == "Filepath_R_Scripts" ~ paste0(getwd(), "/Scripts"),                                     ## If no filepath for scripts, uses default directory based on Rproject
                                      is.na(.$Argument) & Calibration_Settings == "Filepath_Calib_Data_In" ~ paste0(getwd(), "/Raw_In"),                                  ## If no Filepath for Raw Data, uses default directory of RProject 
                                      is.na(.$Argument) & Calibration_Settings == "Filepath_ProcData_Out" ~ getwd(),                                                      ## If no Work Directory in controller tab, uses default directory of RProject; No subfolder is defined because multiple folders receive the output
                                      is.na(.$Argument) & Calibration_Settings == "Filepath_Model_Data_In" ~ paste0(getwd(), "/Raw_In"),                                  ## If no BMD2 filepath provided, uses the Raw_In filepath
                                      TRUE ~ Argument))
        
        Model_Run_Code <- Settings$Argument[Settings$Calibration_Settings == "Model_Run_Code"]
        
        Path_Scripts <- gsub("\\\\", "/", Settings$Argument[Settings$Calibration_Settings == "Filepath_R_Scripts"]) %>%                                                   ## Sets scripts filepath and replaces backward slash w/ forward slash  
          gsub("/$", "", .)                                                                                                                                               ## If subfolder drop results in trailing forward slash, drops the trailing slash
        
        Path_Model_In <- gsub("\\\\", "/", Settings$Argument[Settings$Calibration_Settings == "Filepath_Model_Data_In"]) %>%                                                   ## Sets input data filepath and replaces backward slash w/ forward slash
          gsub("/$", "", .) 
        
        Path_Calib_In <- gsub("\\\\", "/", Settings$Argument[Settings$Calibration_Settings == "Filepath_Calib_Data_In"]) %>%                                               ## Sets input data filepath and replaces backward slash w/ forward slash
          gsub("/$", "", .)                                                                                                                                               ## If subfolder drop results in trailing forward slash, drops the trailing slash
        
        Path_Data_Out <- gsub("\\\\", "/", Settings$Argument[Settings$Calibration_Settings == "Filepath_ProcData_Out"])                                                   ## Sets output data filepath and replaces backward slash w/ forward slash
        
        if (Model_Type == "PREPROCESSED") {
          RawBMD_Filename <- NA
        } else {
          RawBMD_Filename <- Settings$Argument[Settings$Calibration_Settings == "Model_BMD2_Filename"]                                                                    ## Sets filename of RAW WASP output
        }
        
        Model_Extracted_Filename <- Settings$Argument[Settings$Calibration_Settings == "Model_Extracted_Filename"]                                                        ## Sets filename of Extracted WASP output
        
        RawCalib_Filename <- Settings$Argument[Settings$Calibration_Settings == "Calib_Data_In_Filename"]                                                                 ## Sets filename of calibration data  
        
        # Sets which plots to generate -----------------------------------------------------
        
        Plot_Types <- Controls$Type[!is.na(Controls$Plot)] %>%                                                                                                            ## Creates vector of plots to be generated
          str_replace(., "Calibration", "Calib") %>%
          str_replace(., "Boxplot", "Box") %>%
          str_replace(., "Export_as_HTML", "HTML") %>%
          str_to_upper(.)
        
        # Sets Statistic to Plot in Boxplot -----------------------------------------------------
        
        Boxplot_Stat <- Controls[6,8]                                                                                                                                  ## Extracts boxplot statistic based on position
        
        if(!str_to_upper(Boxplot_Stat) %in% c("MEDIAN", "MEAN", "AVG", "AVERAGE", as.character(NA))) { stop("Boxplot statistic has to be set to either: MEDIAN, MEAN, AVG, AVERAGE, or left blank; defaults to MEAN if argument is missing")
        } else {
          
          if(is.na(Boxplot_Stat)) {
            Boxplot_Stat <- "MEAN"                                                                                                                                           ## If statistic missing, defaults to MEAN
          }
          
          Boxplot_Stat <- str_to_upper(Boxplot_Stat)
          
          # Sets settings to filter data -----------------------------------------------------
          
          Filter_Args <- Controls %>%                                                                                                                                         ## Extracts project settings subtable
            as.data.frame() %>%
            select(Filter_Settings, Filter_Argument) %>%
            .[1:5, 1:2]
          
          Month_Geom <- seq.int(as.numeric(Filter_Args$Filter_Argument[Filter_Args$Filter_Settings == "Geomean_First_Month"]), 
                                as.numeric(Filter_Args$Filter_Argument[Filter_Args$Filter_Settings == "Geomean_Last_Month"]))                                                 ## Creates vector of months to be used for geomean
          
          Min_Obs <- as.numeric(Filter_Args$Filter_Argument[Filter_Args$Filter_Settings == "Minimum_Observations"])                                                           ## Sets the minimum of observation that a sampling location needs (applied to multipanel and GoF calcs)
          
          Min_Obs_BoxDens <- as.numeric(Filter_Args$Filter_Argument[Filter_Args$Filter_Settings == "Boxplot_Density_Min_Obs"]) %>%                                            ## Sets a second observation cutoff that will be applied to boxplots/density plots
            ifelse(is.na(.), 0, .)                                                                                                                                            ## If this field is left blank, it will use a cutoff that was used for the Multipanel Calib (inclusive)
          
          RCode_Exclusion <- Filter_Args %>%                                                                                                                                  ## Sets the RCodes that will be excluded when running calibration
            dplyr::filter(., Filter_Settings == "Rcode_Flag_Exclusion") %>%
            .[1,2] %>%
            {ifelse(is.na(.), NA, str_split(., ","))} %>%                                                                                                                     ## Splits values into a list, but replaces with NA when no flagged RCode provided
            unlist(.) %>%                                                                                                                                                     ## Converts list into a string
            str_trim(., side = "both") %>%                                                                                                                                    ## Removes errant whitespace
            toupper(.) %>%                                                                                                                                                    ## Casts all to uppercase
            paste(., collapse = "|")                                                                                                                                          ## Collapse string into single character separated by |, allows use in GREP
          
          Model_TZ = Controls$Model_TimeZone[!is.na(Controls$Selected_TZ)]                                                                                                    ## Sets time zone when matching model and calibration data
          
          # Sets Shapefile settings -----------------------------------------------------
          
          Shapefile_Settings <- Controls %>%                                                                                                                                  ## Extracts project settings subtable
            as.data.frame() %>%
            select(Shapefile_Settings, Shapefile_Argument) %>%
            drop_na(Shapefile_Settings) %>%
            mutate(Shapefile_Argument = case_when(is.na(.$Shapefile_Argument) & Shapefile_Settings == "Filepath_Shapefile" ~ paste0(getwd(), "/Shapefile"),                   ## If no filepath for scripts, uses default directory based on Rproject
                                                  TRUE ~ as.character(Shapefile_Argument)))
          
          Path_Shape <- gsub("\\\\", "/", Shapefile_Settings$Shapefile_Argument[Shapefile_Settings$Shapefile_Settings == "Filepath_Shapefile"])   %>%                         ## Sets shapefile filepath and replaces backward slash w/ forward slash  
            gsub("/$", "", .)                                                                                                                                                 ## If subfolder drop results in trailing forward slash, drops the trailing slash
          
          Watershed_Filename <- Shapefile_Settings$Shapefile_Argument[Shapefile_Settings$Shapefile_Settings == "Watershed_Shapefile_Filename"]                                ## Sets filename of RAW WASP output
          Stream_Network_Filename <- Shapefile_Settings$Shapefile_Argument[Shapefile_Settings$Shapefile_Settings == "Stream_Network_Filename"]                                ## Sets filename of RAW WASP output
          
          Shape_CRS <- as.numeric(Shapefile_Settings$Shapefile_Argument[Shapefile_Settings$Shapefile_Settings == "Coord_Ref_Sys_Code"])                                       ## Sets CRS of shapefiles
          
        }
      } 
    } 
  } 
}else {                                                                                                                  
  
  #********************************************************************************************************************
  #                                Rest of IF Statement to Set Work Directory based on .BAT File
  #                                             
  #                                             Same as above, but used during .BAT Mode
  #                                               
  #                             Sets working directory based on the filepath passed by .BAT file
  #********************************************************************************************************************
  
  if (length(args) != 0) {
    if(length(args) > 4) {
      stop(".BAT file requires one argument and additional optional arguments to be passed to master R script: \n ARG1 = Filepath and filename to calibration lookup table (e.g., Calib_Lookup.xlsx) \n ARG2 = Filepath for PANDOC library (needed for HTML plotting) \n ARG3 = Model Run Code (optional) \n ARG4 = Save Image (optional)")
    } else {
      
      print(sessionInfo())                                                                                                                                 ## For .BAT mode, writes out session info
      
      Lookup_Filename <-   gsub("\\\\", "/", args[1]) %>%                                                                                                  ## Uses replaces backward slash with forward slash
        str_extract(., '[^/]*$')                                                                                                                           ## Uses regex to extract the Lookup Filename from end of args[1]
      
      Lookup_Filepath <- gsub("\\\\", "/", args[1]) %>%                                                                                                    ## Uses replaces backward slash with forward slash                                                    
        gsub(Lookup_Filename, "", .)                                                                                                                       ## Uses regex to extract the Lookup FilePath by removing filename
      
      setwd(gsub("/Lookup/", "", Lookup_Filepath))                                                                                                         ## Sets WD to Arg[1] w/o subfolder or filename
      
      Save_Image <- args[4] %>%                                                                                                                            ## If activated in .BAT mode (arg[4]), will save image once scripts are completed
        str_to_upper(.) %>%
        substr(., 1, 1) %>%
        grepl("Y", .)
      
      
      ## Sets WD to Arg[1].  R needs /, so GSUB changes errant \
      
      #setwd(gsub("\\\\", "/", args[1]))                                                                                                                   ## Sets WD to Arg[1].  R needs /, so GSUB changes errant \
      
      
      if(length(setdiff(c(Lookup_Filename),   list.files(path = Lookup_Filepath , pattern = ".xlsx$"))) != 0) {                                             ## Validates that Lookup tables properly named       
        stop(paste("Lookup table not correctly named or in wrong filepath.  Filepath = ", getwd(), "/Lookup/",                                              ## Error / ends script
                   "\n Lookup Filename =", Lookup_Filename, sep= ""))                                                           
      } else {
        if(length(setdiff(c("Controller"), getSheetNames(file = args[1]))) != 0) {                                                                          ## Validates that lookup table has the Controller tab
          stop("Calib Lookup Excel spreadsheet located, but the Controller tab is missing or incorrectly named")                                            ## Error / ends script
        } else {
          
          #********************************************************************************************************************
          #                                             Import Station and Pcode Lookup
          #********************************************************************************************************************
          
          Controls <- openxlsx::read.xlsx(xlsxFile = args[1], sheet = "Controller", colNames = TRUE, rows = c(1:9))                                                          ## Imports controller tab, but skips rows beyond Row 8 
          
          #********************************************************************************************************************
          #                                     Import Model Type Switch
          #                                     
          #********************************************************************************************************************
          
          Model <- openxlsx::read.xlsx(xlsxFile = args[1], sheet = "Controller", colNames = TRUE, rows = c(11:12), na.strings = c("", " ", NA))        ## Imports controller tab, and sets Model_Type argument; If no argument provided sets to NA
          
          if(is.null(Model)) {
            Model <- data.frame(Model_Type = "Model_Type", Selected_Model = "WASP", stringsAsFactors = FALSE)                                          ## If Model Type field is missing, defaults to WASP model
          }
          
          Model_Type <- Model$Selected_Model %>%                                                                                                                       ## Sets Model_Type argument
            str_trim(., side = "both") %>%                                                                                                                             ## Strips white space
            str_to_upper(.)                                                                                                                                            ## Casts to upper case
          
          Model_Type <- Model$Selected_Model %>%                                                                                                        ## Sets Model_Type argument
            str_trim(., side = "both") %>%
            str_to_upper(.) 
          
          if(is.na(Model_Type)) {                                                                                                                                      ## If no model is selected, assumes data PREPROCESSED
            Model_Type <- "PREPROCESSED"
          }
          
          if(!Model_Type %in% c("WASP", "LSPC", "EFDC", "PREPROCESSED", as.character(NA))) { stop("Model Type has to be set to either: WASP, LSPC, EFDC, PREPROCESSED, or left blank") }
          
          print(paste0(Model_Type, " was the selected model type on the Lookup Controller Tab"))                                                                             ## Reports the model type selected to process
          
          # Stat_Lookup <- openxlsx::read.xlsx(xlsxFile = args[1], sheet = "Station_Lookup", colNames =  TRUE, cols = c(1:8))                           ## Imports Station Crosswalk, but skips columns beyond col 8
          
          #********************************************************************************************************************
          #                                     IF Statement to import either LSPC or WASP Station Lookup
          #                                     
          #                                  Assumes Station_Lookup tab to be used when model data PREPROCESSED
          #                                  
          #                           QA Check: Verifies that the Station and Pcode tabs are in the Excel Lookup file
          #********************************************************************************************************************
          
          if (Model_Type %in% c("EFDC", "WASP", "PREPROCESSED")) {
            
            if(length(setdiff(c("Controller", "Station_Lookup", "PCode_Lookup"), getSheetNames(file = args[1]))) != 0) {
              stop(paste("Calib Lookup Excel spreadsheet located, but the following worksheet(s) are missing or incorrectly named: ",                                          ## Verifies that WASP Station and PCode Lookup table is in Excel file
                         paste(setdiff(c("Controller", "Station_Lookup", "PCode_Lookup"), getSheetNames(file = args[1])), collapse = "; ")))
            } else {
              
              Stat_Lookup <- openxlsx::read.xlsx(xlsxFile = args[1], sheet = "Station_Lookup", colNames =  TRUE, cols = c(1:8)) %>%                       ## Imports Station Crosswalk, but skips columns beyond Col 8
                rename(Segment_ID = WASP_Segment,                                                                                                                              ## Renames columns to standard names
                       Segment_Name = WASP_Name)
              
              PCode_Lookup <- openxlsx::read.xlsx(xlsxFile = args[1], sheet = "PCode_Lookup", colNames = TRUE, cols = c(1:4))                             ## Imports Parameter Crosswalk, but skips columns beyond Col 4
              
            }
          } else {
            
            if(length(setdiff(c("Controller", "LSPC_Station", "LSPC_PCode"), getSheetNames(file = args[1]))) != 0) {
              stop(paste("Calib Lookup Excel spreadsheet located, but the following worksheet(s) are missing or incorrectly named: ", 
                         paste(setdiff(c("Controller", "LSPC_Station", "LSPC_PCode"), getSheetNames(file = "./Lookup/Calib_Lookup.xlsx")), collapse = "; ")))
            } else {
              
              Stat_Lookup <- openxlsx::read.xlsx(xlsxFile = args[1], sheet = "LSPC_Station", colNames =  TRUE, cols = c(1:8)) %>%                          ## Imports LSPC Station Crosswalk, but skips columns beyond Col 8
                rename(Segment_ID = LSPC_Subbasin,                                                                                                                              ## Renames columns to standard names
                       Segment_Name = LSPC_Name)
              
              PCode_Lookup <- openxlsx::read.xlsx(xlsxFile = args[1], sheet = "LSPC_PCode", colNames = TRUE, cols = c(1:4))                                ## Imports LSPC Pcode Crosswalk
            }
          }
          
          if(!is.numeric(as.numeric(Controls$Filter_Argument[Controls$Filter_Settings == "Geomean_First_Month"])) | !is.numeric(as.numeric(Controls$Filter_Argument[Controls$Filter_Settings == "Geomean_Last_Month"]))) {                                                                   ## Validates geomean month filters are numeric 
            stop("On the controller tab, the Geomean_First_Month and Geomean_Last_Month need to be numeric integers between 1 and 12, inclusive")                            ## Error / ends script
          } else {
            
            if(nchar(Controls$Argument[Controls$Calibration_Settings == "Model_Run_Code"]) > 10) {
              print("On the controller tab, the Model Run Code was over 10 characters in length.  Overly long Model Run Codes may cause problems when saving files.")         ## Prints warning when an overly long Model Run Code is used, as this can cause filepath saving issues
            }
            
            Settings <- Controls %>%                                                                                                                                       ## Extracts project settings subtable
              as.data.frame() %>%
              select(Calibration_Settings, Argument) %>%
              drop_na(Calibration_Settings) %>%
              mutate(Argument = case_when(is.na(.$Argument) & Calibration_Settings == "Filepath_R_Scripts" ~ paste0(getwd(), "/Scripts"),                                  ## If no filepath for scripts, uses default directory based on Rproject
                                          is.na(.$Argument) & Calibration_Settings == "Filepath_Calib_Data_In" ~ paste0(getwd(), "/Raw_In"),                               ## If no Filepath for Raw Data, uses default directory of RProject 
                                          is.na(.$Argument) & Calibration_Settings == "Filepath_ProcData_Out" ~ getwd(),                              ## If no Work Directory in controller tab, uses default directory of RProject
                                          is.na(.$Argument) & Calibration_Settings == "Filepath_Model_Data_In" ~ paste0(getwd(), "/Raw_In"),                               ## If no BMD2 filepath provided, uses the Raw_In filepath
                                          TRUE ~ Argument))
            
            if(length(args) == 0 | length(args) == 1) {                                                                                                                    ## If no model run code passed by .BAT, sets Model Run Code to Lookup Table 
              Model_Run_Code <- Settings$Argument[Settings$Calibration_Settings == "Model_Run_Code"]
            } else { 
              Model_Run_Code <- args[3]                                                                                                                                    ## If optional run code passed by .BAT, overwrites Lookup Table Code w/ .BAT Arg
            }
            
            Path_Scripts <- gsub("\\\\", "/", Settings$Argument[Settings$Calibration_Settings == "Filepath_R_Scripts"]) %>%                                                ## Sets scripts filepath and replaces backward slash w/ forward slash  
              gsub("/$", "", .)                                                                                                                                            ## If included incorrectly, drops trailing forward slash to standardize filepath
            
            Path_Model_In <- gsub("\\\\", "/", Settings$Argument[Settings$Calibration_Settings == "Filepath_Model_Data_In"]) %>%                                                ## Sets input data filepath and replaces backward slash w/ forward slash
              gsub("/$", "", .) 
            
            Path_Calib_In <- gsub("\\\\", "/", Settings$Argument[Settings$Calibration_Settings == "Filepath_Calib_Data_In"]) %>%                                            ## Sets input data filepath and replaces backward slash w/ forward slash
              gsub("/$", "", .)                                                                                                                                            ## If included incorrectly, drops trailing forward slash to standardize filepath
            
            Path_Data_Out <- gsub("\\\\", "/", Settings$Argument[Settings$Calibration_Settings == "Filepath_ProcData_Out"]) %>%                                            ## Sets output data filepath and replaces backward slash w/ forward slash
              gsub("/$", "", .)                                                                                                                                            ## If included incorrectly, drops trailing forward slash to standardize filepath
            
            RawBMD_Filename <- Settings$Argument[Settings$Calibration_Settings == "Model_BMD2_Filename"]                                                                   ## Sets filename of RAW WASP output
            Model_Extracted_Filename <- Settings$Argument[Settings$Calibration_Settings == "Model_Extracted_Filename"]                                                     ## Sets filename of Extracted WASP output
            RawCalib_Filename <- Settings$Argument[Settings$Calibration_Settings == "Calib_Data_In_Filename"]                                                              ## Sets filename of calibration data  
            
            # Sets which plots to generate -----------------------------------------------------
            
            Plot_Types <- Controls$Type[!is.na(Controls$Plot)] %>%                                                                                                           ## Creates vector of plots to be generated
              str_replace(., "Calibration", "Calib") %>%
              str_replace(., "Boxplot", "Box") %>%
              str_replace(., "Export_as_HTML", "HTML") %>%
              str_to_upper(.)
            
            
            # Sets Statistic to Plot in Boxplot -----------------------------------------------------
            
            Boxplot_Stat <- Controls[6,8]                                                                                                                                  ## Extracts boxplot statistic based on position
            
            if(!str_to_upper(Boxplot_Stat) %in% c("MEDIAN", "MEAN", "AVG", "AVERAGE", as.character(NA))) { stop("Boxplot statistic has to be set to either: MEDIAN, MEAN, AVG, AVERAGE, or left blank; defaults to MEAN if argument is missing")
            } else {
              
              if(is.na(Boxplot_Stat)) {
                Boxplot_Stat <- "MEAN"                                                                                                                                           ## If statistic missing, defaults to MEAN
              }
              
              Boxplot_Stat <- str_to_upper(Boxplot_Stat)
              
              # Sets settings to filter data -----------------------------------------------------
              
              Filter_Args <- Controls %>%                                                                                                                                         ## Extracts project settings subtable
                as.data.frame() %>%
                select(Filter_Settings, Filter_Argument) %>%
                .[1:5, 1:2]
              
              Month_Geom <- seq.int(as.numeric(Filter_Args$Filter_Argument[Filter_Args$Filter_Settings == "Geomean_First_Month"]), 
                                    as.numeric(Filter_Args$Filter_Argument[Filter_Args$Filter_Settings == "Geomean_Last_Month"]))                                                 ## Creates vector of months to be used for geomean
              
              Min_Obs <- as.numeric(Filter_Args$Filter_Argument[Filter_Args$Filter_Settings == "Minimum_Observations"])                                                           ## Sets the minimum of observation that a sampling location needs (applied to multipanel and GoF calcs)
              
              Min_Obs_BoxDens <- as.numeric(Filter_Args$Filter_Argument[Filter_Args$Filter_Settings == "Boxplot_Density_Min_Obs"]) %>%                                            ## Sets a second observation cutoff that will be applied to boxplots/density plots
                ifelse(is.na(.), 0, .)                                                                                                                                            ## If this field is left blank, it will use a cutoff of >= 0 (inclusive)
              
              RCode_Exclusion <- Filter_Args %>%                                                                                                                                  ## Sets the RCodes that will be excluded when running calibration
                dplyr::filter(., Filter_Settings == "Rcode_Flag_Exclusion") %>%
                .[1,2] %>%
                {ifelse(is.na(.), NA, str_split(., ","))} %>%                                                                                                                     ## Splits values into a list, but replaces with NA when no flagged RCode provided
                unlist(.) %>%                                                                                                                                                     ## Converts list into a string
                str_trim(., side = "both") %>%                                                                                                                                    ## Removes errant whitespace
                toupper(.) %>%                                                                                                                                                    ## Casts all to uppercase
                paste(., collapse = "|")                                                                                                                                          ## Collapse string into single character separated by |, allows use in GREP
              
              Model_TZ = Controls$Model_TimeZone[!is.na(Controls$Selected_TZ)]                                                                                                     ## Sets time zone when matching model and calibration data
              
              # Sets Shapefile settings -----------------------------------------------------
              
              Shapefile_Settings <- Controls %>%                                                                                                                                  ## Extracts project settings subtable
                as.data.frame() %>%
                select(Shapefile_Settings, Shapefile_Argument) %>%
                drop_na(Shapefile_Settings) %>%
                mutate(Shapefile_Argument = case_when(is.na(.$Shapefile_Argument) & Shapefile_Settings == "Filepath_Shapefile" ~ paste0(getwd(), "/Shapefile"),                   ## If no filepath for scripts, uses default directory based on Rproject
                                                      TRUE ~ as.character(Shapefile_Argument)))
              
              Path_Shape <- gsub("\\\\", "/", Shapefile_Settings$Shapefile_Argument[Shapefile_Settings$Shapefile_Settings == "Filepath_Shapefile"])   %>%                         ## Sets shapefile filepath and replaces backward slash w/ forward slash  
                gsub("/$", "", .)                                                                                                                                                 ## If subfolder drop results in trailing forward slash, drops the trailing slash
              
              Watershed_Filename <- Shapefile_Settings$Shapefile_Argument[Shapefile_Settings$Shapefile_Settings == "Watershed_Shapefile_Filename"]                                ## Sets filename of RAW WASP output
              Stream_Network_Filename <- Shapefile_Settings$Shapefile_Argument[Shapefile_Settings$Shapefile_Settings == "Stream_Network_Filename"]                                ## Sets filename of RAW WASP output
              
              Shape_CRS <- as.numeric(Shapefile_Settings$Shapefile_Argument[Shapefile_Settings$Shapefile_Settings == "Coord_Ref_Sys_Code"])                                       ## Sets CRS of shapefiles
              
              #********************************************************************************************************************
              #                   When run in .bat mode, conversion to HTML throws a PANDOC error due to bug in saveWidget()
              #               This is a workaround that requires PANDOC.exe to be installed with RStudio or installed separately
              #                       
              #       When Plot_HTML is activated, a filepath to PANDOC.exe needs to be included as the second argument of the .BAT file (args[[2]])
              #                       
              #                       
              #********************************************************************************************************************
              
              
              if("HTML" %in% Plot_Types) {
                
                ## args <- c("TEST", "TEST2")
                
                Sys.setenv(RSTUDIO_PANDOC = gsub("\\\\", "/", args[2]))                                                                                                              ## Sets location of Pandoc.exe based on the second argument of .BAT file
                
                #Sys.setenv(RSTUDIO_PANDOC = Path_Scripts)                                                                                                                           ## Sets location of the Pandoc.exe, which is manually saved in Path_Scripts
                print(rmarkdown:::find_pandoc())
                
                if(is.null(rmarkdown:::find_pandoc()[[2]]) & "HTML" %in% Plot_Types) {                                                                                               ## QA Check that verifies PANDOC.exe properly loaded if PLOT_HTML is selected
                  stop(paste0("HTML plotting selected, but filepath to pandoc library was invalid; Verify filepath included as second argument in .BAT file: ", args[[2]], "\nIf no PANDOC library is available, leave ARG2 blank and deactivate Plot_HTML"))
                }
              }
            } 
          }
        }
      }
    }  
  }
}


#********************************************************************************************************************
#                          Custom function to create required subfolders 
#           This section verifies that the required subfolders exist and creates them when they are missing
#           
#                       Filepath based on the Path_Data_Out argument
#               
#               Creates all of the subfolders, even when they are not being plotted
#********************************************************************************************************************

CREATE_SUBFOLDER <- function(PLOT_TYPE) {
  
  Crosswalk <- data.frame(Type = c("PROC_OUT", "GOF_STATISTICS", "TIME_SERIES", "REGRESSION", "ECDF", "BOX", "DENSITY", "GEOMEAN", "CALIBRATION", "HTML"), 
                          Subfolder = c("/Proc_Out", "/Stats", "/Plot_TS", "/Plot_Reg", "/Plot_ECDF", "/Plot_Box", "/Plot_Dens", "/Plot_Geom", "/Plot_Calib", "/Plot_HTML"))
  
  temp <- dplyr::filter(Crosswalk, Type == PLOT_TYPE)
  
  if(dir.exists(paste0(str_remove(Path_Data_Out, "/Proc_Out"),temp$Subfolder)) != TRUE) {                                               ## Verifies if subfolder already exists
    dir.create(paste0(str_remove(Path_Data_Out, "/Proc_Out"), temp$Subfolder))                                                          ## When subfolder is missing, creates the subfolder
    print(paste(paste0(str_remove(Path_Data_Out, "/Proc_Out"), temp$Subfolder), "was missing and was created by the R Scripts"))}       ## Prints message when subfolder was created
  
}

walk(.x = c("PROC_OUT", "GOF_STATISTICS", "BOX", "DENSITY", "GEOMEAN", "CALIBRATION"), CREATE_SUBFOLDER)                     ## Calls custom function

#********************************************************************************************************************
#                          Custom function to create Year subfolder in Geomean and Boxplot required subfolders 
#           This section verifies that the required subfolders exist and creates them when they are missing
#           
#                       Filepath based on the Path_Data_Out argument
#               
#               Creates all of the subfolders, even when they are not being plotted
#********************************************************************************************************************

CREATE_ANNUAL_SUBFOLDER <- function(PLOT_TYPE) {
  
  Crosswalk <- data.frame(Type = c("BOX", "GEOMEAN"), Subfolder = c("/Plot_Box/Year", "/Plot_Geom/Year"))
  
  temp <- dplyr::filter(Crosswalk, Type == PLOT_TYPE)
  
  if(dir.exists(paste0(str_remove(Path_Data_Out, "/Proc_Out"),temp$Subfolder)) != TRUE) {                                               ## Verifies if subfolder already exists
    dir.create(paste0(str_remove(Path_Data_Out, "/Proc_Out"), temp$Subfolder))                                                          ## When subfolder is missing, creates the subfolder
    print(paste(paste0(str_remove(Path_Data_Out, "/Proc_Out"), temp$Subfolder), "was missing and was created by the R Scripts"))}       ## Prints message when subfolder was created
  
}

walk(.x = c("BOX", "GEOMEAN"), CREATE_ANNUAL_SUBFOLDER)                                                                                 ## Calls custom function

if("HTML" %in% Plot_Types){CREATE_SUBFOLDER("HTML")}                                                                                    ## Only creates HTML subfolder when it is activated

#********************************************************************************************************************
#                     QA that All Necessary Arguments Correctly Imported from Controller Tab
#********************************************************************************************************************

if(length(setdiff(c("PCode_Lookup", "Stat_Lookup", "Model_Run_Code", "Path_Model_In", "Path_Calib_In", "Path_Scripts", "Path_Data_Out", "Plot_Types",                                                ## Verifies all objects were created in controller tab
                    "RawBMD_Filename", "Model_Extracted_Filename", "RawCalib_Filename", "Min_Obs", "RCode_Exclusion",
                    "Model_TZ"), ls())) != 0) {
  stop(paste("Excel lookup table not correctly loaded.  Verify Pcode and Station tabs properly named and that controller tab includes necessary calibration arguments:", 
             paste(setdiff(c("Path_Calib_In", "Path_Scripts", "Path_Model_In", "Path_Data_Out", "Plot_Types", "RawBMD_Filename", "Model_Extracted_Filename", 
                             "RawCalib_Filename", "Min_Obs", "RCode_Exclusion", "Model_TZ"), ls()), collapse = "; "), sep = " "))                               ## Error / ends script if missing controller argument                                                                   
} else {
  
  #********************************************************************************************************************
  #                     QA that All Shapefile Arguments Correctly Imported
  #                     
  #               If filepath is provided for Shapefile location, uses the full filepath without addign Subfolder
  #********************************************************************************************************************
  
  
  if(any(str_detect(Plot_Types, "MAP|HTML"))) {
    
    #if("MAP" %in% Plot_Types) {
    
    if(length(setdiff(c("Path_Shape", "Watershed_Filename", "Stream_Network_Filename", "Shape_CRS"), ls())) != 0) {                                             ## Verifies that all Shapefile arguments created
      stop(paste("Map creation was selected, but the controller tab in the Calibration Lookup table is missing the following arguments:",
                 paste(setdiff(c("Path_Shape", "Watershed_Filename", "Stream_Network_Filename", "Shape_CRS"), ls()), collapse = "; ", sep = " " )))
    } else {
      
      Lat_Long_Chk <- Stat_Lookup %>%                                                                                                                           ## Verifies that Lats/Longs provided for Monitoring Stations
        select(Lat, Long) %>%
        drop_na() %>%
        nrow()
      
      Shapefile_Chk <- Shapefile_Settings %>%                                                                                                                   ## Verifies that shapefile arguments contain info
        drop_na() %>%
        distinct(Shapefile_Settings) %>%
        nrow()
      
      if(Lat_Long_Chk == 0) {
        stop("Map creation selected, but no monitoring station latitudes or longitudes were provided on the controller tab in the Calibration Lookup table")    ## Verifies lat/longs exist
        
      } else {
        
        if(Shapefile_Chk != 4){
          stop("Map creation selected, but missing Shapefile Arguments on the controller tab in the Calibration Lookup table")                                  ## Verifies shapefile arguments include info
        } else {
          
          if(is.na(Controls$Shapefile_Argument[Shapefile_Settings == "Filepath_Shapefile"]) & length(setdiff(c("Shapefile"), grep("Shapefile$", dir(str_remove(Path_Shape, "/Shapefile")), value = TRUE))) != 0) {                               ## Validates that data subfolders properly named    
            stop(paste(paste(setdiff(c("Shapefile"), grep("Shapefile$", dir(Path_Shape), value = TRUE)), collapse = " / "),                                     ## Error / ends script
                       "subfolder(s) missing from shapefile input data directory: ", Path_Shape))                                                                                     
          } else {
            
            if(length(setdiff(c(Watershed_Filename, Stream_Network_Filename),   list.files(path = Path_Shape, pattern = ".shp$"))) != 0) {                      ## Validates that shapefiles properly named       
              stop(paste("Shapefiles not correctly named or in wrong filepath  Filepath = ", Path_Shape,                                                        ## Error / ends script
                         "\n Watershed shapefile name = ", Watershed_Filename, " and Stream Network Shapefile name = ", Stream_Network_Filename, sep= ""))                                                           
            } else {
              #********************************************************************************************************************
              #                           Create basemap for watershed map in MULTIPANEL PLOT
              #********************************************************************************************************************
              
              Wtrshd_Map <- st_read(paste(Path_Shape, Watershed_Filename, sep = "/")) %>%                                           ## Reads in watershed boundary shapefile
                st_transform(., st_crs(as.numeric(Shape_CRS)))                                                                      ## Reprojects watershed to the CRS reported in Lookup table and match monitoring data
              
              Stream_Map <- st_read(paste(Path_Shape, Stream_Network_Filename, sep = "/")) %>%                                      ## Reads in WASP shapefile
                st_transform(., st_crs(as.numeric(Shape_CRS)))                                                                      ## Reprojects WASP to the CRS reported in Lookup table and match monitoring data
              
              Base_Map <- ggplot() +
                geom_sf(data = Wtrshd_Map, size = 0.25, color = "gray38", fill = "skyblue2") +                                      ## Plots watershed
                geom_sf(data = Stream_Map, size = 0.25, color = "black", fill = "black") 
              
            }
          }
        }
      }
    }
  }
  
  #********************************************************************************************************************
  #                                        QA Raw Input Files Correctly Named in Working Directory
  #********************************************************************************************************************
  
  if(is.na(Controls$Argument[Controls$Calibration_Settings == "Filepath_Calib_Data_In"]) & length(setdiff(c("Raw_In"), grep("_In$", dir(str_remove(Path_Calib_In, "/Raw_In")), value = TRUE))) != 0) {                 ## Validates that data subfolders properly named    
    stop(paste(paste(setdiff(c("Raw_In"), grep("_In$", dir(Path_Calib_In), value = TRUE)), collapse = " / "),                    ## Error / ends script
               "subfolder missing from input data directory: ", Path_Calib_In))                                                                                     
  } else {
    
    if(is.na(Controls$Argument[Controls$Calibration_Settings == "Filepath_ProcData_Out"]) & length(setdiff(c("Proc_Out"), grep("_Out$", dir(str_remove(Path_Data_Out, "/Proc_Out")), value = TRUE))) != 0) {         ## Validates that data subfolders properly named    
      stop(paste(paste(setdiff(c("Proc_Out"), grep("_Out$", dir(Path_Data_Out), value = TRUE)), collapse = " / "),              ## Error / ends script
                 "subfolder(s) missing from input data directory: ", Path_Data_Out))                                                                                     
    } else {
      
      if(length(setdiff(RawCalib_Filename,   list.files(path = Path_Calib_In, pattern = ".txt$"))) != 0) {                         ## Validates that input calibration datafiles properly named and in correct filepath
        stop(paste("Monitoring data input files not correctly named or in wrong filepath.  Check filepath = ", Path_Calib_In,      ## Error / ends script
                   "\n Raw Measured Filename = ", RawCalib_Filename, sep= ""))                                                           
      } else {
        
        #********************************************************************************************************************
        #                                        IF BMD2 Filename Provided, SOURCES the Extract_BMD2.EXE script
        #                       This IF statement allows the USER to use a preformatted .txt file, bypassing the .BMD2 extractor .exe
        #                            This allows the user to apply the calibration scripts to output from models other than WASP
        #********************************************************************************************************************
        
        if(is.na(RawBMD_Filename) & Model_Type == "PREPROCESSED") {
          print(paste0("No model type was selected nor .BMD2 filename was provided in the Excel lookup table.  Scripts assumed model data was PREPROCESSED and used the existing .txt file for data interpolation/calibration: ", Model_Extracted_Filename))       ## Writes progress statement when no BMD2 filename was provided
        }    
        
        if(!is.na(RawBMD_Filename) & Model_Type %in% c("WASP", "EFDC")) {                                                                                                                                                                                      ## If a BMD2 filename was provided, the scripts will call the model data extractor script
          
          print(paste0("A .BMD2 filename was provided in the Excel lookup table.  Scripts used the BMD2 file for data interpolation/calibration: ", RawBMD_Filename))                                                      ## Writes progress statement indicating data source for interpolation
          
          if(length(setdiff(RawBMD_Filename,   list.files(path = Path_Model_In, pattern = ".BMD2$|.bmd2$")))!= 0) {                                                                                                                   ## Validates that input datafiles properly named       
            stop(paste("Model .BMD2 input files not correctly named or in wrong filepath.  Check filepath = ", Path_Model_In,                                                                                                   ## Error / ends script
                       "\n Raw Model Filename = ", RawBMD_Filename, sep= ""))                                                           
          } else {
            
            Extract_Script <- grep("^Extract_BMD2_EXE", dir(Path_Scripts, pattern = ".R$"), value = TRUE) %>%
              dplyr::last(.)  
            source(paste(Path_Scripts, Extract_Script, sep = "/"))                                                                                                      ## Calls data extractions Script, uses latest version
            
            print(paste("Raw BMD2 data extracted using script", Extract_Script, sep = " "))
            
          }
        }
        
        #********************************************************************************************************************
        #                                        IF Model Type = LSPC, SOURCES the LSPC Extract script
        #********************************************************************************************************************
        
        if(Model_Type == "LSPC") {                                                                                                                                                                                      ## If a BMD2 filename was provided, the scripts will call the model data extractor script
          
          if(length(list.files(path = Path_Model_In, pattern = "*.out$")) == 0) {                                                                                                                   ## Validates that input datafiles properly named       
            stop(paste("LSPC model type was selected, but there are no .OUT files in the filepath.  Check filepath = ", Path_Model_In,                                                                                                   ## Error / ends script
                       "\n Raw Model Filename = ", RawBMD_Filename, sep= ""))                                                           
          } else {
            
            Extract_Script <- grep("^Extract_LSPC", dir(Path_Scripts, pattern = ".R$"), value = TRUE) %>%
              dplyr::last(.)  
            source(paste(Path_Scripts, Extract_Script, sep = "/"))                                                                                                      ## Calls data extractions Script, uses latest version
            
            print(paste("Raw LSPC .OUT files extracted using script", Extract_Script, sep = " "))
            
          }
        }
        
        #********************************************************************************************************************
        #                                        SOURCES the data preprocessing and interpolation script
        #********************************************************************************************************************
        
        Intrp_Script <- grep("^Intrp_Linear_Funx", dir(Path_Scripts, pattern = ".R$"), value = TRUE) %>%
          dplyr::last(.)  
        source(paste(Path_Scripts, Intrp_Script, sep = "/"))                                                                                                            ## Calls interpolation Script, uses latest version
        print(paste("Raw data processed and interpolated using script", Intrp_Script, sep = " "))
        
        #********************************************************************************************************************
        #                                        Prints Progress Statements
        #********************************************************************************************************************
        
        if(all(("RO" %in% unique(PCode_Lookup$LSPC_Code)|"PERO" %in% unique(PCode_Lookup$LSPC_Code)|"Flow.Out.of.Segment" %in% unique(PCode_Lookup$Anal_Name)), "CONVERT_FLOW_TO_AVGDAILY" %in% Plot_Types))  {                                                                                                                 ## When flow is activated, it will print the following messages
          print("Conversion of simulated flows to average daily flow was selected on the lookup table")
        } else print("Simulated flows were used as-is and not converted to average daily flow based on the selection in the lookup table")                                          ## Otherwise keeps as-is
        
        # if(Model_Type == "LSPC") {
        #   if("RO" %in% unique(PCode_Lookup$LSPC_Code)|"PERO" %in% unique(PCode_Lookup$LSPC_Code)) {                                                                                                                 ## When flow is activated, it will print the following messages
        #     if("CONVERT_FLOW_TO_AVGDAILY" %in% Plot_Types) {                                                                                                                              ## Indicates if flow was converted to average daily flow (averaging done in Intrp script)
        #       print("Conversion of simulated flows to average daily flow was selected on the lookup table")                                      
        #     } else (print("Simulated flows were used as-is and not converted to average daily flow based on the selection in the lookup table"))                                          ## Otherwise keeps as-is
        #   }
        #   
        # }
        # 
        # 
        # if(Model_Type != "LSPC") {
        #   if("Flow.Out.of.Segment" %in% unique(PCode_Lookup$Anal_Name)) {                                                                                                                 ## When flow is activated, it will print the following messages
        #     if("CONVERT_FLOW_TO_AVGDAILY" %in% Plot_Types) {                                                                                                                  ## Indicates if flow was converted to average daily flow (averaging done in Intrp script)
        #       print("Conversion of simulated flows to average daily flow was selected on the lookup table")                                      
        #     } else (print("Simulated flows were used as-is and not converted to average daily flow based on the selection in the lookup table"))                                          ## Otherwise keeps as-is
        #   }
        # }
        # 
        # 
        print(paste("Calibration station required at least", Min_Obs, "records for GoF calculations", sep = " "))                                                                                                  ## Prints min obs needed for GOF 
        print(paste("Boxplot, density plot, and geomean plot stations required at least", Min_Obs_BoxDens, "records to be included in the figure", sep = " "))                                                     ## Prints min obs needed for GOF 
        print(paste("Flagged RCodes were set to: ", paste(RCode_Exclusion, collapse = ", "), sep = " "))                                                                                                           ## Prints which RCodes were flagged
        print(paste("The following plot types were activated:", paste(Plot_Types[!grepl("CONVERT_FLOW_TO_AVGDAILY", Plot_Types)], collapse = ", "), sep = " "))                                                    ## Prints plot types called, but drops the convert to average daily flow option
        print(paste("Geomean calculations based on months", lubridate::month(min(Month_Geom), abbr = TRUE, label = TRUE), 
                    "to", lubridate::month(max(Month_Geom), abbr = TRUE, label = TRUE), sep = " "))                                                                                                                ## Prints the months used to calculate Geomean
        
        #********************************************************************************************************************
        #              Rest of IF Statement QA Interpolated Data, Processes Datafiles, and Plots Figures
        #********************************************************************************************************************
        
        #********************************************************************************************************************
        #                                        QA that Plot Types and Geomean Months Assigned
        #********************************************************************************************************************
        
        if(length(intersect(Plot_Types, c("CALIB", "BOX", "GEOMEAN", "DENSITY", "MAP", "HTML", "ANNUAL", "CONVERT_FLOW_TO_AVGDAILY"))) != length(Plot_Types)) {
          stop("Plot types not assigned or spelled correctly; \n At least one of the following plot types are required: Calib, Box, Geomean, Density")                           ## Error / ends script if plot types missing or misspelled
        } else { 
          month_chk <- c(suppressWarnings(as.numeric(as.character(Month_Geom[1]))), suppressWarnings(as.numeric(as.character(Month_Geom[length(Month_Geom)]))))  
          if(length(month_chk[!is.na(month_chk)]) != 2 | !(month_chk[1] %in% c(1:12)) | !(month_chk[2] %in% c(1:12))) {
            stop("Start and end month for geomean calculations either missing or not assigned to numeric integers between 1 and 12, inclusive")                         ## Error / ends script if geomean month filter not assigned
          } else { 
            
            
            #********************************************************************************************************************
            #                                        QA Subfolders Correctly Named in Working Directory
            #********************************************************************************************************************
            
            plot_dir <- grep("^Plot|*Stats", dir(str_remove(Path_Data_Out, "Proc_Out")), value = TRUE)
            if(length(setdiff(c("Plot_Box", "Plot_Calib", "Plot_Geom", "Plot_Dens", "Stats"), plot_dir)) != 0) {                                         ## Validates that plot subfolders properly named    
              stop(paste(paste(setdiff(c("Plot_Box", "Plot_Calib", "Plot_Geom", "Plot_Dens", "Stats"), plot_dir), collapse = " / "),                     ## Error / ends script
                         "subfolder(s) missing from working directory: ", getwd()))                                                                                        
            } else {
              yr_geom <- grep("*Year", dir(paste(str_remove(Path_Data_Out, "Proc_Out"), "/Plot_Geom/", sep = "")), value = TRUE)                         ## Validates that year subfolders in plot subfolders
              yr_box <- grep("*Year", dir(paste(str_remove(Path_Data_Out, "Proc_Out"), "/Plot_Box/", sep = "")), value = TRUE)
              yr_fold <- c(yr_geom, yr_box)
              
              if(sum(yr_fold == "Year") != 2){
                stop("Year subfolder missing from Boxplot and/or Geoplot subfolder")                                                                    ## Error / ends script
              } else {                                                                                                                                  ## If passes, then runs rest of script
                
                #********************************************************************************************************************
                #                           Reassigns interpolated datasets for the rest of the scripts
                #********************************************************************************************************************
                
                #ModelData <- Model_Intrp
                
                #MeasuredData <- Measured_Paired
                
                #********************************************************************************************************************
                #                                        QA Measured and Model Files Contain Records
                #********************************************************************************************************************
                
                if(nrow(Measured_Paired) == 0){                                                                                            ## Error check / ends script if Measured DF contains no records
                  stop("Measured dataframe contains no data records; Check datafile")
                } else {
                  if(nrow(Model_Intrp) == 0) {                                                                                             ## Error check / ends script if Model DF contains no records
                    stop("Model dataframe contains no data records; Check datafile")
                  } else {                                                                                                                 ## Runs entire script if DF contain data records
                    #Measured_Paired <- Measured_Paired %>% 
                    #  dplyr::rename_at(vars(contains('Plot_Title')), funs(sub('Plot_Title', 'Station_Name', .)))                          ## Renames column to Station_Name
                    
                    #********************************************************************************************************************
                    #                           QA Measured and Model Dataframes Contain Columns Required for Scripts
                    #********************************************************************************************************************
                    
                    MeasCol_Req <- c("Date_Time", "Station_ID", "Plot_Title", "Segment_ID",                                              ## Measured DF columns required in scripts
                                     "PCode", "Result_Obs", "Result_Sim", "Plot_Title", "Units", "RCode", "Depth")
                    ModCol_Req <- c("Date_Time", "Station_ID", "Plot_Title")                                                               ## Model DF columns required in scripts
                    
                    if (length(setdiff(MeasCol_Req, names(Measured_Paired))) != 0){
                      stop(paste(paste(setdiff(MeasCol_Req, names(Measured_Paired)), collapse = "; "), "column(s) is missing from input Measured dataframe"))                         ## Error / ends script if Measured DF is missing a required column
                    } else {
                      if (length(setdiff(ModCol_Req, names(Model_Intrp))) != 0){
                        stop(paste(paste(setdiff(ModCol_Req, names(Model_Intrp)), collapse = "; "), "column(s) is missing from input Model dataframe"))                               ## Error / ends script if Model DF is missing a required column
                      } else {
                        Pcode_Chk <- Model_Intrp[ , !(colnames(Model_Intrp) %in% c(ModCol_Req, "Segment_ID", "Pcode", "PCode", "Result", "Value")), drop = FALSE]                   ## Check that MODEL data in wide format
                        if(ncol(Pcode_Chk) < 1) {
                          warning("Number of Pcode columns in Model Data < 1; \nVerify that input Model dataframe is shaped wide, with each Pcode in a separate column")              ## Prints Warning to verify shape of DF; script will still run
                        }
                      }
                    }
                  }
                }
              }
              
              #********************************************************************************************************************
              #                                        Calls Data Processing Script via SOURCE
              #********************************************************************************************************************
              
              print("Input data passed preprocessing QA tests and sent to processing script")                                                                       
              Proc_Script <- grep("^Proc_Funx", dir(Path_Scripts), value = TRUE, ignore.case = TRUE) %>%                       ## Extracts data processing script
                dplyr::last(.)                                                                                                 ## Will use the latest script version
              source(paste(Path_Scripts, Proc_Script, sep = "/"))                                                              ## Calls data processing script
              print(paste("Data processed using script", Proc_Script, sep = " "))
              
              
              #********************************************************************************************************************
              #                                        Final QA of Processed Data Before GoF Stats and Plotting 
              #********************************************************************************************************************
              
              ######################### ERROR CHECK FOR NO DATA IN DF1 or DF2
              #Measured_Paired <- anti_join(Measured_Paired,Measured_Paired)
              #Model_Intrp <- anti_join(Model_Intrp,Model_Intrp)
              ####################################################
              
              ## If statement to report out any Pcodes selected for calibration that do not have paired measured/model data at any station (i.e., after processing and interpolation, no records matched)
              
              if(length(setdiff(unique(PCode_Lookup$PCode), unique(Measured_Intrp$PCode))) != 0) {
                print(paste("The following Pcodes were selected for calibration, but did not have paired measured and model data records at any monitoring station:  ", paste(setdiff(unique(PCode_Lookup$PCode), unique(Measured_Intrp$PCode)), collapse = ", "), 
                            "\nThe missing PCode(s) were excluded from calibration", sep = " "))
              }
              
              if(count(Missing) != 0) {                                                                                       ## Final IF statement that runs Plotting scripts
                stop("Processed model dataframe is missing a PCode or Station contained in the measured data")                ## Error /ends script if processed Model DF is missing a Station/PCode from Measured DF
              } else {
                if(nrow(Measured_Paired) == 0){
                  stop("Processed measured dataframe contains no data records")                                               ## Error / ends script if processed Measured DF has no records
                } else {
                  if(nrow(Model_Intrp) == 0) {
                    stop("Processed model dataframe contains no data records")                                                ## Error / ends script if processed Model DF has no records
                  } else {
                    
                    #********************************************************************************************************************
                    #                              Calls Script to Calculate Goodness of Fit Statistics
                    #********************************************************************************************************************
                    
                    print("Input data fully processed and exported to graphing scripts")
                    
                    
                    GOF_Script <- grep("^GOF_Stats", dir(Path_Scripts), value = TRUE) %>%
                      dplyr::last(.)  
                    
                    source(paste(Path_Scripts, GOF_Script, sep = "/"))                                                     ## Calls Density Script, uses latest version
                    
                    print(paste("GOF stats calculated using script", GOF_Script, sep = " "))
                    
                    #********************************************************************************************************************
                    #                          Loads the various plotting scripts and uses WALK to SOURCE them
                    #                              
                    #                       This only loads the custom functions for Density, Boxplots, and Calib.  
                    #                       
                    #               NOTE: When activated, geomean statistics are calculated when script SOURCED, but Geomeans not plotted until later
                    #********************************************************************************************************************
                    
                    Density_Script <- grep("^Plot_Funx_Density", dir(Path_Scripts), value = TRUE) %>%                                                  ## Extracts most recent density plot script
                      dplyr::last(.)  %>%
                      paste(Path_Scripts, ., sep = "/")
                    
                    Geomean_Script <- grep("^Plot_Funx_Geomean", dir(Path_Scripts), value = TRUE) %>%                                                  ## Extracts most recent geomean plot script
                      dplyr::last(.)  %>%
                      paste(Path_Scripts, ., sep = "/")
                    
                    Calib_Script <- grep("^Plot_Funx_Calib", dir(Path_Scripts), value = TRUE) %>%                                                      ## Extracts most recent mulicalib plot script
                      dplyr::last(.) %>%
                      paste(Path_Scripts, ., sep = "/")
                    
                    Box_Script <- grep("^Plot_Funx_Box", dir(Path_Scripts), value = TRUE) %>%                                                          ## Extracts most recent boxplot script
                      dplyr::last(.) %>%
                      paste(Path_Scripts, ., sep = "/")
                    
                    walk(.x = c(Density_Script, Calib_Script, Box_Script, Geomean_Script), ~source(.x))                                                ## Uses walk to SOURCE each of the scripts
                    
                    #********************************************************************************************************************
                    #                             For .BAT mode, will save image when activated in the .bat file
                    #********************************************************************************************************************
                    
                    if("Save_Image" %in% ls()) { 
                      if(Save_Image == TRUE) {
                        save.image()
                      }
                    }
                    
                    #********************************************************************************************************************
                    #                         Uses nested IF to WALK to generate and save each GEOMEAN PLOT as a separate .jpg
                    #********************************************************************************************************************
                    
                    if ("GEOMEAN" %in% Plot_Types) {
                      
                      if(length(PCode_Geom) == 0) {
                        print("Geomean plotting activated, but required PCodes not selected; PCode_Lookup table requires at least one of the following to be activated: TP, TN, or Chlorophyll-a")                                           ## Error message when geomean activated, but TN, TP, or CHLA not selected
                      } 
                      
                      if(length(NestData_Box$MeasSeg_Box[NestData_Box$PCode %in% PCode_Geom] %>% unlist()) == 0) {
                        print(paste0("Geomean plotting was activated, but no segments exceeded the ", Min_Obs_BoxDens, " minimum record cutoff (Min_Obs_BoxDens); Verify TP, TN, or Chlorophyll is activated in PCode_Lookup and cutoff"))   ## Error message when geomean activated, but data does not contain TN, TP, or CHLA data
                      } 
                      
                      NestData_Box %>%                                                                                                        ## Uses filtered Nested DF
                        dplyr::filter(PCode %in% PCode_Geom) %>%                                                                              ## Extracts only those Pcodes selected for geomean calculation
                        {walk2(.x = .$Model_Filt , .y = .$Meas_Filt , PLOT_SAVE_NESTED_GEOMEAN)}                                              ## Calls the geomean plotting function to generate GEOMEAN PLOTS
                      
                      #********************************************************************************************************************
                      #                         Uses nested IF to WALK to generate and save each ANNUAL GEOMEAN PLOT as a separate .jpg
                      #********************************************************************************************************************
                      
                      if ("ANNUAL" %in% Plot_Types) {
                        
                        if(length(NestData_Box$MeasSeg_Box[NestData_Box$PCode %in% PCode_Geom] %>% unlist()) == 0) {
                          print(paste0("Annual geomean plotting was activated, but no segments exceeded the ", Min_Obs_BoxDens, " minimum record cutoff (Min_Obs_BoxDens); Verify TP, TN, or Chlorophyll is activated in PCode_Lookup and cutoff"))   ## Error message when annual geomean activated, but data does not contain any TN, TP, or CHLA records
                        }
                        
                        NestData_Box %>%                                                                                                       ## Uses filtered Nested DF
                          dplyr::filter(PCode %in% PCode_Geom) %>%                                                                             ## Extracts only those Pcodes selected for geomean calculation
                          {walk2(.x = .$Model_Filt , .y = .$Meas_Filt , PLOT_ANNUAL_NESTED_GEOMEAN)}                                           ## Calls the annual geomean function for ALL GEOMEAN PCODES
                        
                      }
                      
                      print(paste("Geomean figures generated using script", Geomean_Script, sep = " "))                                        ## Reports script used to generate geomean figure
                      
                    }
                    
                    # source(paste(Path_Scripts, "Cleanup_Environ.R", sep = "/"))                                                                ## Eliminates R objects   
                    
                    #********************************************************************************************************************
                    #                         Uses WALK to generate and save each DENSITY PLOT as a separate .jpg
                    #********************************************************************************************************************
                    
                    
                    if ("DENSITY" %in% Plot_Types & !"HTML" %in% Plot_Types) {                                                                  ## Plots Density figure if called in plotting vector
                      
                      #PLOT_SINGLE_DENS(c("DO", "TP", "FLOWCMS"))                                                                               ## Calls single density function based on selected PCODES
                      
                      walk(.x = NestData_Box$Meas_Filt, PLOT_SAVE_NESTED_DENSITY)                                                               ## Calls the density plotting function to create a faceted Density plot for ALL PCODES
                      
                      print(paste("Density figures generated using script", Density_Script, sep = " "))
                      
                    }
                    
                    #********************************************************************************************************************
                    #                         Uses WALK to generate and save MULTIPANEL CALIBRATION PLOT 
                    #                         
                    #                   Uses Plot_Type to determine if export as static plot .jpg or interactive HTML plot
                    #                   
                    #                    The scripts do **not** allow both HTML and Static JPGS to be created simultaneously
                    #                         
                    #                     This optimizes script run times due to lengthy processing time for saving .JPG
                    #
                    #              Will skip any Pcode selected for calibration, which does not appear in the calibration dataset
                    #********************************************************************************************************************
                    
                    if ("CALIB" %in% Plot_Types) {                                                                                              ## Plots multipanel figure if called in plotting vector
                      if ("HTML" %in% Plot_Types) {                
                        
                        Vec_HTML <- intersect(unique(PCode_Lookup$PCode), unique(NestData_Multi$PCode))                                         ## Creates vector of Pcodes and only keeps those selected that are included in the calibration data
                        
                        #walk(.x = unique(PCode_Lookup$PCode), ~withr::with_dir(paste0(Path_Data_Out, "/Plot_HTML/"), saveWidget(HTML_CALIB(.x), title = paste0(.x, "_Scnrio"), selfcontained = TRUE, file = paste0(.x, "_Calib.html"))))
                        
                        walk(.x = Vec_HTML, ~withr::with_dir(paste0(Path_Data_Out, "/Plot_HTML/"), saveWidget(HTML_CALIB(.x), title = paste0(.x, "_Scnrio"), selfcontained = TRUE, file = paste0(.x, "_Calib.html"))))
                        
                        
                        
                        print("Multipanel calibration plots exported as interactive HTML plots rather than static plots")
                        
                      } else {
                        
                        ## Next line can be used to create JPGs w/o Figure Number
                        
                        walk2(.x = NestData_Multi$Model_Filt, .y = NestData_Multi$Meas_Filt, PLOT_SAVE_NESTED_MULTIPANEL)                         ## Calls the nested multipanel function for ALL PCODES
                        
                        print("Multipanel calibration plots exported as static plots rather than HTML plots")
                        
                        
                        #********************************************************************************************************************
                        #                         Manually set switch to either plot JPGs or PDF, including FIGURE NUMBER
                        #                   
                        #                                Numbered PDFs allows merging of PDFs into report appendices
                        #********************************************************************************************************************
                        
                        #PLOT_SINGLE_MULTI(c("TP"), c("1", "5"))                                                                       ## Calls multipanel function based on selected PCODES/SEGMENTS
                        
                        
                        ## Next line can be used to create numbered PDFs in sequential order of Figure Number
                        
                        #pwalk(.l = list(MODEL_DF = NestData_Multi$Model_Filt, MEAS_DF = NestData_Multi$Meas_Filt, FIG_TYPE = "PDF", FIG_NUMB = NestData_Multi$Fig_Numb), PLOT_SAVE_NESTED_MULTIPANEL)                                 ## Calls the nested multipanel function for ALL PCODES; but adds Figure # and saves as PDF
                        #pwalk(.l = list(MODEL_DF = NestData_Multi$Model_Filt, MEAS_DF = NestData_Multi$Meas_Filt, FIG_TYPE = "JPG", FIG_NUMB = NestData_Multi$Fig_Numb), PLOT_SAVE_NESTED_MULTIPANEL)                                 ## Calls the nested multipanel function for ALL PCODES; but adds Figure # and saves as PDF
                        
                      }
                      
                      print(paste("Multipanel calibration figures generated using script", Calib_Script, sep = " "))                            ## Reports script used to generate multipanel calibration figure
                      
                    }
                    
                    # source(paste(Path_Scripts, "Cleanup_Environ.R", sep = "/"))                                                                 ## Removes up R objects     
                    
                    #********************************************************************************************************************
                    #                         Uses nested IF to WALK to generate and save BOXPLOT PLOT as a separate .jpg
                    #                         
                    #********************************************************************************************************************
                    
                    if ("BOX" %in% Plot_Types & !"HTML" %in% Plot_Types) {                                                                                                 ## Plots Boxplot figure if called in plotting vector
                      
                      #PLOT_SINGLE_BOX(c("FLOW_CMS", "TP"))                                                                                            ## Calls boxplot function based on selected PCODES
                      
                      walk2(.x = NestData_Box$Model_Filt, .y = NestData_Box$Meas_Filt, PLOT_SAVE_NESTED_BOXPLOT)                                 ## Calls the nested boxplot function for ALL PCODES
                      
                      #********************************************************************************************************************
                      #                         Uses nested IF to WALK to generate and save each ANNUAL BOXPLOT PLOT as a separate .jpg
                      #********************************************************************************************************************
                      
                      if ("ANNUAL" %in% Plot_Types) {
                        
                        walk2(.x = NestData_Box$Model_Filt, .y = NestData_Box$Meas_Filt, PLOT_ANNUAL_NESTED_BOXPLOT)                               ## Calls nested annual boxplot function for ALL PCODES
                        
                      }
                      
                      print(paste("Boxplot figures generated using script", Box_Script, sep = " "))                                              ## Reports script used to generate boxplot
                      
                      print(paste("Boxplots set to display", Boxplot_Stat))
                      
                      #********************************************************************************************************************
                      #                         Uses IF statement to generate Distribution HTML maps
                      #                     Generated if HTML selected and either Density or Boxplot activated
                      #                     
                      #          Scripts automatically generate BOTH Density and Boxplot when either is selected AND HTML option is activated
                      #          
                      #                   HTML and static plots are generated simultaneously for distribution plots
                      #
                      #              Will skip any Pcode selected for calibration, which does not appear in the calibration dataset
                      #          
                      #********************************************************************************************************************
                      
                    }
                    
                    if (any(str_detect(Plot_Types, "BOX|DENSITY")) & "HTML" %in% Plot_Types) {
                      
                      Vec_HTML <- intersect(unique(PCode_Lookup$PCode), unique(NestData_Box$PCode))                                                                                                                                     ## Create vector of pcodes to plot,removing any selected PCodes that are not in calibration data
                      
                      #walk(.x = unique(PCode_Lookup$PCode), ~withr::with_dir(paste0(Path_Data_Out, "/Plot_HTML/"), saveWidget(HTML_DISTRO(.x), title = paste0(.x, "_Dist"), selfcontained = TRUE, file = paste0(.x, "_Dist.html"))))
                      
                      walk(.x = Vec_HTML, ~withr::with_dir(paste0(Path_Data_Out, "/Plot_HTML/"), saveWidget(HTML_DISTRO(.x), title = paste0(.x, "_Dist"), selfcontained = TRUE, file = paste0(.x, "_Dist.html"))))
                      
                      print("Boxplots and Density plots exported as interactive HTML plots")
                      
                    } 
                    
                    #source(paste(Path_Scripts, "Cleanup_Environ.R", sep = "/"))                                                                ## Removes R objects
                    
                  }
                }
              } 
            }
          }
        }   
      }
    }
  }
}


