
#********************************************************************************************************************
#
#                    Includes spatial viewer scripts and arguments (including HTML vs. SAVE argument to either save figures or create spatial HTML map)
#                                  
#                           SOURCE FUNCTION Script to generate faceted BOXPLOT plot for Pcodes
#                                    Based on paired Sim vs. Obs DF and full Sim DF
#
#                           Input data needs to be preprocessed based on Proc_Source Script
#                      Designed to Run as SOURCE Script Called in Master_Script v 6.0 or later
#
#                            Includes a switch to either plot Medians or Avgs on boxplots
#
#         Requires purrr and nested dataframe (NestedData_Box) and will exclude segments that do not exceed the Min_Obs_BoxDens cutoff  
#         
#                                         J. Davis v2021-07-14                                                   
#                           
#********************************************************************************************************************


#********************************************************************************************************************
#                         FUNCTION to create faceted boxplot plot
#                         
#         Includes IF statement to plot either AVG or MEDIAN of simulated/observed data
#                 
#                Plots statistics based on paired and full simulated dataset
#                         
#                   
#********************************************************************************************************************

PLOT_SAVE_NESTED_BOXPLOT <- function(MODEL_DF, MEAS_DF, OUTPUT = "SAVE") {
  
  if(nrow(MEAS_DF) > 0) {
    
    file_name <- paste0(Path_Data_Out, "/Plot_Box/", unique(unique(MEAS_DF$PCode)), "_Box")                                                                                                ## Creates filepath/filename for the saved figure
    

    
    ggplot() +
      geom_boxplot(data = MODEL_DF, aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)),                                                                                                         ## Generates the boxplot for PCode; coef arg sets whiskers to data range
                                        y = Result_Sim_Full), color = "black", coef = 1000) +
                                   
      ## Plot Avg PAIRED OBS data
      {if(nrow(MEAS_DF) > 0 & Boxplot_Stat %in% c("MEAN", "AVG", "AVERAGE"))stat_summary(data = MEAS_DF, aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)), y = Result_Obs, color = "Obs-Avg-Pair", shape = "Obs-Avg-Pair", size = "Obs-Avg-Pair"),        ## IF statement to plot measured average when available & activated
                                                                  fun = mean, na.rm = TRUE, geom = "point", show.legend = TRUE)} +
      ## Plot Median PAIRED OBS data
      {if(nrow(MEAS_DF) > 0 & Boxplot_Stat == "MEDIAN")stat_summary(data = MEAS_DF, aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)), y = Result_Obs, color = "Obs-Med-Pair", shape = "Obs-Med-Pair", size = "Obs-Med-Pair"),                             ## IF statement to plot measured median when available & activated
                                                                    fun = median, na.rm = TRUE, geom = "point", show.legend = TRUE)} +
      ## Plot Avg of PAIRED & ALL SIM data
      {if(Boxplot_Stat %in% c("MEAN", "AVG", "AVERAGE"))stat_summary(data = MODEL_DF, aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)), y = Result_Sim_Full, color = "Sim-Avg-All", shape = "Sim-Avg-All", size = "Sim-Avg-All"),                         ## IF statement to plot mean of full simulated data when activated
                                              fun = mean, na.rm = TRUE, geom = "point")}+
      {if(Boxplot_Stat %in% c("MEAN", "AVG", "AVERAGE"))stat_summary(data = MEAS_DF, aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)), y = Result_Sim, color = "Sim-Avg-Pair", shape = "Sim-Avg-Pair", size = "Sim-Avg-Pair"),                            ## IF statement to plot mean  paired simulated data when activated
                                              fun = mean, na.rm = TRUE, geom = "point")} +                                                                                                                                                                                                
      ## Plot median of only PAIRED SIM data
      {if(Boxplot_Stat == "MEDIAN")stat_summary(data = MEAS_DF, aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)), y = Result_Sim, color = "Sim-Med-Pair", shape = "Sim-Med-Pair", size = "Sim-Med-Pair"), 
                                                fun = mean, na.rm = TRUE, geom = "point")} +
      ## Manually sets shape type, color, and size based on crosswalks
      geom_jitter(data = MEAS_DF, aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)),                                                                                                           ## Generates the scatter plot of MEAS_DF data for PCode
                                      y = Result_Obs, fill = factor(RCode, levels = c("Accepted", "Flagged"))), shape = 21, size = 2.5, stroke = 0, width = 0.2, alpha = 0.6, color = "white", show.legend = TRUE) +    
      scale_color_manual(values = c("Obs-Avg-Pair" = "red", 
                                    "Sim-Avg-Pair" = "green", 
                                    "Sim-Avg-All" = "black", 
                                    "Obs-Med-Pair" = "red", 
                                    "Sim-Med-Pair" = "green" ), 
                         drop = FALSE, name = "Average") +
      scale_fill_manual(values = c("Accepted" = "dodgerblue", "Flagged" = "red"), 
                        drop = FALSE, name = "Remark Code") +                                                                                                                                                                  
      scale_shape_manual(values = c("Obs-Avg-Pair" = 5, 
                                    "Sim-Avg-Pair" = 20, 
                                    "Sim-Avg-All" = 17,
                                    "Obs-Med-Pair" = 5, 
                                    "Sim-Med-Pair" = 20), 
                         drop = FALSE, name = "Average") +
      scale_size_manual(values = c("Obs-Avg-Pair" = 5, 
                                   "Sim-Avg-Pair" = 7, 
                                   "Sim-Avg-All" = 4,
                                   "Obs-Med-Pair" = 5, 
                                   "Sim-Med-Pair" = 7), 
                        drop = FALSE, name = "Average") +
      ## Changes aesthetics
      {if(str_detect(str_to_upper(unique(MEAS_DF$PCode)), "PH|WSE"))coord_cartesian(ylim = c(min(c(MEAS_DF$Result_Obs, MODEL_DF$Result_Sim_Full), na.rm = TRUE), max(c(MEAS_DF$Result_Obs, MODEL_DF$Result_Sim_Full), na.rm = TRUE)))} +    ## Changes axis range when PCODE is pH or WSE                                                                                          ## Scales the y-axis based on combined max of model and MEAS_DF data  
      labs(title = paste0(unique(MEAS_DF$PCode), "\n(Simulation Period)"),                                                                                                                                                                  ## Labels the plot and axes 
           x = "Stations \n Downstream to Upstream",
           y = paste(unique(MEAS_DF$PCode), " (",  unique(MEAS_DF$Units), ")", sep = ""),
           caption = paste0("Model Run: ", Model_Run_Code, "\nBoxplot: Full simulated dataset; Scatter: Measured dataset", "\nDate Plotted: ", Sys.Date())) +
      theme_bw() +
      theme(plot.title = element_text(hjust = 0.5),                                                                                                                                                                                         ## Formats the plot        
            axis.text = element_text(size = 6),
            axis.text.x = element_text(angle = 80, hjust = 1,face = "bold"),
            legend.position = "right",
            legend.key.size =  unit(0.75, "cm"),
            plot.caption = element_text(size = 6)) 
    
    #********************************************************************************************************************
    #                              IF Statement to either save figure or render as HTML for spatial comparisons
    #********************************************************************************************************************
    
    if(OUTPUT == "HTML") {
      ggplot2::last_plot()
    } else {
      
    ggsave(paste0(file_name, ".jpg"), ggplot2::last_plot(),                                                                                                                                                                                ## Saves the plot based on filename
           width = 12, height = 6, units = "in", dpi = 300)   
    }
  }
}


#********************************************************************************************************************
#                         FUNCTION to create faceted boxplot plot w/ separate panels by year
#                  Includes IF statement to plot either AVG or MEDIAN of simulated/observed data
#                 
#                         Plots statistics based on paired and full simulated dataset
#                            
#********************************************************************************************************************

PLOT_ANNUAL_NESTED_BOXPLOT <- function(MODEL_DF, MEAS_DF) {
  
  if(nrow(MEAS_DF) > 0) {
    
    file_name <- paste0(Path_Data_Out, "/Plot_Box/Year/", unique(unique(MEAS_DF$PCode)), "_Box")                                                                                               ## Creates filepath/filename for the saved figure
    
    ggplot() +
      geom_boxplot(data = MODEL_DF, aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)),                                                                                                       ## Generates the boxplot for PCode coef arg sets whiskers to data range
                                        y = Result_Sim_Full), color = "black", coef = 1000) +
      ## Plot Avg PAIRED OBS data
      {if(nrow(MEAS_DF) > 0 & Boxplot_Stat %in% c("MEAN", "AVG", "AVERAGE"))stat_summary(data = MEAS_DF, aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)), y = Result_Obs, color = "Obs-Avg-Pair", shape = "Obs-Avg-Pair", size = "Obs-Avg-Pair"),        ## IF statement to plot measured average when available & activated
                                                                  fun = mean, na.rm = TRUE, geom = "point", show.legend = TRUE)} +
      ## Plot Median PAIRED OBS data
      {if(nrow(MEAS_DF) > 0 & Boxplot_Stat == "MEDIAN")stat_summary(data = MEAS_DF, aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)), y = Result_Obs, color = "Obs-Med-Pair", shape = "Obs-Med-Pair", size = "Obs-Med-Pair"),                             ## IF statement to plot measured median when available & activated
                                                                    fun = median, na.rm = TRUE, geom = "point", show.legend = TRUE)} +
      ## Plot Avg of PAIRED & ALL SIM data
      {if(Boxplot_Stat %in% c("MEAN", "AVG", "AVERAGE"))stat_summary(data = MODEL_DF, aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)), y = Result_Sim_Full, color = "Sim-Avg-All", shape = "Sim-Avg-All", size = "Sim-Avg-All"),                         ## IF statement to plot mean of full simulated data when activated
                                              fun = mean, na.rm = TRUE, geom = "point")}+
      {if(Boxplot_Stat %in% c("MEAN", "AVG", "AVERAGE"))stat_summary(data = MEAS_DF, aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)), y = Result_Sim, color = "Sim-Avg-Pair", shape = "Sim-Avg-Pair", size = "Sim-Avg-Pair"),                            ## IF statement to plot mean of paired simulated data when activated
                                              fun = mean, na.rm = TRUE, geom = "point")} +                                                                                                                                                                                                
      ## Plot median of only PAIRED SIM data
      ## The scripts do NOT calculate a median based on ALL data separately because the boxplot function already plots the median of all simulated data as a line.
      {if(Boxplot_Stat == "MEDIAN")stat_summary(data = MEAS_DF, aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)), y = Result_Sim, color = "Sim-Med-Pair", shape = "Sim-Med-Pair", size = "Sim-Med-Pair"), 
                                                fun = mean, na.rm = TRUE, geom = "point", show.legend = TRUE)} +
      geom_jitter(data = MEAS_DF, aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)),                                                                                                         ## Generates the scatter plot of MEAS_DF data for PCode
                                      y = Result_Obs, fill = factor(RCode, levels = c("Accepted", "Flagged"))), shape = 21, size = 2.5, stroke = 0, width = 0.2, alpha = 0.6, color = "white", show.legend = TRUE) +
      ## Manually sets shape type, color, and size based on manual crosswalk
      scale_color_manual(values = c("Obs-Avg-Pair" = "red", 
                                    "Sim-Avg-Pair" = "green", 
                                    "Sim-Avg-All" = "black", 
                                    "Obs-Med-Pair" = "red", 
                                    "Sim-Med-Pair" = "green" ), 
                         drop = FALSE, name = "Average") +
      scale_fill_manual(values = c("Accepted" = "dodgerblue", "Flagged" = "red"), 
                        drop = FALSE, name = "Remark Code") +                                                                                                                                                                  ## Manually recolors the MEAS_DF data
      scale_shape_manual(values = c("Obs-Avg-Pair" = 5, 
                                    "Sim-Avg-Pair" = 20, 
                                    "Sim-Avg-All" = 17,
                                    "Obs-Med-Pair" = 5, 
                                    "Sim-Med-Pair" = 20), 
                         drop = FALSE, name = "Average") +
      scale_size_manual(values = c("Obs-Avg-Pair" = 5, 
                                   "Sim-Avg-Pair" = 7, 
                                   "Sim-Avg-All" = 4,
                                   "Obs-Med-Pair" = 5, 
                                   "Sim-Med-Pair" = 7), 
                        drop = FALSE, name = "Average") +
      ## Changes aesthetics
      {if(str_detect(str_to_upper(unique(MEAS_DF$PCode)), "PH|WSE"))coord_cartesian(ylim = c(min(c(MEAS_DF$Result_Obs, MODEL_DF$Result_Sim_Full), na.rm = TRUE), max(c(MEAS_DF$Result_Obs, MODEL_DF$Result_Sim_Full), na.rm = TRUE)))} +    ## Changes axis range when PCODE is pH or WSE                                                                                          ## Scales the y-axis based on combined max of model and MEAS_DF data
      labs(title = paste0(unique(MEAS_DF$PCode), "\n(Simulation Period)"),                                                                                                                                                                  ## Labels the plot and axes
           x = "Stations \n Downstream to Upstream",
           y = paste(unique(MEAS_DF$PCode), " (",  unique(MEAS_DF$Units), ")", sep = ""),
           caption = paste0("Model Run: ", Model_Run_Code, "\nBoxplot: Full simulated dataset; Scatter: Measured dataset", "\nDate Plotted: ", Sys.Date())) +
      theme_bw() +
      theme(plot.title = element_text(hjust = 0.5),                                                                                                                                                                                         ## Formats the plot
            axis.text = element_text(size = 6),
            axis.text.x = element_text(angle = 80, hjust = 1,face = "bold"),
            legend.position = "right",
            legend.key.size =  unit(0.75, "cm"),
            plot.caption = element_text(size = 6))  +
      facet_wrap_paginate(~Year, ncol = 1, nrow = 2, page = 1, labeller = label_both)                                                                                                            ## Uses facet wrap to split across pages; this first call will be used to calculate total numbe of pages needed
    
    pages <- n_pages(ggplot2::last_plot())                                                                                                                                                                ## Calculate number of pages needed to capture all facet pages
    
    for (j in 1:pages) {                                                                                                                                                                         ## For loop to iteratively plot and save each page of facet wrap paginate
      ggsave(paste0(file_name, "_", j, ".jpg", sep = ""), ggplot2::last_plot() +
               facet_wrap_paginate(~Year, ncol = 1, nrow = 2, page = j, labeller = label_both), width = 8, height = 10, dpi = 600)
    }
  }
}

#********************************************************************************************************************
#                         FUNCTION to create single boxplot plot based on targeted Pcodes
#********************************************************************************************************************

PLOT_SINGLE_BOX <- function(PCODE_VEC) {
  
  NestData_Box %>%                                                                                                   ## Filters NestedDF based on vector of PCodes
    dplyr::filter(PCode %in% PCODE_VEC) %>%
    {walk2(.x = .$Model_Filt, .y = .$Meas_Filt, PLOT_SAVE_NESTED_BOXPLOT)}                                                ## Feeds filtered dataframe into boxplot function
  
}

# 
# #********************************************************************************************************************
# #                         Uses WALK to save each figure as a separate .Jpg
# #********************************************************************************************************************
# 
# #walk2(.x = NestedData$Model_Data, .y = NestedData$Meas_Data, PLOT_SAVE_NESTED_BOXPLOT)                                 ## Calls the nested boxplot function for ALL PCODES
# 
# #PLOT_SINGLE_BOX(c("DO", "TP"))                                                                                          ## Calls boxplot function based on selected PCODES
# 
# #walk2(.x = NestedData$Model_Data, .y = NestedData$Meas_Data, PLOT_ANNUAL_NESTED_BOXPLOT)                               ## Calls nested annual boxplot function for ALL PCODES
# 
# # walk2(.x = NestData_Box$Model_Filt, .y = NestData_Box$Meas_Filt, PLOT_SAVE_NESTED_BOXPLOT)                                 ## Calls the nested boxplot function for ALL PCODES
# # 
# # PLOT_SINGLE_BOX(c("DO", "TP"))                                                                                          ## Calls boxplot function based on selected PCODES
# # 
# # walk2(.x = NestData_Box$Model_Filt, .y = NestData_Box$Meas_Filt, PLOT_ANNUAL_NESTED_BOXPLOT)                               ## Calls nested annual boxplot function for ALL PCODES
