

#********************************************************************************************************************
#
#             Includes spatial viewer scripts and arguments (including HTML vs. SAVE argument to either save figures or create spatial HTML map)
#
#                           SOURCE FUNCTION Script to generate faceted density plot for Pcodes
#                                    Based on paired Sim vs. Obs data
#
#                           Input data needs to be preprocessed based on Proc_Source Script
#                         Designed to Run as SOURCE Script Called in Master_Script v6.0 or later
#                              
#       Requires purrr and nested dataframe (NestedData_Box) and will exclude segments that do not exceed the Min_Obs_BoxDens cutoff     
#       
#                                         J. Davis v2021-07-14      
#                                                                        
#                           
##********************************************************************************************************************


#********************************************************************************************************************
#                         FUNCTION to create faceted density plot
#                         
#                   OUTPUT argument defaults to SAVE, but can be set to HTML (allowing HTML maps to be displayed)
#********************************************************************************************************************

PLOT_SAVE_NESTED_DENSITY <- function(MEAS_DF, OUTPUT = "SAVE") {
  
  if(nrow(MEAS_DF) > 0) {                                                                                                                 ## If statement so skip PCodes that had no measured data at any segment
    
    file_name <- paste0(Path_Data_Out, "/Plot_Dens/", unique(unique(MEAS_DF$PCode)), "_Dens")              ## Creates custom filename  
    
    ggplot(data = MEAS_DF) +
      geom_density(aes(x = Result_Obs, y = -after_stat(scaled), fill = "Obs"), trim = TRUE) +                                               ## Plots scaled density of observed data
      geom_density(aes(x = Result_Sim, y = after_stat(scaled), fill = "Sim"), trim = TRUE) +                                                  ## Plots scaled density of simulated data, but reverses the sign to allow paired plotting
      coord_flip() +                                                                                                                  ## Flips X and Y axes
      labs(title = paste0(unique(MEAS_DF$PCode), "\n(Simulation Period)"),                                                            ## Provides plot title by extracting Plot title and Stat_code from nested DF
           x = paste(unique(MEAS_DF$PCode), " (",  unique(MEAS_DF$Units), ")", sep = ""),                                             ## Labels y axis by extracting nested Label_Y value
           y = "Scaled density",
           caption = paste0("Date Plotted: ", Sys.Date())) +
      theme_bw() +
      theme(plot.title = element_text(hjust = 0.5, size = 14),                                                                        ## Formats the plot        
            axis.text = element_text(size = 12),
            axis.text.x = element_text(angle = 80, hjust = 1, face = "bold"),
            legend.position = "bottom",
            legend.title = element_blank(),
            legend.spacing.x = unit(0.5, 'cm')) + 
      scale_y_continuous(labels = c(1.0, 0.5, 0, 0.5, 1.0)) +                                                                         ## Forces scale to be labeled w/o the negative label, which is an artificial value
      scale_fill_manual(values = c("Obs" = "black", "Sim" = "red")) +                                                                 ## Manually changes legend
      facet_wrap(~factor(MEAS_DF$Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)), scales = "free_y", drop = TRUE)       ## Facet wraps by plot-title (ordered)
    
    #********************************************************************************************************************
    #                        IF Statement to either save figure or render as HTML map for spatial comparisons
    #                              
    #                        The scripts will simultaneously create both HTML and Static JPGS for distribution plots
    #                         
    #********************************************************************************************************************
    
    
    if(OUTPUT == "HTML") {
      ggplot2::last_plot()
    } else {
      
      ggsave(paste0(file_name, ".jpg"), ggplot2::last_plot(),                                                                             ## Saves the faceted plot
             width = 12, height = 6, units = "in", dpi = 300)   
    }
  }
}

#********************************************************************************************************************
#                         FUNCTION to create single boxplot plot based on targeted Pcodes
#********************************************************************************************************************

PLOT_SINGLE_DENS <- function(PCODE_VEC) {
  
  NestData_Box %>%                                                                                                   ## Filters NestedDF based on vector of PCodes
    dplyr::filter(PCode %in% PCODE_VEC) %>%
    {walk(.x = .$Meas_Filt, PLOT_SAVE_NESTED_DENSITY) }                                                                          ## Feeds filtered dataframe into boxplot function
  
}



#********************************************************************************************************************
#                         FUNCTION to render multipanel density and boxplot figures with interactive watershed HTML map
#                   
#********************************************************************************************************************


HTML_DISTRO <- function(PCODE) {
  
  #********************************************************************************************************************
  #                                        Preprocesses DF for HTML mapping
  #                         
  #         Because these plots are not faceted in the spatial HTML map, the nested DF requires additional processing to
  #                 allow individual boxplots and density plots to be generated for each map pin
  #********************************************************************************************************************
  
  
  temp <- NestData_Box %>%                                                                                                                 ## Filters NestedDF based on vector of PCodes
    dplyr::filter(PCode == PCODE) %>%                                                     
    select(Meas_Filt) %>%                                                                                                                  ## Extracts and unnests paired measured data                                                                   
    unnest(Meas_Filt) 
  
  if(nrow(temp) != 0) {                                                                                                                       ## Will skip PCODES that are not in the calibration DF
    temp <- temp %>%
      select(PCode, Segment_ID, Plot_Title, Date_Time, Result_Obs, Result_Sim, RCode, Units) %>%                                             ## Keeps these columns from nested DF
      group_nest(PCode, Segment_ID, Plot_Title, keep = TRUE, .key = "Meas_Filt") %>%                                                         ## Renests selected paired measured data
      ## This next line of code extracts and reformats the Model_Filt data inline before applying left_join
      left_join(., NestData_Box %>%                                                                                                          ## Joins paired measured data with reprocessed model data
                  dplyr::filter(PCode == PCODE) %>%                                                                                          ## Extracts PCode                                  
                  select(Model_Filt) %>%                                                                                                     ## Select and unnest model DF
                  unnest(Model_Filt) %>%             
                  select(PCode, Segment_ID, Plot_Title, Date_Time, Result_Sim_Full) %>%                                                      ## Keeps these columns from nested DF
                  group_nest(PCode, Segment_ID, keep = TRUE, .key = "Model_Filt"), by = c("PCode", "Segment_ID")) %>%                        ## Renests selected model data and then joins to paired meas
      mutate(dens = map2(.x = .$Meas_Filt, .y = rep("HTML", times = length(.$Meas_Filt)), PLOT_SAVE_NESTED_DENSITY)) %>%                      ## Creates density plot and appends DF
      mutate(box = pmap(.l = list(.$Model_Filt, .$Meas_Filt, rep("HTML", times = length(.$Meas_Filt))), PLOT_SAVE_NESTED_BOXPLOT)) %>%        ## Creates boxplot and appeds DF
      mutate(Long = Station_Order_Lookup$Long[match(Segment_ID, Station_Order_Lookup$Segment_ID)],                                           ## Adds Lat/Long/Plot_Title metadata for spatial mapping
             Lat = Station_Order_Lookup$Lat[match(Segment_ID, Station_Order_Lookup$Segment_ID)],
             Plot_Title = Station_Order_Lookup$Plot_Title[match(Segment_ID, Station_Order_Lookup$Segment_ID)]) %>%
      mutate(mrgd = map2(.x = .$dens, .y = .$box, ~as_ggplot(arrangeGrob(.x, .y, ncol = 2))))                                                ## Merges density and boxplot into 2 panel figure; then converts to ggplot object
    
    
    #********************************************************************************************************************
    #                         Creates spatial HTML map of boxplot and density figures w/ pop out maps
    #********************************************************************************************************************
    
   
    leaflet() %>%
      addTiles() %>%                                                                                                                                   ## Adds basemap, which is OpenStreets by default
      addPolygons(data = st_as_sf(Wtrshd_Map), color = "Blue", opacity = 1) %>%                                                                        ## Adds watershed map
      addPolylines(data = st_as_sf(Stream_Map), color = "Black", opacity = 1, weight = 0.75, fill = TRUE, fillColor = "Black", fillOpacity = 1) %>%    ## Adds stream network map
      addMarkers(data = temp, lng = ~Long, lat = ~Lat, label = ~Plot_Title, group = "pt", labelOptions = labelOptions(textsize = "14px")) %>%          ## Adds monitoring stations pins w/ labels
      addPopupGraphs(temp$mrgd, group = "pt", width = 600, height = 600)                                                                               ## Adds popup density/boxplot figures to each pin
  }
}















#********************************************************************************************************************
#                         Uses WALK to save each figure as a separate .Jpg
#********************************************************************************************************************

#PLOT_SINGLE_DENS(c("DO", "TP"))                                                                                              ## Calls boxplot function based on selected PCODES

#walk(.x = NestedData$Meas_Data, PLOT_SAVE_NESTED_DENSITY)                                                                   ## Creates faceted Density plot for ALL PCODES


# PLOT_SINGLE_DENS(c("DO", "TP", "FLOWCMS"))                                                                                              ## Calls boxplot function based on selected PCODES
# 
# walk(.x = NestData_Box$Meas_Filt, PLOT_SAVE_NESTED_DENSITY)                                                                   ## Creates faceted Density plot for ALL PCODES
