
#********************************************************************************************************************
#
#                    Script to Generate interactive PLOTLY Time Series, 1:1 Regression, and ECDF plots
#                           Input data needs to be preprocessed based on PLOT_FUNX_PROC Script
#                    Designed to Run as SOURCE Script after DF fully processed and converted into nested DF
#                            
#          The TS, REG, and ECDF require a third argument (NUMBER or NAME) to flexibly filter the DF based on Segment Number or Name
#          
#          If filtering by number, a numeric segment number must be supplied.  If filtering by name, a known segment name must be supplied
#                      
#                         The last function is a multipanel filter reporting TS, ECDF, and 1:1 linear regress
#                         
#           Requires purrr and a nested dataframe (NestedData_Multi) and will exclude segments that do not meet the Min_Obs cutoff
#
#********************************************************************************************************************


#********************************************************************************************************************
#                                          Loads plotly library
#********************************************************************************************************************

library(plotly)

#********************************************************************************************************************
#                             Creates basic Time Series plot that will be called by plotly
#********************************************************************************************************************

PLOT_BASIC_TS <- function(MODEL_DF, MEAS_DF) {
  
  if(nrow(MEAS_DF) > 0) {
  
  ggplot() +
    geom_line(data = MODEL_DF, aes(x = Date_Time, y = Result_Sim_Full), linetype = 1, linewidth = 0.75, alpha = 0.7, color = "black") +                                                      ## Creates the second time series for the Median replacement values
    geom_point(data = MEAS_DF, aes(x = Date_Time, y = Result_Obs, fill = factor(RCode, levels = c("Accepted", "Flagged"))), shape = 21, size = 2, stroke = 0, na.rm = TRUE, show.legend = TRUE) +           ## Adds the measured observations to the time series
    scale_fill_manual(values = c("Accepted" = "dodgerblue", "Flagged" = "red", alpha = 0.6), drop = FALSE, name = "Remark Code", na.translate = FALSE) +                                ## Manually recolors the measured data
    theme_bw() +
    scale_x_datetime(date_breaks = "2 years", date_labels = "%Y") +                                                                                                                     ## Sets the breakpoints on the x-axis
    #geom_hline(aes(yintercept = 0), color = "black", linetype = 3) +                                                                                                                    ## Adds a horizontal reference line at 0
    {if(str_detect(str_to_upper(unique(MEAS_DF$PCode)), "PH|WSE")) coord_cartesian(ylim = c(range(c(MODEL_DF$Result_Sim_Full, MEAS_DF$Result_Obs), na.rm = TRUE)))} +             ## For pH and WSE, changes y-range to min and max of the dataset, rather than 0 to max
    #scale_color_manual(values = plot_colors[-1]) +
    labs(title = paste0(unique(MODEL_DF$PCode), "\n Segment Name: ", unique(MODEL_DF$Plot_Title), "; Segment ID: ", unique(MODEL_DF$Segment_ID)),                                     ## Provides plot title 
       y = paste(unique(MODEL_DF$PCode), " (",  unique(MEAS_DF$Units), ")", sep = ""),                                                                                                  ## Provides a y-axis label
       x = "Date") +                                                                                                                                                                    ## Provides a x-axis label
  theme(
    plot.title= element_text(size = 12, hjust = 0.5, color = 'blue'),                                                      ## Justifies and changes font size of title
    panel.background = element_rect(fill = "white", color = "black"),                                                      ## Change the panel background and creates a black border
    text = element_text(size = 11),                                                                                        ## Resizes the text
    legend.position = "bottom",
    legend.title = element_blank(),
    plot.margin = margin(unit(c(15, 10, 10, 10), "cm")),                                                                   ## Resizes plot margins
    legend.margin = margin(unit(c(-10, 0, -1, 0), "cm")))                                                                  ## Adjusts legend margins
  }
}

#********************************************************************************************************************
#                             Creates basic 1:1 Linear Regression plot that will be called by plotly
#********************************************************************************************************************

PLOT_BASIC_REG <- function(MODEL_DF, MEAS_DF) {

  if(nrow(MEAS_DF) > 0) {
    
ggplot(data = MEAS_DF, aes(x = Result_Obs, y = Result_Sim)) +
  geom_point(aes(fill = factor(RCode, levels = c("Accepted", "Flagged"))), shape = 21, size = 2.5, alpha = 0.80, stroke = 0, color = "white", show.legend = TRUE) +                      ## Plots Simulated vs Observed data
  geom_abline(aes(intercept = 0, slope = 1, color = "1:1", linetype = "1:1"), linewidth = 1, show.legend = FALSE) +                                                   ## Adds 1:1 line with constant color, size, and type (allows legend mapping)
  geom_smooth(aes(color = "Linear Regression", linetype = "Linear Regression"), linewidth = 1, method = "lm", se = FALSE, formula = "y ~ x") +          ## Adds linear regression line with constant color, size, and type (allows legend mapping)
  scale_fill_manual(values = c("Accepted" = "dodgerblue", "Flagged" = "red"), drop = FALSE, name = "Remark Code") +                                                  ## Manually rescales fill color of datapoints (border is white)
  scale_color_manual(name = "Lines", values = c("Linear Regression" = "red", "1:1" = "blue")) +                                                                      ## Manually sets color legend based on crosswalk   
  scale_linetype_manual(name = "Lines", values = c("Linear Regression" = 2, "1:1" = 1)) +                                                                            ## Manually sets linetype legend based on crosswalk
  labs(title = paste0(unique(MEAS_DF$PCode), "\n Segment Name: ", unique(MEAS_DF$Plot_Title), "; Segment ID: ", unique(MEAS_DF$Segment_ID)),
       x = paste("Observed", unique(MEAS_DF$PCode), sep = " "),                                                                                                      ## Provides x-axis label
       y = paste("Simulated", unique(MEAS_DF$PCode), sep = " ")) +                                                                                                   ## Provides a y-axis label
  theme_bw() +
  theme(plot.title= element_text(size = 12, hjust = 0.5, color = 'blue'),   
    panel.background = element_rect(fill = "white", color = "black"),                                        ## Change the panel background and creates a black border
    text = element_text(size = 11),                                                                          ## Resizes the text
    legend.position = "bottom",
    plot.margin = margin(unit(c(15, 10, 10, 10), "cm")),
    legend.margin = margin(unit(c(-10, 0, -1, 0), "cm")),                                                    ## Positions the legend
    legend.title = element_blank(),                                                                          ## Drops legend title
    legend.key.width = unit(1, "cm"))                                                                        ## Widens legend key box to make lines easier to see

  }
}

# 
# temp_Meas <- NestData_Multi %>%
#   dplyr::mutate(Meas_Filt = map(Meas_Filt, ~dplyr::filter(.x, Segment_ID == SEGMENT)))                            ## Creates copy of Measured DF based on filtered Segment #
# } 
# 
# NestData_Multi %>%
#   dplyr::filter(Segment_ID == 150 & PCode == "FLOW_CMS") %>%
#   dplyr::mutate(Meas_Filt = map(Meas_Filt, ~dplyr::filter(.x, Segment_ID == 150))) %>%
#   select(Meas_Filt) %>%
#   unnest(Meas_Filt) %>%
#   select(Date_Time, PCode, Segment_ID, Plot_Title, Result_Obs, Result_Sim, Depth, Units) %>%                        ## Selects columns
#   #spread(Result_Sim) %>%                                                                                             ## Casts wide
#   gather(-Date_Time, -Segment_ID, -Plot_Title, -PCode, -Units, -Depth, key = "Type", value = "Result") %>%          ## Casts long to put all Results in a single column (format required by geom_ECDF), grouped by OBS vs Model Run
#   dplyr::mutate(Type = gsub("^Result_", "", Type)) %>%
#   PLOT_BASIC_ECDF(.)



#********************************************************************************************************************
#                             Creates basic ECDF plot that will be called by plotly
#********************************************************************************************************************

PLOT_BASIC_ECDF <- function(MEAS_DF) {
  
  if(nrow(MEAS_DF) > 0) {
    
  prob_brks <- c(0, 0.01, 0.05, 0.10, 0.20, 0.25, 0.40, 0.50, 0.60, 0.75, 0.80, 0.90, 0.95, 1)                                                        ## Sets up the percentile labels
  
  ggplot(MEAS_DF, aes(x = Result, color = Type)) +
    theme_bw() +                                                                                                                                      ## sets up the initial plot
    stat_ecdf(geom = "line", linewidth = 0.8, pad = FALSE, na.rm = TRUE, alpha = 0.7) +                                                                                 ## Plots the cumulative distribution curve
    coord_flip() +                                                                                                                                    ## Flips the X and Y axis to match WRDB graph format.  NOTE: I have to remember that Y is on the bottom and X is on the left
    scale_y_continuous(breaks = prob_brks, labels = prob_brks*100) +                                                                                  ## Sets the range and breakpoints for teh Y-axis (bottom) using a probability normal distribution
    {if(str_detect(str_to_upper(unique(MEAS_DF$PCode)), "^FLOW"))scale_x_log10()} +                                                                ## For FLOW, plots values based on log10scale
    scale_colour_manual(values = c("Obs" = "black", "Sim" = "red")) +
    labs(title = paste0(unique(MEAS_DF$PCode), "\n Segment Name: ", unique(MEAS_DF$Plot_Title), "; Segment ID: ", unique(MEAS_DF$Segment_ID)),      ## Adds plot title
         y = "Percent",                                                                                                                               ## Adds y-axis label (axes are flipped)
         x = paste(unique(MEAS_DF$PCode), " (",  unique(MEAS_DF$Units), ")", sep = "")) +                                                             ## Adds x-axis label (axes are flipped)
    theme(
      plot.title = element_text(size = 12, hjust = 0.5, color = 'blue'),                                                                              ## Formats title
      panel.border = (element_rect(fill = NA, color = "black")),                                                                                      ## Adjusts plot formatting
      panel.background = element_rect(fill = "white"),
      plot.margin = margin(unit(c(15, 10, 10, 10), "cm")),
      text = element_text(size = 11),                                                                                                                 ## Resizes the text
      legend.position = "bottom",
      legend.title = element_blank())                                                                                                                 ## Drops legend title
  }
}

#********************************************************************************************************************
#                             Custom function to plot time series based on EITHER Segment Name OR Segment Number
#                             
#            Requires three arguments: known PCODE, known SEGMENT, argument telling R to filter by Segment Name OR Segment Number
#            
#         Function includes several QA error checks to verify that appropriate Segment IDs are passed to the function
#********************************************************************************************************************

PLOTLY_TS <- function(PCODE, SEGMENT, PLOT_BY = c("NUMBER", "NAME") ) {
 
  if(length(PLOT_BY) != 1) {                                                                                                                                  ## Verifies that a PLOT_BY argument was supplied
    warning("No PLOT_BY argument supplied.  Will use NUMBER as default PLOT_BY argument. The PLOT_BY argument must be either NUMBER or NAME")
  }
  
  PLOT_BY <- str_to_upper(PLOT_BY)                                                                                                                            ## Converts PLOT_BY argument to ALLCAPS
  
  plot_by <- match.arg(PLOT_BY, c("NUMBER", "NAME"))
  
  if(!plot_by %in% c("NUMBER", "NAME")) {                                                                                                                     ## Verifies that a PLOT_BY argument is set to Name or Number
    stop("The PLOT_BY argument can only be set to either NUMBER or NAME.")
  }
  
  if(plot_by == "NUMBER" && !is.numeric(as.numeric(SEGMENT))) {
    stop("Plotting by segment number selected, but the supplied segment is not numeric.  Please supply a numeric value when plotting by segment number")                          ## If plotting by number, verifies that Segment argument is numeric
  }
  
  if(plot_by == "NAME" && !SEGMENT %in% unique(Station_Order_Lookup$Plot_Title)) {
    stop("Plotting by segment name selected, but supplied Plot_Title is not in Station Lookup table. Check spelling of supplied Plot_Title")                                      ## If plotting by name, verifies that Segment argument is in the lookup table Plot_Title
  }
  
  if(plot_by == "NUMBER" && !SEGMENT %in% unique(Station_Order_Lookup$Segment_ID)) {
    stop("Plotting by segment number selected, but supplied segment number is not in Station Lookup table. Verify that selected segment number is in the Station_Lookup table")   ## If plotting by number, verifies that Segument argument is in Station Lookup Segment IDs
  }
  
  ## If statment to filter either by Plot_Title or Segment number depending on the PLOT_BY argument
  
  if(plot_by == "NUMBER") {
    temp_Meas <- NestData_Multi %>%                                                                    ## Creates copy of Measured DF based on filtered Segment #
      dplyr::mutate(Meas_Filt = map(Meas_Filt, ~dplyr::filter(.x, Segment_ID == SEGMENT))) 

    temp_Model <- NestData_Multi %>%                                                                   ## Creates copy of Model DF based on filtered Segment #
      dplyr::mutate(Model_Filt = map(Model_Filt, ~dplyr::filter(.x, Segment_ID == SEGMENT)))
    
  } else {
    temp_Meas <- NestData_Multi %>%
      dplyr::mutate(Meas_Filt = map(Meas_Filt, ~dplyr::filter(.x, Plot_Title == SEGMENT)))             ## Creates copy of Measured DF based on filtered Segment NAME
  
    temp_Model <- NestData_Multi %>%
      dplyr::mutate(Model_Filt = map(Model_Filt, ~dplyr::filter(.x, Plot_Title == SEGMENT)))           ## Creates copy of MOdel DF based on filtered Segment NAME
    
    }
  
  if(PCODE %in% unique(temp_Meas$PCode)) {                                                                              ## IF statement to only run when PCODE in dataframe
    
  temp_Meas <- temp_Meas %>%
    dplyr::filter(PCode == PCODE) %>%                                                           ## Extracts PCODE of interest in Filtered Measured
    select(Meas_Filt) %>%                                                                       ## Drops other columns
    unnest(Meas_Filt) %>%                                                                       ## Unnests DF and converts to DF
    as.data.frame()
  
  temp_Model <- temp_Model %>%                                                              
    dplyr::filter(PCode == PCODE) %>%                                                           ## Extracts PCODE of interest in Filtered Model data
    select(Model_Filt) %>%                                                                      ## Drops other columns
    unnest(Model_Filt) %>%                                                                      ## Unnests and converts to DF
    as.data.frame()
  
  temp_TS <- PLOT_BASIC_TS(temp_Model, temp_Meas)                                                          ## Creates basis ggplot TS
  
  ggplotly(temp_TS, tooltip = c("x", "y"), dynamicTicks = TRUE) %>%   ## Converts to plotly TS
    plotly::layout(legend = list(orientation = "h", x = 0.2, y = -0.15)) 

  } else {warning(paste0(PCODE, " selected for segment ", SEGMENT, ", but missing from nested dataframe"))}                           ## Generates warning message when selected PCODE missing from dataframe
  
}



#********************************************************************************************************************
#                             Custom function to plot time series based on EITHER Segment Name OR Segment Number
#                             
#            Requires three arguments: known PCODE, known SEGMENT, argument telling R to filter by Segment Name OR Segment Number
#            
#         Function includes several QA error checks to verify that appropriate Segment IDs are passed to the function
#********************************************************************************************************************

PLOTLY_REG <- function(PCODE, SEGMENT, PLOT_BY = c("NUMBER", "NAME") ) {
  
  if(length(PLOT_BY) != 1) {                                                                                                                                  ## Verifies that a PLOT_BY argument was supplied
    warning("No PLOT_BY argument supplied.  Will use NUMBER as default PLOT_BY argument. The PLOT_BY argument must be either NUMBER or NAME")
  }
  
  PLOT_BY <- str_to_upper(PLOT_BY)                                                                                                                            ## Converts PLOT_BY argument to ALLCAPS
  
  plot_by <- match.arg(PLOT_BY)
  
  if(!plot_by %in% c("NUMBER", "NAME")) {                                                                                                                     ## Verifies that a PLOT_BY argument is set to Name or Number
    stop("The PLOT_BY argument can only be set to either NUMBER or NAME.")
  }
  
  if(plot_by == "NUMBER" && !is.numeric(as.numeric(SEGMENT))) {
    stop("Plotting by segment number selected, but the supplied segment is not numeric.  Please supply a numeric value when plotting by segment number")                          ## If plotting by number, verifies that Segment argument is numeric
  }
  
  if(plot_by == "NAME" && !SEGMENT %in% unique(Station_Order_Lookup$Plot_Title)) {
    stop("Plotting by segment name selected, but supplied Plot_Title is not in Station Lookup table. Check spelling of supplied Plot_Title")                                      ## If plotting by name, verifies that Segment argument is in the lookup table Plot_Title
  }
  
  if(plot_by == "NUMBER" && !SEGMENT %in% unique(Station_Order_Lookup$Segment_ID)) {
    stop("Plotting by segment number selected, but supplied segment number is not in Station Lookup table. Verify that selected segment number is in the Station_Lookup table")   ## If plotting by number, verifies that Segument argument is in Station Lookup Segment IDs
  }
  
  ## If statment to filter either by Plot_Title or Segment number depending on the PLOT_BY argument
  
  if(plot_by == "NUMBER") {
    temp_Meas <- NestData_Multi %>%                                                                ## Creates copy of Measured DF based on filtered Segment #
      dplyr::mutate(Meas_Filt = map(Meas_Filt, ~dplyr::filter(.x, Segment_ID == SEGMENT))) 
    
    temp_Model <- NestData_Multi %>%                                                               ## Creates copy of Model DF based on filtered Segment #
      dplyr::mutate(Model_Filt = map(Model_Filt, ~dplyr::filter(.x, Segment_ID == SEGMENT)))
    
  } else {
    temp_Meas <- NestData_Multi %>%
      dplyr::mutate(Meas_Filt = map(Meas_Filt, ~dplyr::filter(.x, Plot_Title == SEGMENT)))         ## Creates copy of Measured DF based on filtered Segment NAME
    
    temp_Model <- NestData_Multi %>%
      dplyr::mutate(Model_Filt = map(Model_Filt, ~dplyr::filter(.x, Plot_Title == SEGMENT)))       ## Creates copy of MOdel DF based on filtered Segment NAME
  }
  
  if(PCODE %in% unique(temp_Meas$PCode)) {                                                      ## IF statement to only run when PCODE in dataframe
    
  temp_Meas <- temp_Meas %>%
    dplyr::filter(PCode == PCODE) %>%                                                           ## Extracts PCODE of interest in Filtered Measured
    select(Meas_Filt) %>%                                                                       ## Drops other columns
    unnest(Meas_Filt) %>%                                                                       ## Unnests DF and converts to DF
    as.data.frame()
  
  temp_Model <- temp_Model %>%                                                              
    dplyr::filter(PCode == PCODE) %>%                                                           ## Extracts PCODE of interest in Filtered Model data
    select(Model_Filt) %>%                                                                      ## Drops other columns
    unnest(Model_Filt) %>%                                                                      ## Unnests and converts to DF
    as.data.frame()
  
  temp_REG <- PLOT_BASIC_REG(temp_Model, temp_Meas)                                             ## Creates basis ggplot TS
  
  ggplotly(temp_REG, tooltip = c("x", "y"), dynamicTicks = TRUE) %>%                            ## Converts to plotly TS
    plotly::layout(legend = list(orientation = "h", x = 0.2, y = -0.15))
  
  } else {warning(paste0(PCODE, " selected for segment ", SEGMENT, ", but missing from nested dataframe"))}                           ## Generates warning message when selected PCODE missing from dataframe
}



#********************************************************************************************************************
#                             Custom function to plot ECDF based on EITHER Segment Name OR Segment Number
#
#            Requires three arguments: known PCODE, known SEGMENT, argument telling R to filter by Segment Name OR Segment Number
#            
#            Function includes several QA error checks to verify that appropriate Segment IDs are passed to the function
#********************************************************************************************************************

PLOTLY_ECDF <- function(PCODE, SEGMENT, PLOT_BY = c("NUMBER", "NAME")) {
  
  if(length(PLOT_BY) != 1) {                                                                                                                                  ## Verifies that a PLOT_BY argument was supplied
    warning("No PLOT_BY argument supplied.  Will use NUMBER as default PLOT_BY argument. The PLOT_BY argument must be either NUMBER or NAME")
  }
  
  PLOT_BY <- str_to_upper(PLOT_BY)                                                                                                                            ## Converts PLOT_BY argument to ALLCAPS
  
  plot_by <- match.arg(PLOT_BY)
  
  if(!plot_by %in% c("NUMBER", "NAME")) {                                                                                                                     ## Verifies that a PLOT_BY argument is set to Name or Number
    stop("The PLOT_BY argument can only be set to either NUMBER or NAME.")
  }
  
  if(plot_by == "NUMBER" && !is.numeric(as.numeric(SEGMENT))) {
    stop("Plotting by segment number selected, but the supplied segment is not numeric.  Please supply a numeric value when plotting by segment number")                          ## If plotting by number, verifies that Segment argument is numeric
  }
  
  if(plot_by == "NAME" && !SEGMENT %in% unique(Station_Order_Lookup$Plot_Title)) {
    stop("Plotting by segment name selected, but supplied Plot_Title is not in Station Lookup table. Check spelling of supplied Plot_Title")                                      ## If plotting by name, verifies that Segment argument is in the lookup table Plot_Title
  }
  
  if(plot_by == "NUMBER" && !SEGMENT %in% unique(Station_Order_Lookup$Segment_ID)) {
    stop("Plotting by segment number selected, but supplied segment number is not in Station Lookup table. Verify that selected segment number is in the Station_Lookup table")   ## If plotting by number, verifies that Segument argument is in Station Lookup Segment IDs
  }
  
  
  ## If statment to filter either by Plot_Title or Segment number depending on the PLOT_BY argument
  
  if(plot_by == "NUMBER") {
    temp_Meas <- NestData_Multi %>%
      dplyr::mutate(Meas_Filt = map(Meas_Filt, ~dplyr::filter(.x, Segment_ID == SEGMENT)))                            ## Creates copy of Measured DF based on filtered Segment #
  } else {
    temp_Meas <- NestData_Multi %>%
      dplyr::mutate(Meas_Filt = map(Meas_Filt, ~dplyr::filter(.x, Plot_Title == SEGMENT)))                              ## Creates copy of Measured DF based on filtered Segment NAME 
  }
  
  if(PCODE %in% unique(temp_Meas$PCode)) {                                                                              ## IF statement to only run when PCODE in dataframe
  
  temp_Meas <- temp_Meas %>%
    dplyr::filter(PCode == PCODE) %>%                                                                                   ## Filters DF based on PCode
    select(Meas_Filt) %>%                                                                                               ## Keeps only filtered DF
    unnest(Meas_Filt) %>%                                                                                               ## Unnests and converts to DF
    as.data.frame() %>%
    select(Date_Time, PCode, Segment_ID, Plot_Title, Result_Obs, Result_Sim, Depth, Units) %>%                        ## Selects columns
    #spread(Result_Sim) %>%                                                                                             ## Casts wide
    gather(-Date_Time, -Segment_ID, -Plot_Title, -PCode, -Units, -Depth, key = "Type", value = "Result") %>%          ## Casts long to put all Results in a single column (format required by geom_ECDF), grouped by OBS vs Model Run
    dplyr::mutate(Type = gsub("^Result_", "", Type)) 
  
  temp_ECDF <- PLOT_BASIC_ECDF(temp_Meas)                                                                                            ## Creates ggplot ECDF
  
  temp_ECDF 
  
  ggplotly(temp_ECDF, tooltip = c("x", "y"), dynamicTicks = TRUE) %>%                                        ## Converts ggplot ECDF to plotly figure
    plotly::layout(legend = list(orientation = "h", x = 0.2, y = -0.15)) 
  } else {warning(paste0(PCODE, " selected for segment ", SEGMENT, ", but missing from nested dataframe"))}                           ## Generates warning message when selected PCODE missing from dataframe
  
}

#********************************************************************************************************************
#                             Custom function to create combined 2 panel ECDF and TS PLOTLY figure
#                             
#            Requires three arguments: known PCODE, known SEGMENT, argument telling R to filter by Segment Name OR Segment Number
#********************************************************************************************************************

PLOTLY_MULTI <- function(PCODE, Segment_ID, PLOT_BY = c("NUMBER", "NAME") ) {                                           
  subplot(PLOTLY_TS(PCODE, Segment_ID, PLOT_BY), PLOTLY_REG(PCODE, Segment_ID, PLOT_BY), PLOTLY_ECDF(PCODE, Segment_ID, PLOT_BY), nrows = 3, margin = c(0.02, 0.02, 0.02, 0.02)) 
    }


#********************************************************************************************************************
#                             Calls the various functions
#                             
#                 Functions require a PCODE, SEGMENT (NAME or NUMBER), and NUMBER/NAME argument to tell R how to filter DF
#********************************************************************************************************************

#PLOTLY_ECDF("DO", 1, "NUMBER")                                                                                          ## Creates PLOTLY ECDF figure

#PLOTLY_ECDF("FLOW_CMS", 16, "NUMBER")                                                                                     ## Creates PLOTLY ECDF figure
# 
# 
#PLOTLY_TS("FLOW_CMS", 16, "Number")                                                                                           ## Creates PLOTLY TS figure
# 
# PLOTLY_REG("FLOW_CMS", 16, "Number")   
# 
# PLOTLY_TS("NOX", 1, "NUMBER")
# 
# PLOTLY_MULTI("FLOW_CMS", 16, "Number")                                                                                    ## Create multipanel figure with TS, REG, and ECDF figure
# 
PLOTLY_REG("TN", 18, "Number")                                                                                    ## Create multipanel figure with TS, REG, and ECDF figure
PLOTLY_ECDF("TN", 18, "Number")                                                                                    ## Create multipanel figure with TS, REG, and ECDF figure
PLOTLY_TS("TN", 18, "Number")                                                                                    ## Create multipanel figure with TS, REG, and ECDF figure

#map(.x = c(16, 25, 36, 150, 159, 121, 125), ~PLOTLY_MULTI("FLOW_CMS", .x, "Number"))
 
PLOTLY_MULTI("TP", 18, "Number")


#********************************************************************************************************************
#                               Uses PMAP to plot multiple segments and Pcodes simultaneously
#********************************************************************************************************************

#pmap(list(c("TN", "NH3", "NOX"), 1, "Number"), PLOTLY_MULTI)                                                        ## Plot multiple PCODES for single SEGMENT

#pmap(list(c("TN"), c(16, 1), "Number"), PLOTLY_ECDF)                                                                    ## Plot multiple PCODES for single SEGMENT

#pmap(list("TN", c(1,4), "NUMBER"), PLOTLY_MULTI)                                                                      ## Plot multiple SEGMENTS for single PCODE

#pmap(list(rep(c("TN", "NH3", "NOX"), 2), c(rep(1, 3), rep(4, 3)), "Number"), PLOTLY_PAIRED)                         ## Plot multiple PCodes for multiple SEGMENTS
#
#
