#********************************************************************************************************************
#
#                   SOURCE Script to Calculate Summary Statistics and Goodness of Fit (hydroGOF) Statistics
#       
#              ****Statistics not calculated on full measured dataset; Use only Accepted Rcode records*****       
#
#                     Statistics are calculated independent of multipanel calibration figure
#                     
#         GoF Stats not calculated for Pcode/Segment combinations that have <2 records or SD = 0; Only report summary stats
#                          When a Pcode/Segment combination did not meet requirements of hydroGOF, it will report a NA in its place
#
#                     If no records were 'ACCEPTED' Rcodes, then all stats reported as NA  
#                         
#                         Input data needs to be preprocessed based on Proc_Source Script
#                              Designed to Run as SOURCE Script Called in Master_Script 
#                              
#                    Adjusted R square is manually calculated using a linear regression model                          
#
#********************************************************************************************************************



#********************************************************************************************************************
#                                   Vectors to be used in the FOR Loop
#********************************************************************************************************************

#********************************************************************************************************************
Calib_Segmnt <- unique(Station_Order_Lookup$Segment_ID)                                                                 ## Gets the vector of calibration segments based on WASP Segment IDs in the input dataframe
Calib_PCodes <- sort(unique(Measured_Paired$PCode))                                                                       ## Vector of PCodes to loop over

Output_GOF <- NULL                                                                                                        ## Create blank GOF Output table

#********************************************************************************************************************

#********************************************************************************************************************
#                                   Loop Over Calibration Segments and PCodes
#                                   
#                           Uses hydroGOF to calculate goodness of fit and summary statistics
#                           Statistics calculated with purrr, in case Loop converted into FUNX
#********************************************************************************************************************

for (j in Calib_Segmnt) {

    for (i in Calib_PCodes)  { 
    #********************************************************************************************************************
    #                                     Filter Data Prior to Plotting
    #********************************************************************************************************************
    Plot_Meas <- Measured_Paired %>%
      dplyr::filter(Segment_ID == j & PCode == i & 
                      (Date_Time >= min(Model_Intrp$Date_Time) & Date_Time <= max(Model_Intrp$Date_Time))) %>%                                             ## Subsets the measured data to only the model period
      select(Date_Time, PCode, Result_Obs, Result_Sim, Plot_Title, Segment_ID, RCode, Depth, Units) 
    
    
    
    #********************************************************************************************************************
    #                       Only calculate GoF stats if number of records > Minimum Number Records Threshold
    #                       
    #                       Initial threshold applied prior to exclusion of the flagged records.  
    #   However, after excluding flagged records, an additional IF statement will not calculate GoF stats when number of accepted records < 2 or have StDev == 0 (will only calculate summary stats)
    #                      If there are no accepted records after flagged records are excluded, no summary stats nor GoF stats are calculated
    #                                       Stats that were not calculated reported as NA
    #********************************************************************************************************************
    
    if(nrow(Plot_Meas) >= Min_Obs){                                                                                                                        ## To generate GoF Stats, a station needs this many calibration observations
      
      Plot_Meas <- Plot_Meas %>%
        dplyr::mutate(RCode = Rcode_Mapping$RCode_Flag[match(RCode, Rcode_Mapping$RCode)])                                                                 ## Maps Rcode to Rcode Flag
      
      Accept_Meas <- Plot_Meas %>%                                                                                                                         ## Creates new DF that only contains 'Accepted' records
        dplyr::filter(RCode != "Flagged") 

      #********************************************************************************************************************
      #                   If only a single ACCEPTED record or stdev of observed = 0, only calculate summary stats
      #********************************************************************************************************************
    
      if(nrow(Accept_Meas) < 2 || sd(Accept_Meas$Result_Obs) == 0){                                                                                             ## If statement to check > 1 accepted record and SD not equal to 0 (otherwise, hydroGoF will throw an error) 
        
        Obs_Total <- Plot_Meas %>%                                                                                                                              ## Based on full data (Flagged + Accepted), calculate total number of records
          group_nest(PCode, Segment_ID) %>%                                                                                                                   ## Group and nest DF (in case I turn this loop back into a FUNX)
          dplyr::mutate(Numb_Accepted = map(.x = data, ~count(as.data.frame(.x)) %>% rename(Numb_Obs_Total = n))) %>%                                           ## Map over nested dataframe to calculate total number of records (then restructure nested result)
          select(-data) %>%                                                                                                                                     ## Drop nested data
          unnest(Numb_Accepted) %>%                                                                                                                             ## Unnest accepted number of records
          gather(-PCode, -Segment_ID, key = "Metric", value = "Result")                                                                                       ## Cast long
        
        GOF_Meas <- Accept_Meas %>%                                                                                                                             ## Calculates GoF and Summary stat based on DF w/o flagged records
          ## This next line sets observed values == true 0 to 1E-20 to allow the scripts to calculate rNSE
          dplyr::mutate(Result_Obs = case_when(Result_Obs == 0 ~ as.numeric(1E-20),
                                               TRUE ~ Result_Obs)) %>%
          group_nest(PCode, Segment_ID) %>%                                                                                                                   ## Group and nest DF (in case I turn this loop back into a FUNX)
          ## The next two lines calculate summary stats (stat.desc) and quantiles, and then reshapes the nested stats.  Uses purrr and map in case I want to turn this back into a FUNX
          dplyr::mutate(Summary_Sim = map(.x = data, ~c(round(stat.desc(.$Result_Sim), 3), round(quantile(.$Result_Sim, c(0.1, 0.20, 0.50, 0.80, 0.90)), 2)) %>%       ## Calculates descriptive stats for SIM data
                                            t() %>%                                                                                                                    ## Transposes data
                                            t() %>%                                                                                                                    ## Transposes data again 
                                            as.data.frame() %>%                                                                                                        ## Converts to DF
                                            rownames_to_column(var = "Metric") %>%                                                                                     ## Adds Metric column based on rownames                     
                                            mutate(Metric = paste(Metric, "SIM", sep = "_")) %>%                                                                       ## Appends Metric name with _Sim                                   
                                            rename(Value = V1) %>%                                                                                                     ## Renames V1 column as Value     
                                            spread(key = Metric, value = Value))) %>%                                                                                  ## Casts DF wide                       
          dplyr::mutate(Summary_Obs = map(.x = data, ~c(round(stat.desc(.$Result_Obs), 3), round(quantile(.$Result_Obs, c(0.1, 0.20, 0.50, 0.80, 0.90)), 2)) %>%       ## Calculates descript stats for OBS data
                                            t() %>%                                                                                                                    ## Transposes data                                                                                                                    ## Calculates descr. stats for OBS data
                                            t() %>%                                                                                                                    ## Transposes data again
                                            as.data.frame() %>%                                                                                                        ## Converts to DF
                                            rownames_to_column(var = "Metric") %>%                                                                                     ## Adds metric column based on rownames
                                            mutate(Metric = paste(Metric, "OBS", sep = "_")) %>%                                                                       ## Appends metric name with _OBS
                                            rename(Value = V1) %>%                                                                                                     ## Renames V1 column as Value    
                                            spread(key = Metric, value = Value))) %>%                                                                                  ## Casts DF wide
          select(-data) %>%                                                                                                                                     ## Drop nested dataset
          unnest(c(Summary_Sim, Summary_Obs)) %>%                                                                                                               ## Unnests summary stats
          gather(-PCode, -Segment_ID, key = "Metric", value = "Result" ) %>%                                   
          select(PCode, Segment_ID,  Metric, Result) %>%
          arrange(PCode, Segment_ID) %>%
          mutate_if(is.factor, as.character) %>%                                                                                                                ## Changes factors to characters
          mutate(Result = as.numeric(Result)) %>%                                                                                                               ## Converts characters to numbers
          drop_na(., PCode, Segment_ID)                                                                                                                       ## For those Segments that had 0 accepted records, I drop their stats
        
        Output_GOF <- rbind(Output_GOF, GOF_Meas, Obs_Total)                                                                                                    ## combines blank DF with Summary Stats and Obs_Total
        
      } else {
        #********************************************************************************************************************
        #                                 If >1 accepted record and SD not equal to zero, calculate GoF stats and summary stats
        #********************************************************************************************************************
        
        Obs_Total <- Plot_Meas %>%                                                                                                                              ## Based on full data (Flagged + Accepted), calculate total number of records
          group_nest(PCode, Segment_ID) %>%                                                                                                                   ## Group and nest DF (in case I turn this loop back into a FUNX)
          dplyr::mutate(Numb_Accepted = map(.x = data, ~nrow(as.data.frame(.x)) %>% as.data.frame() %>% rename(Numb_Obs_Total = deparse(substitute(.))))) %>%   ## Map over nested dataframe to calculate total number of records (then restructure nested result)
          select(-data) %>%                                                                                                                                     ## Drop nested data
          unnest(Numb_Accepted) %>%                                                                                                                             ## Unnest accepted number of records
          gather(-PCode, -Segment_ID, key = "Metric", value = "Result")                                                                                       ## Cast long
        
        GOF_Meas <- Accept_Meas %>%                                                                                                                                         ## Calculates GoF and Summary stat based on DF w/o flagged records                                                                                     
          ## This next line sets observed values == true 0 to 1E-20 to allow the scripts to calculate rNSE
          dplyr::mutate(Result_Obs = case_when(Result_Obs == 0 ~ as.numeric(1E-20),
                                               TRUE ~ Result_Obs)) %>%
          group_nest(PCode, Segment_ID) %>%                                                                                                                               ## Group and nest DF (in case I turn this loop back into a FUNX)
          ## The next lines calculate GoF stats and Kling-Gupta, and then reshapes the nested stats.  Uses purrr and map in case I want to turn this back into a FUNX
          dplyr::mutate(GoF_Calc = map(.x = data, ~gof(.$Result_Sim, .$Result_Obs, do.spearman = TRUE, digits = 4, norm = "maxmin") %>%                                     ## Calculates goodness of fit stats
                                         as.data.frame %>%                                                                                                                  ## Converts to DF
                                         rownames_to_column(var = "Metric") %>%                                                                                             ## Adds metric name column based on rownames
                                         rename(Value = V1) %>%                                                                                                             ## Renames V1 column as Value
                                         spread(key = Metric, value = Value))) %>%                                                                                          ## Casts DF wide
          dplyr::mutate(KGE_Calc = map(.x = data, ~KGE(.$Result_Sim, .$Result_Obs, out.type = c("full"), method = "2012"))) %>%
          ## The next lines manually calculate adjusted R2 based on a linear regression model
          ## This can be used instead of r2 calculated from hydroGOF
          dplyr::mutate(R_Adj2 = map(.x = data, ~pluck(summary(lm(.$Result_Sim ~ .$Result_Obs)), "adj.r.squared") %>%                                                                                            ## Calculates kling gupta effic metric
                                      round(., 4) %>%                                                                                                                        ## Rounds value to 4 decimal places
                                      as.data.frame() %>%
                                      rename(R_Adj2 = "."))) %>%
          ## The next two lines calculate summary stats (stat.desc) and quantiles, and then reshapes the nested stats.  Uses purrr and map in case I want to turn this back into a FUNX
          dplyr::mutate(Summary_Sim = map(.x = data, ~c(round(stat.desc(.$Result_Sim), 3), round(quantile(.$Result_Sim, c(0.1, 0.20, 0.50, 0.80, 0.90)), 2)) %>%                         ## Calculates summary stats for SIM data (workflow as annotated above)
                                            t() %>% 
                                            t() %>% 
                                            as.data.frame() %>% 
                                            rownames_to_column(var = "Metric") %>% 
                                            mutate(Metric = paste(Metric, "SIM", sep = "_")) %>% 
                                            rename(Value = V1) %>% 
                                            spread(key = Metric, value = Value))) %>%
          dplyr::mutate(Summary_Obs = map(.x = data, ~c(round(stat.desc(.$Result_Obs), 3), round(quantile(.$Result_Obs, c(0.1, 0.20, 0.50, 0.80, 0.90)), 2)) %>%                         ## Calculates summary stats for OBS data (workflow as annotated above)
                                            t() %>% 
                                            t() %>%
                                            as.data.frame() %>% 
                                            rownames_to_column(var = "Metric") %>% 
                                            mutate(Metric = paste(Metric, "OBS", sep = "_")) %>% 
                                            rename(Value = V1) %>% 
                                            spread(key = Metric, value = Value))) %>%
          ## The next two lines extract modified Kling-Gupta and its elements, then reshapes the nested stats.  Uses purrr and map in case I want to turn this back into a FUNX
          dplyr::mutate(KGE_Value = map(.x = .$KGE_Calc, ~ pluck(.x, "KGE.value") %>%                                                                                            ## Calculates kling gupta effic metric
                                          round(., 4) %>%                                                                                                                        ## Rounds value to 4 decimal places
                                          as.data.frame() %>%                                                                                                                    ## Converts to DF     
                                          rename(mKGE = deparse(substitute(.))))) %>%                                                                                            ## Renames column by deparsing name
          dplyr::mutate(KGE_Elements = map(.x = .$KGE_Calc, ~ pluck(.x, "KGE.elements") %>%                                                                                      ## Calculates elements of Kling-Gupta Effic (correlation, beta, gamma)
                                             round(., 4) %>%                                                                                                                     ## Rounds to 4 decimal places
                                             as.data.frame() %>%                                                                                                                 ## Converts to DF
                                             rownames_to_column(var = "Metric") %>%                                                                                              ## Creates Metric name column based on rownames
                                             rename(Value = deparse(substitute(.))) %>%                                                                                          ## Renames column as Value based on deparsing  
                                             mutate(Metric = paste0("KGE.", Metric)) %>%                                                                                         ## Preprends Metric name with KGE.
                                             spread(key = Metric, value = Value))) %>%                                                                                           ## Casts DF wide
          select(-data, -KGE_Calc) %>%                                                                                                           
          unnest(c(GoF_Calc, R_Adj2, Summary_Sim, Summary_Obs, KGE_Value, KGE_Elements)) %>%                                                                                         ## Unnest nested stats DFs
          gather(-PCode, -Segment_ID, key = "Metric", value = "Result" ) %>%                                                                                               ## Reshapes and reformats the DF
          select(PCode, Segment_ID, Metric, Result) %>%
          arrange(PCode, Segment_ID) %>%
          mutate_if(is.factor, as.character) %>% 
          mutate(Result = as.numeric(Result))
        
        Output_GOF <- rbind(Output_GOF, GOF_Meas, Obs_Total)                                                                                                    ## combines blank DF with Summary Stats and Obs_Total
        
      }  
    }
  }
}

#********************************************************************************************************************
#                                     Reformats dataframe and adds missing Pcode/Segment combinations
#                                     
#                Missing stats are reported as NA values (i.e., when <2 records or StDev == 0; which throws an error with hydroGOF)
#                                                   If no records were ACCEPTED, all stats reported as NA
#********************************************************************************************************************

Full_Stats <- Output_GOF %>%
  spread(key = Metric, value = Result) %>%                                                                                ## Casts GOF DF wide format
  mutate(nbr.val_OBS = case_when(is.na(nbr.val_OBS) ~ 0,                                                                  ## When a segment/Pcode had no ACCEPTED records (NA), I convert the NA to a 0
                                 TRUE ~ nbr.val_OBS)) %>%                                                                 ## Otherwise, no change
  gather(., -PCode, -Segment_ID, key = "Metric", value = "Value") %>%                                                   ## Casts DF long
  dplyr::mutate(Metric = case_when(Metric == "nbr.val_OBS" ~ "Num Obs-Accepted",                                          ## Remaps these statistics
                                   Metric == "Numb_Obs_Total" ~ "Num Obs-Total",
                                   Metric == "mean_SIM" ~ "Avg_SIM",
                                   Metric == "mean_OBS" ~ "Avg_OBS",
                                   Metric == "var_SIM" ~ "Var_SIM",
                                   Metric == "var_OBS" ~ "Var_OBS",
                                   Metric == "R_Adj2" ~ "R.Adj\u00B2",
                                   Metric == "R2" ~ "R\u00B2",                                                            ## Uses unicode to add superscript 2
                                   TRUE ~ Metric)) %>%
  complete(nesting(Segment_ID), PCode, Metric) %>%                                                                             ## Adds any missing Pcode/Segment combination; assigns them a value of NA
  dplyr::mutate(Station_ID_Mrgd = Station_Order_Lookup$Station_ID_Mrgd[match(Segment_ID, Station_Order_Lookup$Segment_ID)],  ## Uses the WASP_Seg_ID Lookup to remap the WASP Segment ID to the Segment names
                Plot_Title = Station_Order_Lookup$Plot_Title[match(Segment_ID, Station_Order_Lookup$Segment_ID)]) %>%
  arrange(Segment_ID, PCode)

#********************************************************************************************************************
#                                                   Creates several vectors used to organize the DF
#                                             
#                                             Updated to extracted the adjusted R2 from the linear model rather than using the R2 from hydroGOF
#                                                   
#********************************************************************************************************************

GOF_Plot_Vec <- c("Num Obs-Total", "Num Obs-Accepted", "R.Adj\u00B2", "NSE", "RMSE", "NRMSE %", "d")                                                ## Vector of statistics that will be used in the Calibration figure GOF table
SumStat_Plot_Vec <- grep(c("^Avg|^10%|^20%|^50%|^80%|^90%"), unique(Full_Stats$Metric), value = TRUE)                                      ## Vector of statistics that will be used in the Calibration figure Summary stat table
GOF_Excel_Vec <- c("Num_Obs_Total", "Num_Obs_Accepted", "Avg_OBS", "Avg_SIM", "Var_OBS", "Var_SIM", "ME", "MAE", "RMSE", "NRMSE %", "R.Adj\u00B2", 
                   "rSpearman", "PBIAS %", "NSE", "d", "mKGE", "KGE.r", "KGE.Beta", "KGE.Gamma")                                          ## Creates a vector GOF statistics to extract and write out to Excel

Plot_SumStats <- Full_Stats %>%                                      
  dplyr::filter(Metric %in% SumStat_Plot_Vec) %>%                                                                                          ## Extracts Summary stats for the calibration plot GROB
  separate(., Metric, c("Statistic", "Dataset"), sep = "_" ) %>%                                                                           ## Splits the _OBS and _SIM from the metric name
  spread("Statistic", "Value") %>%                                                                                                         ## Casts DF wide
  mutate_at(vars(starts_with("Avg")), ~round(., digits = 3)) %>%                                                                           ## Rounds the average values to 3 places
  rename(Average = Avg) %>%                                                                                                                ## Rename Average column
  select(PCode, Segment_ID, Station_ID_Mrgd, Plot_Title, Dataset, Average, everything()) %>%                                             ## Reorders columns and records
  arrange(PCode, Segment_ID)

Plot_GOF <- Full_Stats %>%                                                                                                                 ## Extracts GOF stats for the calibration GOF GROB
  dplyr::filter(Metric %in% GOF_Plot_Vec) %>%                                                                                             
  spread("Metric", "Value") %>%
  gather(., -PCode, -Segment_ID, -Plot_Title, -Station_ID_Mrgd, key = "GoF_Metric", value = "Value") %>%
  dplyr::mutate(GoF_Metric = factor(GoF_Metric, levels = GOF_Plot_Vec)) %>%                                                                ## Orders GoF stats into custom order
  arrange(Segment_ID, PCode, GoF_Metric) 


#********************************************************************************************************************
#                                   Reformats GOF table for Excel
#********************************************************************************************************************
Calib_GOF_Wide <- Full_Stats %>%                                                                                             ## Casts full stats DF wide
  spread("Metric", "Value")

# ---------------------------Calculates a global average value for each GoF metric ---------------------------

Calib_GOF_Wide <- Calib_GOF_Wide %>%
  group_by(PCode) %>%                                                                                                        ## Groups the GoF output by metric 
  summarise_if(is.numeric,  mean, na.rm = TRUE) %>%                                                                          ## Calculates an average metric across all calibration segments (drops blanks)
  ungroup() %>%
  dplyr::mutate_if(is.numeric, round, 4) %>%                                                                                 ## Rounds all numeric columns to 2 decimal places 
  dplyr::mutate(Segment_ID = "Global_Avg",                                                                                 ## Assigns 'Global_Avg' as the WASP Segment ID and Name 
                Station_ID_Mrgd = "Global_Avg",
                Plot_Title = "Global_Avg") %>%
  rbind(Calib_GOF_Wide, .) %>%                                                                                               ## Binds the global average to full GoF data
  arrange(factor(Segment_ID, levels = unique(Station_Order_Lookup$Segment_ID)), PCode) %>%                               ## Sorts the dataframe
  rename(Num_Obs_Total = "Num Obs-Total", Num_Obs_Accepted = "Num Obs-Accepted") %>%
  .[c("Segment_ID", "Plot_Title", "Station_ID_Mrgd", "PCode", GOF_Excel_Vec)]                                              ## Reorders the columns based on Excel vector above 

Calib_GOF_Long <- Calib_GOF_Wide %>%
  gather(key = "Metric", value = "Result", -Segment_ID, -Plot_Title, -Station_ID_Mrgd, -PCode) %>%                         ## Reformats DF
  select(Segment_ID, Plot_Title, Station_ID_Mrgd, PCode, Metric, Result) %>%
  spread(PCode, Result) %>%
  arrange(factor(Segment_ID, levels = unique(Station_Order_Lookup$Segment_ID)), factor(Metric, levels = GOF_Excel_Vec))

Calib_Data_Den <- Calib_GOF_Long %>%                                                                                        ## Extracts total number of records (Accepted + Flagged) for a separate Worksheet tab
  dplyr::filter(Metric == "Num_Obs_Total") %>%
  select(-Segment_ID, -Station_ID_Mrgd) %>%
  dplyr::mutate(Metric = case_when(Metric == "Num_Obs_Total" ~ "Number Obs-Total"))

#********************************************************************************************************************
#                                    Creates blank Excel workbook to populate
#********************************************************************************************************************

wb_GOF <- openxlsx::createWorkbook()
wb_Pcode <- openxlsx::createWorkbook()                                                              ## Creates empty Excel workbook object

style_colnames <- createStyle(border = "TopBottomLeftRight",                                        ## Sets which borders to draw
                              borderStyle = "medium",                                               ## Sets border style
                              halign = "center",                                                    ## Sets text horiz alignment
                              fgFill = "#DAEEF3",                                                   ## Sets cell background
                              wrapText = TRUE,                                                      ## Wraps text
                              textDecoration = "bold")                                              ## Sets font to bold

style_bdy <- createStyle(border = "TopBottomLeftRight",                                             ## Sets which borders to draw
                         borderStyle = "thin",                                                      ## Sets border style                                 
                         halign = "center",                                                         ## Sets text horiz alignment
                         fgFill = NULL)                                                             ## Sets cell background

#********************************************************************************************************************
#                                    Writes GOF summary stats to Excel file
#********************************************************************************************************************

#  --------------------------- Builds the Wide Formatted GOF Sheet ---------------------------  

addWorksheet(wb_GOF, "GOF_Wide")                                                                                                                          ## Adds worksheet named by Pcode

writeData(wb_GOF, sheet = "GOF_Wide", Calib_GOF_Wide)                                                                                                     ## Writes temp DF to new worksheet

setColWidths(wb_GOF, sheet = "GOF_Wide", cols = c(1, 2, 3, 4:ncol(Calib_GOF_Wide)), widths = c(17, 45, 45, rep(14, times = (ncol(Calib_GOF_Wide) - 3))))  ## Sets widths of columns

addStyle(wb_GOF, sheet = "GOF_Wide", style_colnames, rows = 1, cols = 1:ncol(Calib_GOF_Wide))                                                             ## Sets style for colnames

addStyle(wb_GOF, sheet = "GOF_Wide", style_bdy, rows = 2:(nrow(Calib_GOF_Wide)+1), cols = 1:ncol(Calib_GOF_Wide), gridExpand = TRUE)                      ## Sets style body of table; skips first row


#  --------------------------- Builds the Long Formatted GOF Sheet ---------------------------

addWorksheet(wb_GOF, "GOF_Long")                                                                                                                          ## Adds worksheet named by Pcode

writeData(wb_GOF, sheet = "GOF_Long", Calib_GOF_Long)                                                                                                     ## Writes temp DF to new worksheet

setColWidths(wb_GOF, sheet = "GOF_Long", cols = c(1, 2, 3, 4:ncol(Calib_GOF_Long)), widths = c(17, 45, 45, rep(14, times = (ncol(Calib_GOF_Long) - 3))))  ## Sets widths of columns

addStyle(wb_GOF, sheet = "GOF_Long", style_colnames, rows = 1, cols = 1:ncol(Calib_GOF_Long))                                                             ## Sets style for colnames

addStyle(wb_GOF, sheet = "GOF_Long", style_bdy, rows = 2:(nrow(Calib_GOF_Long)+1), cols = 1:ncol(Calib_GOF_Long), gridExpand = TRUE)                      ## Sets style body of table; skips first row


#  --------------------------- Saves the GOF Summary Stats Excel File  ---------------------------

openxlsx::saveWorkbook(wb_GOF, paste(str_remove(Path_Data_Out, "/Proc_Out"), "/Stats/GOF_Smry_", Model_Run_Code, ".xlsx", sep = ""), overwrite = TRUE)                                                ## Saves the finished workbook


#********************************************************************************************************************
#                                    Create a summary Excel file with GOF statistics listed by PCode tab
#********************************************************************************************************************


#********************************************************************************************************************
#                                    Writes data availability tab to empty Pcode GOF workbook
#********************************************************************************************************************

addWorksheet(wb_Pcode, "Data_Avail")                                                                                                                     ## Adds worksheet named by Pcode

writeData(wb_Pcode, sheet = "Data_Avail", Calib_Data_Den)                                                                                                ## Writes temp DF to new worksheet

setColWidths(wb_Pcode, sheet = "Data_Avail", cols = 1:ncol(Calib_Data_Den), widths = c(45, 25, rep(14, times = ncol(Calib_Data_Den) - 2)))               ## Sets widths of columns

addStyle(wb_Pcode, sheet = "Data_Avail", style_colnames, rows = 1, cols = 1:ncol(Calib_Data_Den))                                                        ## Sets style for colnames

addStyle(wb_Pcode, sheet = "Data_Avail", style_bdy, rows = 2:(nrow(Calib_Data_Den) + 1), cols = 1:ncol(Calib_Data_Den), gridExpand = TRUE)               ## Sets style body of table; skips first row


#********************************************************************************************************************
#                                    Creates function to reformat GOF table for Pcode GOF Excel File
#********************************************************************************************************************


EXCEL_GOF_SUMMARY <- function(PCODE) {

  Met_Order <- c("Number Obs-Total", "Number Obs-Accepted", "Observed Mean", "Observed Variance", "Simulation Mean", 
                 "Simulation Variance", "Mean Error", "Mean Absolute Error",     
                 "RMSE", "NRMSE %", "R.Adj\u00B2", "Spearman Coeff.", "PBias", "Nash", "Index of Agreement",                                ## Uses unicode to add superscript 2
                 "Kling-Gupta Effic.", "Kling-Gupta Effic. Modified", "Kling-Gupta Pear. Coeff.", 
                 "Kling-Gupta Alpha (Ratio SD)", "Kling-Gupta Beta (Ratio Means)",
                 "Kling-Gupta Gamma (Ratio CV)", "Levene Test p-value", "Mann-Whitney U p-value")                                       ## Set order of Metric Rows
  
  temp <- Calib_GOF_Long %>%                        
    dplyr::mutate(Plot_Title = as.character(Plot_Title)) %>%
    select(Plot_Title, Metric, as.character(PCODE)) %>%
    drop_na(as.character(PCODE))                                                                            ## Drops stations w/o calculated metric
  
  col_order <- c("Metric", as.character(unique(temp$Plot_Title)))                                           ## Set order of columns
  
  temp <- temp %>%
    dplyr::mutate(Metric = case_when(Metric == "Num_Obs_Total" ~ "Number Obs-Total",                        ## Crosswalks the GOF Stat abbreviations
                                     Metric == "Num_Obs_Accepted" ~ "Number Obs-Accepted",
                                     Metric == "Avg_OBS" ~ "Observed Mean",
                                     Metric == "Var_OBS" ~ "Observed Variance",
                                     Metric == "Avg_SIM" ~ "Simulation Mean",
                                     Metric == "Var_SIM" ~ "Simulation Variance",
                                     Metric == "ME" ~ "Mean Error",
                                     Metric == "MAE" ~ "Mean Absolute Error",
                                     Metric == "RMSE" ~ "RMSE",
                                     Metric == "NRMSE %" ~ "NRMSE %",
                                     Metric == "R\u00B2" ~ "R\u00B2",                                      ## Uses unicode to add superscript 2
                                     Metric == "R.Adj\u00B2" ~ "R.Adj\u00B2",                                      ## Uses unicode to add superscript 2
                                     Metric == "rSpearman" ~ "Spearman Coeff.",
                                     Metric == "PBIAS %" ~ "PBias",
                                     Metric == "NSE" ~ "Nash",
                                     Metric == "d" ~ "Index of Agreement",
                                     Metric == "Levene.Pval" ~ "Levene Test p-value",
                                     Metric == "KGE" ~ "Kling-Gupta Effic.",
                                     Metric == "mKGE" ~ "Kling-Gupta Effic. Modified",
                                     Metric == "KGE.r" ~ "Kling-Gupta Pear. Coeff.",
                                     Metric == "KGE.Alpha" ~ "Kling-Gupta Alpha (Ratio SD)",
                                     Metric == "KGE.Beta" ~ "Kling-Gupta Beta (Ratio Means)",
                                     Metric == "KGE.Gamma" ~ "Kling-Gupta Gamma (Ratio CV)",
                                     Metric == "MannWhit.Pval" ~ "Mann-Whitney U p-value",
                                     TRUE ~ as.character(NA))) %>%
    spread(key = Plot_Title, value = as.character(PCODE), drop = TRUE) %>%
    arrange(factor(Metric, levels = Met_Order)) %>%                                                   ## Sort table by Metric name
    .[col_order] %>%                                                                                  ## Reorders columns
    dplyr::rename(Average = Global_Avg)
  
  
  #  --------------------------- Builds the Workbook Sheet by Sheet ---------------------------  
  
  addWorksheet(wb_Pcode, PCODE)                                                                                  ## Adds worksheet named by Pcode
  
  writeData(wb_Pcode, sheet = PCODE, temp)                                                                       ## Writes temp DF to new worksheet
  
  setColWidths(wb_Pcode, PCODE, cols = c(1, 2:ncol(temp)), widths = c(30, rep(14, times = ncol(temp) - 1)))      ## Sets widths of columns
  
  addStyle(wb_Pcode, PCODE, style_colnames, rows = 1, cols = 1:ncol(temp))                                       ## Sets style for colnames
  
  addStyle(wb_Pcode, PCODE, style_bdy, rows = 2:(nrow(temp)+1), cols = 1:ncol(temp), gridExpand = TRUE)          ## Sets style body of table; skips first row
  
}

#********************************************************************************************************************
#                                    Reformats GOF table for Excel
#********************************************************************************************************************

walk(.x = unique(Calib_GOF_Wide$PCode), EXCEL_GOF_SUMMARY)                                                       ## Calls the function

#  --------------------------- Saves the PCode GOF Excel File  ---------------------------

openxlsx::saveWorkbook(wb_Pcode, paste(Path_Data_Out, "/Stats/PCode_GOF_", Model_Run_Code, ".xlsx", sep = ""), overwrite = TRUE)  ## Saves the finished workbook for PCode table





#********************************************************************************************************************
#                               Writes out duplicate GoF wide table as .csv
#                                    
#                                    Easier to read into other software
#********************************************************************************************************************

write.csv(Calib_GOF_Wide, paste(Path_Data_Out, "/Stats/GOF_WideSmry_", Model_Run_Code, ".csv", sep =""), row.names = FALSE)

