

#********************************************************************************************************************
#
#          Script to download instantaneous flow, preprocess it, calculate daily statistics, and hourly average flow
#          
#       Will write out WRDB Results tables that report hourly average flow, and also 4 records per day (based on original instantaneous records)
#       
#                                  Will generate average hourly time series plots
#                                  
#                                  
#                           Uses IF statement to skip this subscript when no instantaneous data are available
#                                                      
#********************************************************************************************************************

#********************************************************************************************************************
#                     Progress Statement
#********************************************************************************************************************


print("**************************************************************************************************************")
print("                               Downloading instantaneous FLOW data")
print("**************************************************************************************************************")



#********************************************************************************************************************
#                          Use custom function to create required subfolders for plotting time series data
#           This section verifies that the required subfolders exist and creates them when they are missing
#           
#           HRLY_FLW_PLOT argument will create the subfolder to contain the hourly average flow calculations
#           
#********************************************************************************************************************


walk(.x = c("HRLY_FLW_PLOT", "HRLY_WRDB"), CREATE_SUBFOLDER)                                          ## Calls custom function to create missing subfolderes


#********************************************************************************************************************
#                                     Downloads station metadata 
#********************************************************************************************************************

Keep_Instant_FlwMeta <- readNWISsite(Keep_Gage_Dwnld)                                                                                ## Downloads station metadata from NWIS


#********************************************************************************************************************
#                          Downloads the original instantaneous flow data using dataRetrieval
#           
#********************************************************************************************************************

Keep_USGS_Instant_Flows_Raw <- readNWISuv(siteNumbers = Keep_Gage_Dwnld, 
                                          parameterCd = "00060",                                                                                       ## Downloads instantaneous flow data
                                          startDate = Keep_Query_Dates[1],                                                                             ## Sets start date of query
                                          endDate = Keep_Query_Dates[2],                                                                               ## Sets end data of query
                                          tz = Keep_Model_TZone)                                                                                       ## Applies time zone correction to data


#********************************************************************************************************************
#               IF Statement to only run the processing/plotting scripts when the gages have instantaneous flows available to download
#               
#                            Will not process the flows when no gages have instantaneous flows
#           
#********************************************************************************************************************


if (nrow(Keep_USGS_Instant_Flows_Raw) == 0) {
  message ("Selected gages had no instantaneous flow records for the date range selected; No flow data were downloaded or processed")                                    ## Returns an error when data available for gages
} else {
  
  #********************************************************************************************************************
  #                          Writes out the original RAW instantaneous flow data before any reformatting
  #           
  #********************************************************************************************************************
  
  write.table(Keep_USGS_Instant_Flows_Raw, file = paste("./Raw_Data/USGS_Instant_Flw_Raw_", format(Sys.Date(), "%y%m%d"), ".csv", sep = ""), row.names = FALSE, col.names = TRUE, sep = ",")
  
 
  
  #********************************************************************************************************************
  #                             IF statement to change time to model time zone, if USGS data in UTC
  #********************************************************************************************************************
  
  if (unique(Keep_USGS_Instant_Flows_Raw$tz_cd) == "UTC"){
    Keep_USGS_Instant_Flows_Raw <- Keep_USGS_Instant_Flows_Raw %>%
      dplyr::mutate(Date_TZCorr = ymd_hms(dateTime)) %>%                                                                                                         ## Creates new date_time
      dplyr::mutate(Date_TZCorr = with_tz(Date_TZCorr, tzone = Keep_Model_TZone, truncated = 3 ))                                                                               ## Adjusts time if original tz = UTC
  } else {
    Keep_USGS_Instant_Flows_Raw <- Keep_USGS_Instant_Flows_Raw %>%
      dplyr::mutate(Date_TZCorr = ymd_hms(dateTime, tz = Keep_Model_TZone, truncated = 3))                                                                                      ## Creates new data_time and attaches time zone
  }
  
  
  #********************************************************************************************************************
  #                                     Echo of variables used in NWIS data query
  #********************************************************************************************************************
  
  Instant_Flow_Query_Echo <- cbind(USGS_Queried = str_flatten(Keep_Gage_Dwnld, collapse = "; "),                                                        ## Extracts gages used in query
                                   USGS_Gages_w_Data = str_flatten(pluck(attributes(Keep_USGS_Instant_Flows_Raw)$siteInfo, "site_no"), collapse = "; "),   ## Extracts gages w data
                                   Date_of_Query = pluck(attributes(Keep_USGS_Instant_Flows_Raw), "queryTime"),                                            ## Extracts date queried executed
                                   Date_Range_Queried = str_flatten(Keep_Query_Dates, collapse = "; "),                                                      ## Extrats date range of data queried
                                   pluck(attributes(Keep_USGS_Instant_Flows_Raw), "variableInfo"),                                                         ## Extracts Pcodes w data
                                   pluck(attributes(Keep_USGS_Instant_Flows_Raw), "statisticInfo"))                                                        ## Extracts measurement type
  
  
  
  #********************************************************************************************************************
  #                                     Writes out the query echo file and metadata 
  #********************************************************************************************************************
  write.table(Instant_Flow_Query_Echo, file = paste("./QA_Check/Instant_FlwQuery_Echo_", format(Sys.Date(), "%y%m%d"), ".csv", sep = ""), row.names = FALSE, col.names = TRUE, sep = ",")
  write.table(Keep_Instant_FlwMeta, file = paste("./Raw_Data/USGS_Instant_FlwSiteData_", format(Sys.Date(), "%y%m%d"), ".csv", sep = ""), row.names = FALSE, col.names = TRUE, sep = ",")
  
  
  
  #********************************************************************************************************************
  #                            Reformats station data into lookup table
  #********************************************************************************************************************
  
  Keep_Station_Instant_FlwLookup <- Keep_Instant_FlwMeta %>%
    select(Station_ID = site_no, Station_Name = station_nm, state_cd,                                                           ## Selects and renames columns
           Latitude = dec_lat_va, Longitude = dec_long_va, Elevation_ft = alt_va, 
           HUC = huc_cd, Dr_Area_sq_mi = drain_area_va, Vert_Datum = alt_datum_cd, 
           Datum = dec_coord_datum_cd, Agency = agency_cd) %>%
    dplyr::mutate(State = countyCd$STUSAB[match(state_cd, countyCd$STATE)],                                                     ## Crosswalks the state code with the state abbreviation
                  Station_ID = str_pad(Station_ID, 8, "left", pad = "0"),                                                              ## Adds the leading 0 to gage number
                  HUC = str_pad(HUC, 8, side = "left", pad = 0)) %>%                                                                   ## Adds the leading 0 back into the HUC
    dplyr::mutate(Station_ID = paste("USGS", Station_ID, sep = "-"),
                  Station_Name = str_to_title(Station_Name)) %>%
    select(-state_cd)
  
  
  #********************************************************************************************************************
  #                                          Reformats instantaneous flow data
  #********************************************************************************************************************
  
  Keep_USGS_Instant_Flows_Reformt <- Keep_USGS_Instant_Flows_Raw %>%
    rename(Agency = agency_cd, Station_ID = site_no, Date = dateTime, FLOW_INST_CFS = X_00060_00000, Rcode = X_00060_00000_cd) %>%              ## Renames columns
    #dplyr::mutate(Date = ymd(Date), Lcode = format(Sys.Date(), "%m%d%y")) %>%                                                                     ## Adds LCode and sets it to the date when scripts were run
    distinct(Agency, Station_ID, Date_TZCorr, tz_cd, .keep_all = TRUE) %>%                                                                         ## Removes duplicate records based on station, date-time, and agency
    dplyr::mutate(FLOW_INST_CMS = FLOW_INST_CFS * 0.028316847) %>%                                                                           ## Creats new flow that converts CFS to CMS
    select(Agency, Station_ID, Date, Date_TZCorr, FLOW_INST_CFS, FLOW_INST_CMS, Rcode) %>%                                                   ## Selects columns
    dplyr::mutate(Station_ID = str_pad(Station_ID, 8, "left", pad = "0")) %>%                                                                      ## Because excel strips leading zeros, I have to add them back in
    dplyr::mutate(Station_ID = paste("USGS", Station_ID, sep = "-")) 
  
  #********************************************************************************************************************
  #                                          Prints statement when duplicate records found
  #********************************************************************************************************************
  
  print(paste0(nrow(Keep_USGS_Instant_Flows_Raw) - nrow(Keep_USGS_Instant_Flows_Reformt), " duplicate records were found in the original instantaneous flow records.  Records compared based on Station, Agency, Date_Time, and Time Zone. Duplicate records were removed")) 
  
  #********************************************************************************************************************
  #                                   Casts Raw Data into Long Dataframe
  #                                   
  #                      This can be used to calculate hour averages and daily statistics
  #********************************************************************************************************************
  
  Keep_Instant_Flow <- Keep_USGS_Instant_Flows_Reformt %>%  
    select_at(., vars(Agency, Station_ID, Date_TZCorr, ends_with(c("CFS", "CMS", "Rcode")))) %>%                            ## Extracts the Rcode columns and Results columns
    gather(-Agency, -Station_ID, -Date_TZCorr, -Rcode, key = "Pcode", value = "Result", na.rm = TRUE) %>%                   ## Gathers the DF long, dropping missing records
    mutate(Ccode = case_when(str_detect(Pcode, "_CMS") == TRUE ~ "CAL",                                                     ## Reports Ccode = CAL when Flow in CMS
                             TRUE ~ NA_character_)) %>%
    mutate(Pcode = str_to_upper(Pcode)) %>%                                                                                 ## Casts Pcode to uppercase
    select(Agency, Station_ID, Date_TZCorr, Pcode, Result, Rcode, Ccode) %>%                                                ## Reorders and selects columns
    arrange(Station_ID, Pcode, Date_TZCorr)
  
  #********************************************************************************************************************
  #                                          Calculates annual summary statistics
  #********************************************************************************************************************
  
  Keep_Instant_Flow_Yr_Stats <- Keep_Instant_Flow %>%                                                                   ## Calculates annual summary statistics by gage
    dplyr::mutate(Year = year(Date_TZCorr), Month = month(Date_TZCorr)) %>%                                             ## Adds year and month columns
    group_by(Agency, Station_ID, Pcode, Year) %>%                                                                       ## Group by year
    summarize(N = n(), First_Record = min(Date_TZCorr), Last_Record = max(Date_TZCorr),                                 ## Reports number of records per year, and date of first and last record
              Instant_Avg_Flow = mean(Result),                                                                          ## Calculates annual average
              Instant_Max_Flow = max(Result), Instant_Min_Flow = min(Result), .groups = "keep") %>%                                       ## Calculates annual min and max
    as.data.frame() 
  
  #********************************************************************************************************************
  #                             Calculates daily average, daily min, and daily max flow based on full instantaneous flows
  #********************************************************************************************************************
  
  Keep_Instant_Flow_Dly_Stats <- Keep_Instant_Flow %>%                                                                                   ## Calculates daily summary statistics by gage
    dplyr::mutate(Year = year(Date_TZCorr), Month = month(Date_TZCorr), Day = day(Date_TZCorr)) %>%                                      ## Adds Year, Month, and Day column
    group_by(Agency, Station_ID, Pcode, Year, Month, Day) %>%                                                                            ## Group by year, month, and day
    summarize(N = n(), Dly_Avg_Flow = mean(Result),                                                                                      ## Calculates number records per day and daily average
              Dly_Max_Flow = max(Result), Dly_Min_Flow = min(Result), .groups = "keep") %>%                                                                ## Calculates  min and max for each day
    mutate(Date_TZCorr = make_datetime(year = Year, month = Month, day = Day, hour = 12L, min = 0L, tz = Keep_Model_TZone)) %>%          ## Renerates Date_Time based on year, month, day, and hour = 12; applies time zone
    mutate(Pcode = str_replace(Pcode, "INST", "DLY")) %>%                                                                             ## Replaces INSTANT with DLY
    as.data.frame() %>%
    select(-Year, -Month, -Day) %>%                                                                                                      ## Drops columns
    select(Agency, Station_ID, Date_TZCorr, Pcode, N, Dly_Avg_Flow, Dly_Max_Flow, Dly_Min_Flow)                                          ## Reorders columns
  
  #********************************************************************************************************************
  #                             Calculates average hourly, min flow, and max flow based on full instantaneous flows
  #********************************************************************************************************************
  
  Keep_Instant_Flow_Hrly_Stats <- Keep_Instant_Flow %>%                                                                                  ## Calculates annual summary statistics by gage
    dplyr::mutate(Year = year(Date_TZCorr), Month = month(Date_TZCorr), Day = day(Date_TZCorr), Hour = hour(Date_TZCorr)) %>%            ## Adds Year, Month, Day, and Hour column
    group_by(Agency, Station_ID, Pcode, Year, Month, Day, Hour) %>%                                                                      ## Groups data by year, month, day and hour
    summarize(N = n(), Hrly_Avg_Flow = mean(Result),                                                                                         ## Calculates number of records and hourly average
              Hrly_Max_Flow = max(Result), Hrly_Min_Flow = min(Result), .groups = "keep") %>%                                                              ## Calculates min/max for each hour
    mutate(Date_TZCorr = make_datetime(year = Year, month = Month, day = Day, hour = Hour, min = 0L, tz = Keep_Model_TZone)) %>%         ## REgenerates Date_Time based on year, mont, day, hour, and min = 0; applies time zone
    mutate(Pcode = str_replace(Pcode, "INST", "HRLYAVG")) %>%                                                                         ## Replaces INSTANT with AVGHRLY
    as.data.frame() %>%
    select(-Year, -Month, -Day, -Hour) %>%                                                                                               ## Drops and reorders columns
    select(Agency, Station_ID, Date_TZCorr, Pcode, N, Hrly_Avg_Flow, Hrly_Max_Flow, Hrly_Min_Flow)
  
  #********************************************************************************************************************
  #                            Reformats Hrly Data for WRDB
  #********************************************************************************************************************
  
  Keep_Instant_HrlyAvg_Flow <- Keep_Instant_Flow_Hrly_Stats %>%
    select(Agency, Station_ID, Date_TZCorr, Pcode, Result = Hrly_Avg_Flow) %>%                                                               ## Reorders columns
    mutate(Ccode = "CAL")                                                                                                                ## Adds CCode = CAL for all average hourly values
  
  #********************************************************************************************************************
  #                                   Creates DF of four records per day (midnight, 6am, noon, 6pm)
  #                                   
  #                            Based on the original instantaneous flow records, does not use the average hourly data
  #********************************************************************************************************************
  
  ## This section creates a new date_time based on the date of each USGS record (midnight, 6am, noon, 6pm)
  ## It will then cast the DF long, so I have 4 time stamps for each day a USGS record was measured
  ## In the next section, it uses Find_Interval and a lookup match based on this DF and compares it to the measured NWIS records to extract the NWIS sample that was collected at 
  ## midnight, 6am, noon, and 6pm on that sampling date. Midnight is considered the start of the day.
  ## If no sample was collected exactly at those times, it will extract the next record that was sampled after that time stamp (e.g., 6:15 am)
  ## I remove duplicate records when SONDE records are only collected a couple times of day. 
  
  Flw_Four_Rcrds_Dy <- Keep_Instant_Flow %>%
    select(Agency, Station_ID, Date_TZCorr)                                                                                   ## Creates DF for filtering records
  
  
  Flw_Four_Rcrds_Dy$Midnight <- make_datetime(year = year(Flw_Four_Rcrds_Dy$Date_TZCorr),                                     ## Creates date_time at midnight (Midnight considered start of the day)
                                              month = month(Flw_Four_Rcrds_Dy$Date_TZCorr), 
                                              day = day(Flw_Four_Rcrds_Dy$Date_TZCorr), 
                                              hour = 0L, 
                                              min = 0L, 
                                              sec = 0, 
                                              tz = Keep_Model_TZone)
  
  
  Flw_Four_Rcrds_Dy$AM <- make_datetime(year = year(Flw_Four_Rcrds_Dy$Date_TZCorr),                                            ## Creates date_time at 6am
                                        month = month(Flw_Four_Rcrds_Dy$Date_TZCorr), 
                                        day = day(Flw_Four_Rcrds_Dy$Date_TZCorr), 
                                        hour = 6L, 
                                        min = 0L, 
                                        sec = 0, 
                                        tz = Keep_Model_TZone)
  
  Flw_Four_Rcrds_Dy$Noon <- make_datetime(year = year(Flw_Four_Rcrds_Dy$Date_TZCorr),                                           ## Creates date_time at noon
                                          month = month(Flw_Four_Rcrds_Dy$Date_TZCorr), 
                                          day = day(Flw_Four_Rcrds_Dy$Date_TZCorr),
                                          hour = 12L, 
                                          min = 0L, 
                                          sec = 0, 
                                          tz = Keep_Model_TZone)
  
  Flw_Four_Rcrds_Dy$PM <- make_datetime(year = year(Flw_Four_Rcrds_Dy$Date_TZCorr),                                             ## Creates date_time at 6pm
                                        month = month(Flw_Four_Rcrds_Dy$Date_TZCorr), 
                                        day = day(Flw_Four_Rcrds_Dy$Date_TZCorr), 
                                        hour = 18L, 
                                        min = 0L, 
                                        sec = 0, 
                                        tz = Keep_Model_TZone)
  
  Flw_Four_Rcrds_DeDup <- Flw_Four_Rcrds_Dy %>%                                                                                 ## Reformats the new DF
    select(-Date_TZCorr) %>%                                                                                                    ## Drops Original Date_Time
    gather(-Agency, -Station_ID, key = "Time_of_Day", value  = "Time_Subset") %>%                                               ## Gather DF long for easier joining
    distinct(Agency, Station_ID, Time_of_Day, Time_Subset) %>%                                                                  ## Drops duplicates
    arrange(Station_ID, Time_Subset)
  
  #********************************************************************************************************************
  #                                   For loop to extract gage records four records per day
  #********************************************************************************************************************
  
  ## This section does the actual extraction of the paired NWIS records
  
  Flw_Rcrds_DF <- NULL                                                                                        ## Creates empty DF for LOOP
  
  for (i in unique(Flw_Four_Rcrds_DeDup$Station_ID)){
    temp_extract <- Flw_Four_Rcrds_DeDup %>%                                                                  ## Extracts gage i from DF with 4 records per day
      dplyr::filter(Station_ID == i)
    
    temp_gage <- Keep_Instant_Flow %>%                                                                        ## Extracts gage i from measured DF
      select(Agency, Station_ID, Date_TZCorr, Pcode, Result, Rcode) %>%
      spread(Pcode, Result) %>%                                                                               ## Casts DF wide, so CMS and CFS records on same line (avoids duplicating effort)
      dplyr::filter(Station_ID == i) %>%
      arrange(Date_TZCorr)
    
    ## FindInterval extracts the rowID where 4Record per day matches the measured date_time.  It then uses that rowID to extract the measured record line by line.  
    temp_paired <-  temp_gage[findInterval(temp_extract$Time_Subset, temp_gage$Date_TZCorr),]             ## Uses FindInterval to find the record at or immediately after 4 record per day
    
    temp_paired <- temp_paired %>%                                                                        ## Removes duplicates (e.g., Measured record at 1/1/2018 12:00 and 1/1/2018 12:02 will both have FindInterval record at noon)
      distinct() %>%
      dplyr::mutate(Dup_Date = floor_date(Date_TZCorr, unit = "hour")) %>%                                ## Finds previous record based on Date_Time
      distinct(., Agency, Station_ID, Dup_Date, .keep_all = TRUE)
    
    Flw_Rcrds_DF <- bind_rows(Flw_Rcrds_DF, temp_paired)                                                          ## Binds extracted DF to empty DF
    
  }
  
  #********************************************************************************************************************
  #                                   Reformats the filtered USGS records into a long format 4 records per day
  #                         
  #********************************************************************************************************************
  
  Keep_Instant_Flw_Subset <- Flw_Rcrds_DF %>%                                                                                                
    select(-Dup_Date) %>%                                                                                                                    ## Drops columns
    gather(-Agency, -Station_ID, -Date_TZCorr, -Rcode, key = "Pcode", value = "Result", na.rm = TRUE)                                        ## Gathers DF long
  
  #********************************************************************************************************************
  #                                   Generates Pcode table for average hourly flow and instantaneous FLOWS
  #                                   
  #                             This distinguishes hourly, instantaneous flows from the Avg Daily flows that were processed with other scripts
  #                         
  #********************************************************************************************************************
  
  Keep_Instant_Flow_Pcode_Table <- distinct(Keep_Instant_Flw_Subset, Pcode)  %>%                                                          ## Finds unique Pcodes in DF
    rbind(., distinct(Keep_Instant_HrlyAvg_Flow, Pcode)) %>%
    dplyr::mutate(Anal_Name = case_when(Pcode == "FLOW_INST_CFS" ~ "Instantaneous Flow",                                               ## Maps the pcode to an analyte description
                                        Pcode == "FLOW_INST_CMS" ~ "Instantaneous Flow",                                               ## Maps the pcode to an analyte description
                                        Pcode == "FLOW_HRLYAVG_CFS" ~ "Hourly Average Flow",                                              ## Maps the pcode to an analyte description
                                        Pcode == "FLOW_HRLYAVG_CMS" ~ "Hourly Average Flow",                                              ## Maps the pcode to an analyte description
                                        TRUE ~ as.character(NA))) %>%
    dplyr::mutate(Units = case_when(Pcode == "FLOW_INST_CFS" ~ "CFS",                                                                  ## Maps the pcode to its units
                                    Pcode == "FLOW_INST_CMS" ~ "CMS",                                                                  ## Maps the pcode to its units
                                    Pcode == "FLOW_HRLYAVG_CFS" ~ "CFS",                                                                  
                                    Pcode == "FLOW_HRLYAVG_CMS" ~ "CMS",
                                    TRUE ~ as.character(NA)))                                                                             
  
  
  #********************************************************************************************************************
  #                                          Creates Ccode table
  #********************************************************************************************************************
  
  Keep_HrlyAvg_Flow_Ccode_Table <- distinct(Keep_Instant_HrlyAvg_Flow, Ccode) %>%                                                       ## Adds the Rcodes that were in the data
    dplyr::filter(!is.na(Ccode)) %>%
    dplyr::mutate(Ccode_Description = case_when(Ccode == "CAL" ~ "Calculated value",
                                                TRUE ~ as.character(NA))) 
  
  
  #********************************************************************************************************************
  #                                          Writes out Annual, Daily, and Hourly Flow summary statistics
  #                                          
  #                       These are based on the instantaneous flows, not the average daily flows that were downloaded elsewhere
  #********************************************************************************************************************
  
  wb_Replc_Blnk <- createWorkbook()                                                                                                                                ## Create blank Excel workbook object
  
  walk2(.x = list(Keep_Instant_Flow_Yr_Stats,                                                                                                                            ## Writes out annual stats based on instantaneous flows
                  Keep_Instant_Flow_Dly_Stats %>% rename(Date_Time = Date_TZCorr),                                                                                       ## Writes out daily stats
                  Keep_Instant_Flow_Hrly_Stats %>% rename(Date_Time = Date_TZCorr)),                                                                                     ## Writes out hourly stats
        .y = list("Instant_Flow_Annual_Stats", "Flow_Daily_Stats", "Flow_Hourly_Long"), ~POPULATE_EXCEL_WORKBOOK(.x, .y))                                                ## Uses map and custom funx to populate workbook
  
  openxlsx::saveWorkbook(wb_Replc_Blnk, paste0("./Stats/USGS_Inst_Flw_Stats_", format(Sys.Date(), "%y%m%d"), ".xlsx"), overwrite = TRUE)                              ## Saves the finished workbook
  
  #********************************************************************************************************************
  #                                          Writes out WRDB Ready 4x Day Filtered Flow Records & Hourly Average Flow Records
  #********************************************************************************************************************
  
  write.table(Keep_Instant_Flw_Subset %>% rename(Date_Time = Date_TZCorr), file = paste("./WRDB_Other/USGS_Inst_Flw_Result_Subset_", format(Sys.Date(), "%y%m%d"), ".csv", sep = ""), 
              row.names = FALSE, col.names = TRUE, sep = ",", na = "")
  
  write.table(Keep_Instant_HrlyAvg_Flow %>% rename(Date_Time = Date_TZCorr), file = paste("./WRDB_Other/USGS_HrlyAvg_Flw_Result_", format(Sys.Date(), "%y%m%d"), ".csv", sep = ""), 
              row.names = FALSE, col.names = TRUE, sep = ",", na = "")
  
  write.table(Keep_Instant_Flow_Pcode_Table, file = paste("./WRDB_Other/USGS_Inst_Flw_Pcode_", format(Sys.Date(), "%y%m%d"), ".csv", sep = ""), 
              row.names = FALSE, col.names = TRUE, sep = ",", na = "")
  
  write.table(Keep_HrlyAvg_Flow_Ccode_Table, file = paste("./WRDB_Other/USGS_HrlyAvg_Flw_Ccode_", format(Sys.Date(), "%y%m%d"), ".csv", sep = ""), 
              row.names = FALSE, col.names = TRUE, sep = ",", na = "")
  
  write.table(Keep_Station_Instant_FlwLookup, file = paste("./WRDB_Other/USGS_Inst_Flw_Station_", format(Sys.Date(), "%y%m%d"), ".csv", sep = ""), 
              row.names = FALSE, col.names = TRUE, sep = ",", na = "")
  
  
  #********************************************************************************************************************
  #                            Creates function to generate multiple time series plots: one plot is in CMS and the second is in  CFS
  #                               Plot adds records for missing days, which keeps ggplot from interpolating across datagaps
  #********************************************************************************************************************
  
  PLOT_HRLY_FLOW <- function(DATA)   {                                                                                                     ## Creates function arguments to call dataframe and vector of gage ID
    
    temp_dy <- DATA %>%                                                                                                                    ## Uses measured DF to create blank DF on daily time step that will be used w/ anti_join to find days with no measured value                                                                   
      mutate(Date = make_datetime(year(Date), month(Date), day(Date), hour(Date)),                                                         ## Converts Date_Time into Date only
             Result = NA)  %>%                                                                                                             ## Sets Results column to blank
      distinct(Station_ID, Pcode, Date, .keep_all = TRUE)                                                                                  ## Drops duplicates, so there is only a single record per day
    
    temp_blnk <- expand.grid(Station_ID = unique(DATA$Station_ID),                                                                         ## Creates blank DF the contains all hours within between first and last day of query date
                             Pcode = unique(DATA$Pcode),                                                                                   ## Sets Station_ID and Pcode based on nested DF
                             Date = seq.POSIXt(as.POSIXct(force_tz(min(Keep_Query_Dates), tzone = Keep_Model_TZone)),                
                                               as.POSIXct(force_tz(max(Keep_Query_Dates), tzone = Keep_Model_TZone)), by = "hour"),        ## Sets date range to entire query date, on an hourly time step (forces timezone to model time zone)
                             Result = NA,                                                                                                  ## Sets Results column to blank
                             Units = unique(DATA$Units),                                                                                   ## Adds Units
                             Station_Name = unique(DATA$Station_Name),                                                                     ## Adds Station_Name
                             stringsAsFactors = FALSE) %>%                                                    
      dplyr::mutate(Date = ymd_hms(Date, tz = Keep_Model_TZone, truncated = 3)) %>%                                                                      ## Converts date/time
      anti_join(., temp_dy, by = c("Station_ID", "Pcode", "Date" )) %>%                                                                   ## Uses daily DF from above to find hourly records missing from measured values
      #dplyr::mutate(Date = ymd_hms(Date, truncated = 3, tz = Keep_Model_TZone)) %>%                                                      ## Converts date
      bind_rows(., DATA) %>%                                                                                                              ## Adds missing records to original measured data
      arrange(Station_ID, Pcode, Date)                                                                                                    ## Sorts data
    
    ggplot()+  
      geom_line(data = temp_blnk, aes(x = as.POSIXct(Date), 
                                      y = Result), linewidth = 0.4, linetype = 1, color = "blue", na.rm = TRUE) +                      ## Creates lineplot of time series
      geom_hline(aes(yintercept = 0), color = "black", linetype = 3)+                                                             ## Adds horizontal line at origin
      #scale_x_datetime(date_labels = "%b\n%Y", limits = as.Date(Keep_Query_Dates), date_breaks = "1 year") +                     ## Formats Date axis
      labs(title = paste(temp_blnk$Station_ID,"\n ", temp_blnk$Station_Name, "\n Hourly Average Flow", sep = ""),                 ## Adds title, y-axis, and x-axis labels                                       
           y = paste(temp_blnk$Pcode," (", temp_blnk$Units, ")", sep = ""),       
           x = "Date",
           caption = paste("Date of NWIS query:", format(Sys.Date(), "%m-%d-%Y"), sep = " ")) +    
      theme(
        plot.title = element_text(size = 8, hjust = 0.5),                                                                          ## Justifies and changes font size of title
        panel.background = element_rect(fill = "white", color = "black"),                                                          ## Makes the panel background white with a black border
        panel.grid.major = element_blank(),                                                                                        ## Removes major gridlines
        panel.grid.minor = element_blank(),                                                                                        ## Removes minor gridlines
        plot.caption = element_text(size = 5),
        text = element_text(size = 8),
        axis.text.x = element_text(hjust = 0.5),                                                                                   ## Moves labels closer to plot
        plot.margin = margin(t = 0.25, b = 0.25, r = 0.50,  l = 0.25, unit = "in"),
        legend.position="none")
    
  }
  
  
  #********************************************************************************************************************
  #                            Calls function to plot multiple time series plots
  #********************************************************************************************************************
  
  Keep_Instant_HrlyAvg_Flow %>%
    rename(Date = Date_TZCorr) %>%                                                                                               ## Renames Date_Time column
    mutate(Units = case_when(Pcode == "FLOW_HRLYAVG_CMS" ~ "CMS",                                                                ## Maps units based on the pcode
                             Pcode == "FLOW_HRLYAVG_CFS" ~ "CFS",       
                             TRUE ~ NA_character_)) %>%
    mutate(Station_Name = Keep_Station_Instant_FlwLookup$Station_Name[match(Station_ID, Keep_Station_Instant_FlwLookup$Station_ID)]) %>%         ## Adds Station_Name to DF
    group_nest(Station_ID, Pcode, Units, keep = TRUE) %>%                                                                        ## Group_Nest to allow use of MAP
    mutate(Filename = paste0("./Plot_HrlyTS/", str_remove(Station_ID, "USGS-"), "_", Pcode, ".jpg")) %>%                         ## Generate filename/filepath for saved plot
    mutate(Plot = map(.x = data, ~PLOT_HRLY_FLOW(.x))) %>%                                                                       ## Call plotting function to generate the figure
    {walk2(.x = .$Filename, .y = .$Plot, ~ggsave(.x, .y, width = 7, height = 4, units = "in", dpi = 300))}                       ## Call WALK2 as an expression to call ggsave for each plot
  
}


#********************************************************************************************************************
#                     Exports entire Gage Height time series
#********************************************************************************************************************

write.table(Keep_Instant_Flow %>% rename(Date_Time = Date_TZCorr), file = paste("./WRDB_Other/USGS_Inst_Flw_Result_Full_", format(Sys.Date(), "%y%m%d"), ".csv", sep = ""), row.names = FALSE, col.names = TRUE, sep = ",", na = "")


#********************************************************************************************************************
#                     Progress Statement
#********************************************************************************************************************


print("**************************************************************************************************************")
print("                  Scripts completed the downloading & plotting of instantaneous FLOW data")
print("**************************************************************************************************************")
