
#********************************************************************************************************************
#              SOURCE script to pair Measured Record to Model Record and Linear Interpolates Result_Sim at Exact Time Stamp of Measured Data Record
#                             
#                   This script directly reads the Model datafile from BMD.UTIL export and a Measured datafile from WRDB
#                             
#               **** When activated, this version converts 4 X day simulated flow records into an average daily flow for model flow (will average 4 records per day into a single value) ******
#                                               
#                                              This version uses FindInterval and na.approx interpolation 
#        
#              The Paired Calib File uses the Measured DF as a Starting Point and then Populates the Necessary Simulated Columns
#              
#                 Conceptually, for each observed data record, it finds the simulated data record that occurs immediately before 
#                    DateTime of the observed record (i.e., Sim_Pre) and then finds Sim_Post by taking the next simulated record.
#           It then uses NA.APPROX in conjunction with the Sim_Pre and Sim_Post result values and date_times to linearly interpolate the missing values
#      
#      This is a faster implementation of the origina manual method based on the DateTime of the Sim_Post and DateTime of the Observed Data, it calculates the time difference to exact minute.
#                 An interpolated Sim value is calculated by subtracting the product of the Slope and Delta from Sim_Post.
#                             
#                             
#                             Slope = (Sim_Post - Sim_Pre) / (Sim_DateTime_Post- Sim_DateTime_Pre)
#                                    Delt = Sim_DateTime_Post - Obs_DataTime_Date
#                                 Intrp_Sim = Sim_Post - (Slope * Delt)
#
#********************************************************************************************************************


#********************************************************************************************************************
#                                             Import datasets
#               Currently uses read.csv because FORTRAN extractor embeds NULLS in column header that throws fread error
#********************************************************************************************************************

if(length(setdiff(c(Model_Extracted_Filename),   list.files(path = Path_Model_In, pattern = ".txt$"))) != 0) {                                                 ## Validates that model input datafile exists in the specified filepath properly named       
  stop(paste0("The extracted model input data file is not correctly named or in wrong filepath  Filepath = ", Path_Model_In,                                                                                                              ## Error / ends script
              "\n Filename of extracted model input data = ", Model_Extracted_Filename))                                                           
} else { 
  
  #Model_Raw <- read.csv(paste(Path_Model_In, Model_Extracted_Filename, sep = "/"), sep = "\t",  quote = '"',                                                   ## Reads in the WASP output that was extracted by FORTRAN exe
  #                      header = TRUE, stringsAsFactors = FALSE, na.strings = "", skipNul = TRUE, strip.white = TRUE)
  
  ### Ignore next line unless fix embedded null error in FORTRAN generated file
  
  Model_Raw <- fread(paste(Path_Model_In, Model_Extracted_Filename, sep = "/"), sep = "\t",  quote = '"', 
                     header = TRUE, stringsAsFactors = FALSE, na.strings = "", strip.white = TRUE, check.names = TRUE, data.table = FALSE)               ## Read in the WASP output that was extracted using BMDUtil
  
  Measured_Raw <- fread(paste(Path_Calib_In, RawCalib_Filename, sep = "/"), quote = '"', sep = "\t", 
                        header = TRUE, stringsAsFactors = FALSE, na.strings = "", strip.white = TRUE, check.names = TRUE, data.table = FALSE)             ## Read in the Measured Data that was exported from WRDB
  
  
  #********************************************************************************************************************
  #                                        QA Raw Measured and Raw Datafiles Correctly Imported
  #********************************************************************************************************************
  if (length(grep("_Raw$", ls(), value = TRUE)) != 2) {                                                                                                    ## Verifies that DF correctly imported
    stop(paste("Raw Measured and/or Model input data files found, but error during import; Check data structure, filename, and path. 
       \nRaw Model Filename =", RawBMD_Filename, "and Raw Measured Filename =", RawCalib_Filename, sep = " "))                                          ## Error / ends script
  } else { 
    
    
    ############# ERROR CHECK when a calibration station is missing from Measured Data
    # Measured_Raw <- Measured_Raw %>%
    #   filter(Station_ID != "03298470")
    #######################
    
    
    #********************************************************************************************************************
    #                           QA Station and PCode Lookup Tables Have Required Columns
    #********************************************************************************************************************
    Col_SLook <- c("Calibrate", "Segment_ID", "Segment_Name", "Calib_Station_ID", "Plot_Title", "Plot_Order")
    Col_PLook <- c("Calibrate", "PCode", "Units", "Anal_Name")
    
    if (length(setdiff(Col_SLook, names(Stat_Lookup))) != 0){
      stop(paste(paste(setdiff(Col_SLook, names(Stat_Lookup)), collapse = "; "), "column(s) is missing from station lookup table"))      ## Error / ends script if Measured DF is missing a required column
    }else {
      if (length(setdiff(Col_PLook, names(PCode_Lookup))) != 0){
        stop(paste(paste(setdiff(Col_PLook, names(PCode_Lookup)), collapse = "; "), "column(s) is missing from PCode lookup table "))           ## Error / ends script if Model DF is missing a required column
      }else {
        
        #********************************************************************************************************************
        #                               Select Calibration Segments and Parameters to Plot
        #********************************************************************************************************************
        Stat_Lookup <- Stat_Lookup %>%
          arrange(., Plot_Order) %>%
          dplyr::mutate(Calibrate = toupper(Calibrate)) %>%                                                                   ## Converts Xs to capital
          dplyr::mutate(Segment_Name = gsub("_", "-", Segment_Name)) %>%                                                      ## Replaces _ with -
          dplyr::mutate(Segment_Name = gsub(" ", "-", Segment_Name)) %>%                                                      ## Replaces spaces with -
          dplyr::mutate(Calib_Station_ID = toupper(Calib_Station_ID)) %>%                                                     ## Sets Calib_Station_ID to all capital letters
          dplyr::mutate(Calib_Station_ID = gsub("_", "-", Calib_Station_ID)) %>%                                              ## Replaces _ with -
          dplyr::mutate(Calib_Station_ID = gsub(" ", "-", Calib_Station_ID)) %>%                                              ## Replaces spaces with "-"                                                                         
          drop_na(., Calibrate)
        
        PCode_Lookup <- PCode_Lookup %>%
          dplyr::mutate(Calibrate = toupper(Calibrate)) %>%                                                                  
          drop_na(., Calibrate) %>%
          dplyr::mutate(Anal_Name = str_trim(Anal_Name, side = "both")) %>%                                                   ## Strips white space
          dplyr::mutate(Anal_Name = gsub("[[:punct:][:space:]]", ".", Anal_Name))                                             ## Changes spaces and punctuation into periods 
          
          
        #********************************************************************************************************************
        #                                        QA Measured and Model Files Contain Records
        #********************************************************************************************************************
        
        #### Error check for no data in Raw Datafiles
        #Model_Raw <- Model_Raw %>%
        #  dplyr::filter(Station_ID == "FAKE_STATION")
        
        #  Measured_Raw <- Measured_Raw %>%
        #  dplyr::filter(Station_ID == "FAKE_STATION")
        
        # Stat_Lookup <- Stat_Lookup %>%
        #  dplyr::filter(Calib_Station_ID == "FAKE_STATION")
        
        # PCode_Lookup <- PCode_Lookup %>%
        #  dplyr::filter(PCode == "FAKE_PCODE")
        
        if(nrow(Measured_Raw) == 0){                                                               ## Error check / ends script if Measured DF contains no records
          stop("Raw measured dataframe contains no data records; Check datafile")
        }else {
          if(nrow(Model_Raw) == 0) {                                                               ## Error check / ends script if Model DF contains no records
            stop("Raw model dataframe contains no data records; Check datafile")
          }else {                                                                                  ## Runs entire script if DF contain data records
            if(nrow(Stat_Lookup) == 0){                                                               ## Error check / ends script if Station Lookup is empty
              stop("Station lookup table is empty or no stations selected for calibration; Check datafile")
            }else {
              if(nrow(PCode_Lookup) == 0) {                                                               ## Error check / ends script if PCode Lookup is empty
                stop("Pcode lookup table is empty or no parameters selected for calibration; Check datafile")
              }else {  
                
                #********************************************************************************************************************
                #                           QA Measured and Model Dataframes Contain Columns Required for Scripts
                #********************************************************************************************************************
                Col_RawMeas <- c("Date_Time", "Station_ID", "PCode", "Result", "RCode", "Depth_M")
                Col_RawMod <- c("Date_Time", "Station_ID")                                                          ## Model DF columns required in scripts
                
                if (length(setdiff(Col_RawMeas, names(Measured_Raw))) != 0){
                  stop(paste(paste(setdiff(Col_RawMeas, names(Measured_Raw)), collapse = "; "), "column(s) is missing from input Measured dataframe"))      ## Error / ends script if Measured DF is missing a required column
                }else {
                  if (length(setdiff(Col_RawMod, names(Model_Raw))) != 0){
                    stop(paste(paste(setdiff(Col_RawMod, names(Model_Raw)), collapse = "; "), "column(s) is missing from input Model dataframe"))           ## Error / ends script if Model DF is missing a required column
                  }else {
                    PCode_Columns <- names(Model_Raw[ , !(colnames(Model_Raw) %in% c(Col_RawMod)), drop = FALSE])    ## Check that MODEL data in wide format
                    if(length(intersect(PCode_Lookup$Anal_Name, PCode_Columns)) == 0) {
                      stop("Model dataframe has no data columns for any parameter selected in PCode_Lookup. 
                       \n Verify that raw model input data formatted wide and has column names that exactly match the Anal_Name column in PCode Lookup")
                    }else {
                      
                      #********************************************************************************************************************
                      #                                                 Segment/Station Order Lookup
                      #********************************************************************************************************************
                      Seg_Order <- unique(fct_inorder(as.factor(Stat_Lookup$Segment_ID)))
                      
                      Station_Order_Lookup <- distinct(Stat_Lookup, Calib_Station_ID, Segment_ID) %>%                                                                    ## Extracts combination of Station_Name and Segment_IDs
                        dplyr::mutate(Row_ID = 1:nrow(.)) %>%                                                                                                              ## Adds unique ID for spreading data
                        spread(Row_ID, Calib_Station_ID, fill = "") %>%                                                                                                    ## Spreads DF wide
                        unite(Station_ID_Mrgd, c(2:ncol(.)), sep = "/", remove = TRUE) %>%                                                                                 ## Concatenates Station Names into a Merged Name
                        dplyr::mutate(Station_ID_Mrgd = gsub("/+", "_", Station_ID_Mrgd),                                                                                  ## Replaces forward slash with "_" 
                                      Station_ID_Mrgd = gsub("^_|_$", "", Station_ID_Mrgd)) %>%                                                                           
                        left_join(., distinct(Stat_Lookup, Segment_ID, Segment_Name, Calib_Station_ID, Plot_Title, Plot_Order, Lat, Long), by = "Segment_ID", multiple = "all") %>%                ## Repopulates the original Station Name and Station_ID; allows for multiple matches
                        arrange(as.numeric(Plot_Order)) %>%
                        dplyr::mutate(Segment_ID = fct_inorder(factor(Segment_ID)),                                                                                    ## Converts WASP Segment to factor for plot order
                                      Station_ID_Mrgd = fct_inorder(as.factor(Station_ID_Mrgd)))                                                                           ## Sets order of merged name based on WASP order
                      
                      
                      #********************************************************************************************************************
                      #                          Sets range of simulated data model period based on the first and last record in the datafile
                      #                                       Used to filter measured data & modeled data
                      #********************************************************************************************************************
                      
                      Model_Period <- range(mdy_hms(head(Model_Raw$Date_Time, n = 1), tz = Model_TZ, truncated = 3), mdy_hms(tail(Model_Raw$Date_Time, n = 1), tz = Model_TZ, truncated = 3))      ## Sets simulate period date/time range based on first and last time stamp in the Raw Data                                                                                                           ## Sets simulation period date/time range 
                      
                      
                      #********************************************************************************************************************
                      #                                       Preprocesses and standardizes Raw Model Data
                      #********************************************************************************************************************
                      
                      Model_Intrp <- Model_Raw %>%
                        dplyr::rename(Segment_Name = Station_ID) %>%                                                                                                                 ## Renames WASP Station ID to Segment_Name
                        dplyr::mutate(Segment_Name = gsub("_", "-", Segment_Name)) %>%                                                                                               ## Replaces "_" with "-" in Stat_ID
                        dplyr::mutate(Segment_Name = gsub(" ", "-", Segment_Name)) %>%                                                                                               ## Formats the date as POSIX, records missing h:m:s set to midnightS
                        dplyr::mutate(Date_Time = mdy_hms(Date_Time, tz = Model_TZ, truncated = 3)) %>%
                        dplyr::filter(Date_Time >= min(Model_Period) & Date_Time <= max(Model_Period)) %>%                                                                           ## Trims Data to Time Period of Model Period
                        ## dplyr::filter(Date_Time >= mdy("01-01-2018") & Date_Time <= mdy("12-31-2019")) %>%                                                                        ## Allows manual subset of model period
                        left_join(., distinct(Station_Order_Lookup, Segment_ID, Segment_Name, Station_ID_Mrgd, Plot_Title, Plot_Order), by = ("Segment_Name" = "Segment_Name"), multiple = "all") %>%     ## Adds missing columns; allows for multiple pairings
                        dplyr::filter(Segment_ID %in% Stat_Lookup$Segment_ID) %>%                                                                                                    ## Drops any WASP segment that isn't a calibration Segment
                        select(Date_Time, Segment_ID, Segment_Name, Station_ID_Mrgd, Plot_Title, Plot_Order, everything()) %>%                                                       ## Reorders and selects columns
                        gather(key = "Anal_Name", value = "Result", -Date_Time, -Segment_ID, -Segment_Name, -Station_ID_Mrgd, -Plot_Title, -Plot_Order) %>%                          ## Casts DF into long format
                        dplyr::filter(Anal_Name %in% unique(PCode_Lookup$Anal_Name)) %>% 
                        left_join(PCode_Lookup, by = c("Anal_Name" = "Anal_Name")) %>%                                                                                               ## Maps the Parameter Name to PCode
                        select(-Calibrate) %>%
                        arrange(PCode, Segment_ID, Date_Time)                                                                                                                        ## Sorts DF
                       
                      
                      #********************************************************************************************************************
                      #                           For WASP output, Converts instantaneous Model Flow into Average Daily Flow
                      #                           
                      #              An IF statement will only do this conversion when FLOW is selected AND Convert_flow_to_AvgDaily is selected
                      #                           
                      #            If flow is reported out four times a day, this will calculate the average daily flow and set the date time to noon
                      #            
                      #                         Noon is used because that is how the NWIS scripts report the average daily flow
                      #
                      #                                 No other parameters are aggregated; only flow when it is selected
                      #********************************************************************************************************************
                      
                     if (all(Model_Type == "WASP", "Flow.Out.of.Segment" %in% unique(PCode_Lookup$Anal_Name), "CONVERT_FLOW_TO_AVGDAILY" %in% Plot_Types)) {                                          ## Only applies this section when FLOW is selected AND convert to average daily flow is activated 
                        
                        Flow_Pcode <- PCode_Lookup$PCode[PCode_Lookup$Anal_Name == "Flow.Out.of.Segment"]                                                                   ## Extracts custom Pcode used for flow
                        
                        print(paste0("Simulated ", Flow_Pcode, " records were aggregated on a daily basis to calculate an average daily simulated flow with a time stamp set to noon; Aggregation occurred prior to pairing with measured flow records"))
                        
                        Model_Intrp <- Model_Intrp %>%                                                                                                                      ## Filters the flow records from the model data
                        dplyr::filter(PCode == Flow_Pcode) %>%
                        mutate(Date_Time = make_datetime(year = year(Date_Time), month = month(Date_Time), day = day(Date_Time), hour = 12, min = 0, tz = Model_TZ)) %>%    ## Replaces Date_Time with same day, but all time set to noon
                        group_by(Segment_ID, Segment_Name, Station_ID_Mrgd, Plot_Title, Plot_Order, Anal_Name, PCode, Units, Date_Time) %>%                                  ## Groups the data by day 
                        summarize(Result = mean(Result)) %>%                                                                                                                ## Calculates average daily flow
                        ungroup() %>%                                                                                                                                       ## Ungroups DF
                        bind_rows(., Model_Intrp %>% dplyr::filter(PCode != Flow_Pcode)) %>%                                                                                   ## Binds average daily flow to the original Model data w/o FLOW data
                        arrange(PCode, Segment_ID, Date_Time)
                      }
                      
                      #********************************************************************************************************************
                      #                           For WASP/EFDC output, Converts instantaneous Water Surface Elevation to Average Daily WSE
                      #                           
                      #              An IF statement will only do this conversion when WSE is selected AND Convert_flow_to_AvgDaily is selected
                      #                           
                      #            If flow is reported out four times a day, this will calculate the average daily flow and set the date time to ****MIDNIGHT****
                      #            
                      #                          is used because that is how the NWIS scripts report the average daily flow
                      #
                      #                                 No other parameters are aggregated; only WSE when it is selected
                      #********************************************************************************************************************
                      
                      if (all(Model_Type %in% c("EFDC", "WASP"), "Surface.Elev" %in% unique(PCode_Lookup$Anal_Name), "CONVERT_FLOW_TO_AVGDAILY" %in% Plot_Types)) {                                          ## Only applies this section when WSE is selected AND convert to average daily flow is activated 
                        
                        WSE_Pcode <- PCode_Lookup$PCode[PCode_Lookup$Anal_Name == "Surface.Elev"]                                                                            ## Extracts custom Pcode used for WSE
                        
                        print(paste0("Simulated ", WSE_Pcode, " records were aggregated on a daily basis to calculate an average daily simulated Water Surface Elevation with a time stamp set to midnight; Aggregation occurred prior to pairing with measured flow records"))
                        
                        Model_Intrp <- Model_Intrp %>%                                                                                                                       ## Filters the WSE records from the model data
                          dplyr::filter(PCode == WSE_Pcode) %>%
                          mutate(Date_Time = make_datetime(year = year(Date_Time), month = month(Date_Time), day = day(Date_Time), hour = 0, min = 0, tz = Model_TZ)) %>%    ## Replaces Date_Time with same day, but all time set to MIDNIGHT
                          group_by(Segment_ID, Segment_Name, Station_ID_Mrgd, Plot_Title, Plot_Order, Anal_Name, PCode, Units, Date_Time) %>%                                ## Groups the data by day 
                          summarize(Result = mean(Result)) %>%                                                                                                               ## Calculates average daily WSE
                          ungroup() %>%                                                                                                                                      ## Ungroups DF
                          bind_rows(., Model_Intrp %>% dplyr::filter(PCode != WSE_Pcode)) %>%                                                                                ## Binds average daily WSE to the original Model data w/o WSE data
                          arrange(PCode, Segment_ID, Date_Time)
                      }
                      
                      
                      
                      #********************************************************************************************************************
                      #                           For LSPC output, Converts instantaneous Model Flow into Average Daily Flow
                      #                           
                      #              An IF statement will only do this conversion when FLOW is selected AND Convert_flow_to_AvgDaily is selected
                      #                           
                      #            If flow is reported out four times a day, this will calculate the average daily flow and set the date time to noon
                      #            
                      #                         Noon is used because that is how the NWIS scripts report the average daily flow
                      #
                      #                                 No other parameters are aggregated; only flow when it is selected
                      #********************************************************************************************************************
                      
                      
                      
                      if (all(Model_Type == "LSPC", ("RO" %in% unique(PCode_Lookup$LSPC_Code)|"PERO" %in% unique(PCode_Lookup$LSPC_Code)), "CONVERT_FLOW_TO_AVGDAILY" %in% Plot_Types)) {                                          ## Only applies this section when FLOW is selected AND convert to average daily flow is activated 
                        
                        Flow_Pcode <- PCode_Lookup$PCode[PCode_Lookup$LSPC_Code == "RO"|PCode_Lookup$LSPC_Code == "PERO"]                                                                   ## Extracts custom Pcode used for flow
                        
                        print(paste0("Simulated ", Flow_Pcode, " records were aggregated on a daily basis to calculate an average daily simulated flow with a time stamp set to noon; Aggregation occurred prior to pairing with measured flow records"))
                        
                        Model_Intrp <- Model_Intrp %>%                                                                                                                      ## Filters the flow records from the model data
                          dplyr::filter(PCode == Flow_Pcode) %>%
                          mutate(Date_Time = make_datetime(year = year(Date_Time), month = month(Date_Time), day = day(Date_Time), hour = 12, min = 0, tz = Model_TZ)) %>%    ## Replaces Date_Time with same day, but all time set to noon
                          group_by(Segment_ID, Segment_Name, Station_ID_Mrgd, Plot_Title, Plot_Order, Anal_Name, PCode, Units, Date_Time) %>%                                  ## Groups the data by day 
                          summarize(Result = mean(Result)) %>%                                                                                                                ## Calculates average daily flow
                          ungroup() %>%                                                                                                                                       ## Ungroups DF
                          bind_rows(., Model_Intrp %>% dplyr::filter(PCode != Flow_Pcode)) %>%                                                                                   ## Binds average daily flow to the original Model data w/o FLOW data
                          arrange(PCode, Segment_ID, Date_Time)
                      }
                      
                        
                      
                      #********************************************************************************************************************
                      #                                       Preprocesses and standardizes Raw Measured Data
                      #********************************************************************************************************************
                      
                      
                      if (is.na(min(Model_Intrp$Date_Time)) | is.na(max(Model_Intrp$Date_Time))){
                        stop(paste("Date_Time column in Extracted Model *.txt is not formatted as a Date.  Examine Date_Time column in", Extract_Model_Filename, "located in", Path_Calib_In, sep = " "))           ## Error / ends script if Date_Time column not properly formatted by FORTRAN *.exe
                      }else{
                        
                        Measured_Intrp <- Measured_Raw %>%
                          dplyr::rename(Calib_Station_ID = Station_ID) %>%                                                                                                                   ## Renames WRDB Station ID to Calib_Station_ID
                          dplyr::mutate(Calib_Station_ID = toupper(Calib_Station_ID)) %>%                                                                                                    ## Sets Calib_Station_ID to all capital letters
                          dplyr::mutate(Calib_Station_ID = gsub("_", "-", Calib_Station_ID)) %>%                                                                                             ## Replaces "_" with "-" in Stat_ID
                          dplyr::mutate(Calib_Station_ID = gsub(" ", "-", Calib_Station_ID)) %>%                                                                                             ## Replaces spaces with "-"
                          #dplyr::mutate(Calib_Station_ID = case_when(Calib_Station_ID = is.numeric(as.numeric(Calib_Station_ID)) ~ str_pad(Calib_Station_ID, 8, side = "left", pad = "0"),
                          #                                           TRUE ~ Calib_Station_ID)) %>%                                                                                          ## Adds leading zero if Excel stripped
                          #dplyr::mutate(Calib_Station_ID = gsub("^0", "USGS-0", Calib_Station_ID)) %>%                                                                                      ## Remaps the calibration data because Excel dropped leading zeros 
                          dplyr::filter(Calib_Station_ID %in% Station_Order_Lookup$Calib_Station_ID) %>%
                          left_join(Station_Order_Lookup, by = ("Calib_Station_ID" = "Calib_Station_ID")) %>%                                                                                ## Adds missing column
                          dplyr::mutate(Date_Time = ymd_hms(Date_Time, tz = Model_TZ, truncated = 3)) %>%                                                                                    ## Formats the date as POSIXct
                          dplyr::mutate(RCode = toupper(RCode)) %>%                                                                                                                          ## Standardize RCode values to Uppercase
                          dplyr::filter(Segment_ID %in% unique(Station_Order_Lookup$Segment_ID)) %>%                                                                                     ## Drop any WASP segment that isn't a calibration segment
                          dplyr::filter(Date_Time >= min(Model_Period) & Date_Time <= max(Model_Period)) %>%                                                               ## Trims Calib Data to Time Period of Model Data
                          dplyr::filter(PCode %in% unique(PCode_Lookup$PCode)) %>%                                                                                                           ## Drops any PCode that isn't a calibration PCode
                          left_join(PCode_Lookup, by = c("PCode" = "PCode")) %>%                                                                                                             ## Adds missing columns
                          select(Date_Time, Segment_ID, Segment_Name, Station_ID_Mrgd, Calib_Station_ID, Plot_Title, Plot_Order,                                                              ## Selects these columns going forward
                                 PCode, Result, RCode, Units, Depth_M) %>%
                          dplyr::mutate(Floor = floor_date(Date_Time, unit = "hour" ),
                                        Ceil = ceiling_date(Date_Time, unit = "hour")) %>%
                          arrange(PCode, Segment_ID, Date_Time)                                                                                                                            ## Sorts DF for later
                        
                        
                        #********************************************************************************************************************
                        #                                        QA Measured and Model Files Contain Records
                        #********************************************************************************************************************
                        
                        if(nrow(Measured_Intrp) == 0){                                                                                                                ## Error check / ends script if Measured DF contains no records
                          stop("Measured dataframe does not contain records for any of the calibration stations or parameters selected in Calib_Lookup.xlsx; 
                                  \nVerify selected stations/Pcodes are in raw measured datafile")
                        }else {
                          if(nrow(Model_Intrp) == 0) {                                                                                                               ## Error check / ends script if Model DF contains no records
                            stop("Model dataframe does not contain records for any of the calibration stations or parameters selected in Calib_Lookup.xlsx; 
                                    \nVerify selected stations/Pcodes are in model input datafile")
                          }else {
                            
                            #********************************************************************************************************************
                            #                                 QA All Selected Calibration Segments in Model Output File
                            #                      Loop Only Interpolates Selected Calibration Segments/PCodes that are in the Model Datafile
                            #********************************************************************************************************************
                            
                            if(length(setdiff(Seg_Order, unique(Model_Intrp$Segment_ID))) != 0) {
                              warning(paste("WASP Segment(s)", paste(setdiff(Seg_Order, unique(Model_Intrp$Segment_ID)), collapse = ", "), 
                                            "were selected for calibration in Lookup Table, but had no records in the input Model Data. \nThe missing WASP segment(s) were excluded from calibration", sep = " "))
                              Seg_Order <- intersect(Seg_Order, Model_Intrp$Segment_ID)
                            } 
                            
                            if(length(setdiff(unique(PCode_Lookup$PCode), unique(Model_Intrp$PCode))) != 0) {
                              warning(paste("PCode(s)", paste(setdiff(unique(PCode_Lookup$PCode), unique(Model_Intrp$PCode)), collapse = ", "), 
                                            "were selected for calibration in Lookup Table, but had no records in the input Model Data. \nThe missing PCode(s) were excluded from calibration", sep = " "))
                              Seg_Order <- intersect(Seg_Order, Model_Intrp$Segment_ID)
                            } 
                           
                            
                            #********************************************************************************************************************
                            #                                 QA All Selected Pcodes in Measured Data File
                            #                      Loop Only Interpolates Selected Calibration Segments/PCodes that are in the Calibration Datafile
                            #********************************************************************************************************************
                            
                            
                            if(length(setdiff(unique(PCode_Lookup$PCode), unique(Measured_Raw$PCode))) != 0) {
                              warning(paste("PCode(s)", paste(setdiff(unique(PCode_Lookup$PCode), unique(Measured_Raw$PCode)), collapse = ", "), 
                                            "were selected for calibration in Lookup Table, but the raw Measured Datafile did not contain measured data for those Pcode(s) at any monitoring location. \nThe missing PCode(s) were excluded from calibration", sep = " "))
                            } 
                            
                            if(length(setdiff(unique(PCode_Lookup$PCode), unique(Measured_Intrp$PCode))) != 0) {
                              warning(paste("After pairing Measured and Model data, the following Pcodes were selected for calibration, but did not have paired measured and model data records: ", paste(setdiff(unique(PCode_Lookup$PCode), unique(Measured_Intrp$PCode)), collapse = ", "), 
                                            "\nThe missing PCode(s) were excluded from calibration", sep = " "))
                            }
                            
                            
                            
                            #********************************************************************************************************************
                            #        Use PURRR MAP2 to iterate over over Data to Pair Measured Obs to Model Obs that Occur Immediately Before and After Date_Time of Measured Obs  
                            #                  and Linear Interpolate Model Data at Exact Minute that Measured Data Record was Collected
                            #                  
                            #                          Because the model and meas DF are joined as group_nested DFs, 
                            #           MAP2 can apply findInterval() to the nested DF to find Model Result that is immediately before the time stamp of the measured record
                            #                              It is applied based on group_nested PCode and Segment_ID
                            #********************************************************************************************************************
                            
                            group_WASP <- Model_Intrp %>%                                                                                             
                              rowid_to_column(., var = "Index") %>%                                                                                   ## Adds rowid as a column
                              group_nest(PCode, Segment_ID, .key = "Model_Data", keep = TRUE)                                                       ## group and nest model DF by Pcode and Segment (to allow join)
                            
                            
                            group_pair <- Measured_Intrp %>%                                                                                          
                              select(Segment_ID, Station_ID_Mrgd, Plot_Title, Date_Time, PCode, Result, RCode, Depth_M, Units, Floor, Ceil) %>%                         ## Selects and reorders columns
                              rename(Result_Obs = Result) %>%                                                                                                             ## Renames columns
                              dplyr::mutate(Missing = NA) %>%                                                                                                             ## Creates a blank NA column to make na.approx easier
                              group_nest(PCode, Segment_ID, .key = "Meas_Data", keep = TRUE) %>%                                                                        ## group and nest DF by PCode and Segment (allow join)
                              left_join(., group_WASP, by = c("PCode", "Segment_ID")) %>%                                                                               ## Joins nested model DF with nested measured DF
                              ## These next lines calling map2 are where the Pre- and Post- model records (Date_Time and Result) are extracted.  It adds a new column in the nested Meas_Data DF
                              dplyr::mutate(Meas_Data = map2(.x = Meas_Data, .y = Model_Data, ~ mutate(.x, Sim_Index = findInterval(.x$Date_Time, .y$Date_Time)))) %>%    ## Finds row index of nested model DF based on Date_Time in nested Meas and nested Model. This finds the model record that is immediately before the measured data record.
                              dplyr::mutate(Meas_Data = map2(.x = Meas_Data, .y = Model_Data, ~ mutate(.x, Sim_ResPre = .y$Result[.x$Sim_Index],                          ## Uses row_index to extract Pre_Result from nested model DF
                                                                                                           Sim_ResPost = .y$Result[.x$Sim_Index + 1],                     ## Uses row_index to extract Post_Result from nested model DF (Index + 1 row)
                                                                                                           Sim_DatePre = .y$Date_Time[.x$Sim_Index],                      ## Uses row_index to extract Pre_Date from nested model DF
                                                                                                           Sim_DatePost = .y$Date_Time[.x$Sim_Index + 1]))) %>%           ## Uses row_index to extract Post_Date from nested model DF (Index + 1 row)
                              select(-Model_Data, -PCode, -Segment_ID) %>%                                                                                              ## Drops these columns
                              unnest(Meas_Data) %>%                                                                                                                       ## Unnests Measured DF, which now has teh necessary columns from model DF
                              ### This next mutate call is commented out bc it is the manual calculation.  na.approx does the same thing
                              #dplyr::mutate(TStep_Mins = as.duration(Sim_DatePost - Sim_DatePre) / dminutes(1),                                            ## Calculates time difference between the Sim Record immediately before and after Meas Record (in mins)
                              #              Slope = (Sim_ResPost - Sim_ResPre) / TStep_Mins,                                                               ## Calcs. slope of Result/Time Difference for Sim Record
                              #              Delt = as.duration(Sim_DatePost - Date_Time) / dminutes(1),                                                    ## Calcs. time difference between Date_Time of Meas Record and Date_Time of Model Record immediately after it
                              #              Result_Sim = Sim_ResPost - (Slope * Delt)) %>%                                                                 ## Uses Model Result Value, Slope, and Delta to back-interpolate Result_Sim at exact minute of Measured Record Date_Time
                              as.data.frame() %>%                                                                                                                         ## Converts to DF, for rowwise
                              rowwise() %>%                                                                                                                               ## Applies rowwise for next function
                              ## This next line is where the linear interpolation occurs based on a vector of Results (Result of Model PreRecord, NA, Result of Model PostRecord) relative to vector of dates (Date_Time of Model PreResult Record, Date_Time of Measured Record, Date_Time of Model PostResult Record)
                              dplyr::mutate(Result_Sim = na.approx(c(Sim_ResPre, Missing, Sim_ResPost), c(Sim_DatePre, Date_Time, Sim_DatePost)) %>% .[2]) %>%            ## Uses na.approx to linearly interpolate missing value based on extracted model records
                              as.data.frame()                                                                                                                             ## Converts to dataframe
                            
                             
                            #********************************************************************************************************************
                            #                      Reformat Processed and Paired Calibration Data
                            #********************************************************************************************************************
                            
                            Model_Intrp <- Model_Intrp %>%
                              select(Date_Time, Segment_ID, Station_ID = Station_ID_Mrgd, Plot_Title, PCode, Result) %>%                           ## Selects columns from Model DF
                              spread(key = PCode, value = Result) %>%                                                                              ## Casts DF wide
                              dplyr::mutate(Segment_ID = factor(Segment_ID, levels = unique(Seg_Order))) %>%                                       ## Sets order of WASP segments
                              arrange(Date_Time, Segment_ID)                                                                                       ## Sorts DF to match Plot Order in Lookup
                            
                            Out_WASP_Calib <- group_pair %>%                                                                                       ## Sorts DF to match Plot Order in Lookup
                              arrange(Segment_ID, PCode, Date_Time)
                            
                            Measured_Paired <- Out_WASP_Calib %>%
                              select(Date_Time, Station_ID = Station_ID_Mrgd, Segment_ID, PCode, 
                                     Result_Obs, Result_Sim, Plot_Title, Units, RCode, Depth = Depth_M) %>%                                        ## Selects columns from Paired DF
                              arrange(Segment_ID, PCode, Date_Time)                                                                                ## Sorts DF to match Plot Order in Lookup
                            
                            
                            #********************************************************************************************************************
                            #                      Creates and exports RCode Flagged/Accepted Mapping
                            #                      
                            #             The Rcode crosswalk is used to populate Rcode flags for rest of processing         
                            #             
                            #********************************************************************************************************************
                            
                            Out_WASP_Calib <- Out_WASP_Calib %>%                                                                                                                                                   ## Uses Measured Paired as Starting Point
                              dplyr::mutate(RCode_Flag = case_when(grepl(RCode_Exclusion, RCode) & !anyNA(RCode_Exclusion) ~ "Flagged",                                                                           ## Map Rcodes to either Flagged/Accepted
                                                                   TRUE ~ "Accepted")) 
                            
                            Rcode_Mapping <- Out_WASP_Calib %>%                                                                                                                                                                     
                              distinct(RCode, RCode_Flag)                                                                                                                                                         ## Finds all combinations of Rcodes and Flagged status
                            
                            fwrite(Rcode_Mapping, file = paste(Path_Data_Out, "/Proc_Out/Proc_Rcode_Crosswalk_", Model_Run_Code, ".txt", sep = ""), sep = "\t", na = "",                                          ## Exports as .txt file
                                   col.names = TRUE, row.names = FALSE, dateTimeAs = "write.csv")
                            
                            
                            
                            #********************************************************************************************************************
                            #                      Export Processed and Paired Calibration Data as .TXT
                            #********************************************************************************************************************
                            
                            fwrite(Model_Intrp, file = paste(Path_Data_Out, "/Proc_Out/Proc_Model_", Model_Run_Code, ".txt", sep = ""), sep = "\t", na = "",                                                    ## Saves Processed WASP DF 
                                   col.names = TRUE, row.names = FALSE, dateTimeAs = "write.csv")
                            
                            fwrite(Measured_Paired, file = paste(Path_Data_Out, "/Proc_Out/Proc_CalibModel_", Model_Run_Code, ".txt", sep = ""), sep = "\t", na = "",                                                   ## Saves paired Calib_WASP DF
                                   col.names = TRUE, row.names = FALSE, dateTimeAs = "write.csv")
                            
                            fwrite(Out_WASP_Calib, file = paste(Path_Data_Out, "/Proc_Out/Proc_QACheck_", Model_Run_Code, ".txt", sep = ""), sep = "\t", na = "", 
                                   col.names = TRUE, row.names = FALSE, dateTimeAs = "write.csv")
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }
}



