#********************************************************************************************************************
#
#           Master Script to download average daily flow and instantaneous WQ data from NWIS, process for WRDB, and plot time series
#                                   Uses SOURCE to call the various R subscripts
#                                 
#                                   Flow and WQ downloaded/processed separately 
#             Flow downloaded from daily value server and WQ downloaded from the instantaneous value server
#                      
#                                          
#                      Instantaneous WQ values filtered to 4 records per day (if available)
#                      
#         Scripts also calculate hourly averages and various daily summary statistics for WQ parameters
#                      
#                 These are plotted in various time series figures and exported as WRDB ready files
#                         
#                               An optional script based on plotly package is available. 
#         
#         This optional script uses the fully processed data to create interactive time series plot that can be viewed directly in RStudio
#                         
#                         This NWIS_Plotly.R script is not automatically called via SOURCE and has to be run manually
#                         
#           ***** Updated to include option of downloading instantaneous flow, which is then converted to 4X per day records, and average hourly flow *********
#********************************************************************************************************************

#********************************************************************************************************************
#                                            Loads library for .bat mode
#********************************************************************************************************************

library("gdata")           # for data processing
library("pastecs")         # for stat.desc
library("Rmisc")           # for stats
library("zoo")             # for dates
library("lubridate")       # for dates
library("tidyverse")       # for data formatting
library("dataRetrieval")   # for USGS data download
library("openxlsx")        # for reading Excel files
library("rMR")             # for DOSAT calculation
library("stats")           # for stats
library("RColorBrewer")    # for color palette
library("purrr")           # for iterating over data

#********************************************************************************************************************
#                          Custom function to create required subfolders 
#           This section verifies that the required subfolders exist and creates them when they are missing
#           
#                       Filepath based on the RProject working directory
#               
#               Creates all of the subfolders, even when they are not being plotted
#********************************************************************************************************************

CREATE_SUBFOLDER <- function(PLOT_TYPE) {
  
  Crosswalk <- data.frame(Type = c("WRDB_RDY", "RAW_DATA", "STATS", "QA_CHECK", "TS_PLOT", "DLY_TS_PLOT", "HRLY_FLW_PLOT", "HRLY_WRDB"),                    ## Create crosswalk of Arguments to Folder names
                          Subfolder = c("/WRDB_Rdy", "/Raw_Data", "/Stats", "/QA_Check", "/Plot", "/Plot_DlyTS", "/Plot_HrlyTS", "/WRDB_Other"))
  
  temp <- dplyr::filter(Crosswalk, Type == PLOT_TYPE)
  
  if(dir.exists(paste0(getwd(),temp$Subfolder)) != TRUE) {                                               ## Verifies if subfolder already exists
    dir.create(paste0(getwd(), temp$Subfolder))                                                          ## When subfolder is missing, creates the subfolder
    print(paste(paste0(getwd(), temp$Subfolder), "was missing and was created by the R Scripts"))}       ## Prints message when subfolder was created
  
}

walk(.x = c("WRDB_RDY", "STATS", "RAW_DATA", "QA_CHECK", "TS_PLOT"), CREATE_SUBFOLDER)                    ## Calls custom function


#********************************************************************************************************************
#                                            Import NWIS data download lookup
#********************************************************************************************************************
USGS_Dwnld_Info <- openxlsx::read.xlsx( "./Lookup/USGS_NWIS_Lookup.xlsx", sheet = "Gage_Lookup", 
                                        colNames =  TRUE, detectDates = TRUE, skipEmptyCols = TRUE) %>%   
  #dplyr::mutate(Gage_Number = str_pad(Gage_Number, 8, side = "left", pad = "0")) %>%                                       ## Adds leading zeros stripped by Excel
  dplyr::mutate(Gage_Number = str_remove(Gage_Number, "USGS")) %>%                                                          ## Drops the USGS prepend
  dplyr::mutate(Gage_Number = str_remove(Gage_Number, "-")) %>%                                                             ## Drops any dashes
  dplyr::mutate(USGS_parmCd = str_pad(USGS_parmCd, 5, side = "left", pad = "0"))                                            ## Adds leading zeros to Parameter Cd

#********************************************************************************************************************
#                         Vector of parameters and gages to download from NWIS 
#********************************************************************************************************************
## 00010 = Water Temperature in deg C; 00011 = Water Temperature in deg F;
## 00095 = Spec. Cond @ 25deg C; 00300 = Dissolved Oxygen; 
## 00301 = DO Saturation, 00400 = pH, 00480 = Salinity in ppt; 
## 00060 = Flow; 00065 = Gage Height
## Eastern TZ = US/Eastern; Central TZ = US/Central; Mountain TZ = US/Mountain; Pacific TZ = US/Pacific

Keep_Gage_Dwnld <- USGS_Dwnld_Info$Gage_Number[!is.na(USGS_Dwnld_Info$Gage_Number)]                                         ## Gages to download
Keep_WQ_PCode_Dwnld <- USGS_Dwnld_Info$USGS_parmCd[!is.na(USGS_Dwnld_Info$Selected_WQ)]                                     ## Pcodes to download

Keep_Query_Dates <- c(USGS_Dwnld_Info$Start_Date[1], USGS_Dwnld_Info$End_Date[1])                                           ## Sets the date range
Keep_Model_TZone <-  USGS_Dwnld_Info$Gage_TimeZone[!is.na(USGS_Dwnld_Info$Selected_TZ)]                                     ## Sets time zone for data download output

Keep_WQ_Switch <- ifelse(length(Keep_WQ_PCode_Dwnld) == 0, FALSE, TRUE)                                                     ## Determines whether user downloaded WQ data, turns on WQ scripts

Keep_WQPCode_Crosswalk <- USGS_Dwnld_Info %>%                                                                               ## Creates Pcode crosswalk to allow processed data to repopulate Units and Names based on custom Pcodes
  select(Selected_WQ, USGS_parmCd, Pcode_WQ) %>%                     
  filter(!is.na(USGS_parmCd)) %>%                                                                                           ## Drops Pcodes that do not have a USGS parameter code
  distinct(USGS_parmCd, Pcode_WQ)                                                                                           ## Drops Selection column to avoid JOIN issues later

#********************************************************************************************************************
#                         Sets what parameters to download (flow, gage height, instantaneous or average daily)
#********************************************************************************************************************

Switch_Controls <- USGS_Dwnld_Info %>%
  select(Selected_Flw, Flow) %>%
  drop_na(Flow)

Keep_Flow_Switch <- ifelse(is.na(Switch_Controls$Selected_Flw[Switch_Controls$Flow == "Avg Daily Flow"]), FALSE, TRUE)                                           ## Determines whether user downloaded average daily flow data; turns on flow scripts

Keep_Instant_Flow_Switch <- ifelse(is.na(Switch_Controls$Selected_Flw[Switch_Controls$Flow == "Instantaneous Flow"]), FALSE, TRUE)                                           ## Determines whether user downloaded average daily flow data; turns on flow scripts

Keep_Instant_GHeight_Switch <- ifelse(is.na(Switch_Controls$Selected_Flw[Switch_Controls$Flow == "Instantaneous Gage Height"]), FALSE, TRUE)                                           ## Determines whether user downloaded average daily flow data; turns on flow scripts

#Keep_Flow_Switch <- ifelse(is.na(USGS_Dwnld_Info$Selected_Flw[[1]]), FALSE, TRUE)                                           ## Determines whether user downloaded average daily flow data; turns on flow scripts

#Keep_Instant_Flow_Switch <- ifelse(is.na(USGS_Dwnld_Info$Selected_Flw[[2]]), FALSE, TRUE)                                   ## Determines whether user downloaded Instantaneous flow data; turns on flow scripts



#********************************************************************************************************************
#                                Create function that will populate the Excel workbook
#                                
#                                Requires vector of DFs and vector of sheet names
#********************************************************************************************************************


POPULATE_EXCEL_WORKBOOK <- function(INPUT_DATA, WORKSHEET_NAME) {
  
  negStyle <- createStyle(fontColour = "#9C0006", bgFill = "#FFC7CE")                                                                                                ## Creates cell style for negative values
  
  addWorksheet(wb_Replc_Blnk, WORKSHEET_NAME)                                                                                                                        ## Adds worksheet based on vector of worksheet names
  writeData(wb_Replc_Blnk, sheet = WORKSHEET_NAME, INPUT_DATA)                                                                                                       ## Writes R DF to new worksheet
  setColWidths(wb_Replc_Blnk, sheet = WORKSHEET_NAME, cols = 1:ncol(INPUT_DATA), widths = "auto")                                                                    ## Sets widths of columns    
  
  conditionalFormatting(wb_Replc_Blnk, WORKSHEET_NAME, cols = 1:ncol(INPUT_DATA), rows = 2:(nrow(INPUT_DATA)+1), type = "expression", rule ="<0", style = negStyle)  ## Adds conditional formatting to highlight negative values
  
}

#********************************************************************************************************************
#                           For each gage, sets elevation and salinity for DO.Saturation calculation
#********************************************************************************************************************

Keep_Site_Phys <- USGS_Dwnld_Info %>%                                                                          
  select(Gage_Number, Gage_Elev_m, Gage_Salin_ppt) %>%                                                 ## Selects columns
  drop_na(Gage_Number) %>%                                                                             ## Drops cells with blank gage numbers
  dplyr::mutate(Gage_Number = paste("USGS", Gage_Number, sep = "-"))                                   ## Prepends gage number with USGS

#********************************************************************************************************************
#                   IF statement to download and process average daily flow when selected
#********************************************************************************************************************

if (Keep_Flow_Switch == TRUE) {
  
  #********************************************************************************************************************
  #                   For average daily flow, extracts most recent download, processing, and plotting scripts to be sourced
  #********************************************************************************************************************
  
  Keep_FlwDwnld_Script <- grep("AvgFlw_Download", dir("./Scripts"), value = TRUE) %>%                                 
    dplyr::last(.)
  
  Keep_FlwProc_Script <- grep("AvgFlw_Proc", dir("./Scripts"), value = TRUE) %>%                                 
    dplyr::last(.)
  
  Keep_FlwPlot_Script <- grep("AvgFlw_PlotTS", dir("./Scripts"), value = TRUE) %>%                                 
    dplyr::last(.)
  
  source(paste("./Scripts/", Keep_FlwDwnld_Script, sep = ""))
  source("./Scripts/Cleanup_Environ.R")
  
  #********************************************************************************************************************
  #                    If statement to process/plot data only if there are data available at the gages
  #                          Returns an error if no average daily flow records available on NWIS
  #********************************************************************************************************************
  
  if (nrow(Keep_USGS_AvgDlyFlow_Raw) == 0) {
    message ("Selected gages had no average daily flow records for the date range selected; No flow data were downloaded or processed")                                    ## Returns an error when data available for gages
  } else {
    
    #********************************************************************************************************************
    #                            Sources the data processing script and cleans up environment
    #********************************************************************************************************************
    
    source(paste("./Scripts/", Keep_FlwProc_Script, sep = ""))
    source("./Scripts/Cleanup_Environ.R")
    
    #********************************************************************************************************************
    #                            Sources the time series plotting script and cleans up environment
    #********************************************************************************************************************
    
    source(paste("./Scripts/", Keep_FlwPlot_Script, sep = ""))
    source("./Scripts/Cleanup_Environ.R")
    
  }
}

#********************************************************************************************************************
#                   IF statement to download and process instantaneous flows and hourly flows when selected
#********************************************************************************************************************

if (Keep_Instant_Flow_Switch == TRUE) {
  
  #********************************************************************************************************************
  #                   For instantaneous flows, extracts most recent download, processing, and plotting scripts to be sourced
  #********************************************************************************************************************
  
  Keep_Flw_Instant_Script <- grep("InstantFlw_Download", dir("./Scripts"), value = TRUE) %>%                                                   ## Extracts most recent Download/Process Instant Flow script                             
    dplyr::last(.)
  
  source(paste("./Scripts/", Keep_Flw_Instant_Script, sep = ""))                                                                               ## Sources instant flow scrip
  source("./Scripts/Cleanup_Environ.R")
  
}

#********************************************************************************************************************
#                   IF statement to download and process instantaneous flows and hourly flows when selected
#********************************************************************************************************************

if (Keep_Instant_GHeight_Switch == TRUE) {
  
  #********************************************************************************************************************
  #                   For instantaneous flows, extracts most recent download, processing, and plotting scripts to be sourced
  #********************************************************************************************************************
  
  Keep_GHeight_Instant_Script <- grep("InstantGHeight_Download", dir("./Scripts"), value = TRUE) %>%                                                   ## Extracts most recent Download/Process Instant Flow script                             
    dplyr::last(.)
  
  source(paste("./Scripts/", Keep_GHeight_Instant_Script, sep = ""))                                                                               ## Sources instant flow scrip
  source("./Scripts/Cleanup_Environ.R")
  
}

#********************************************************************************************************************
#                   IF statement to download and process instantaneous WQ when selected
#********************************************************************************************************************

if (Keep_WQ_Switch == TRUE) {                                                                                                                  ## Downloads and processes WQ data when activated
  
  #********************************************************************************************************************
  #                            For instantaneous data, extracts most recent download, processing, and plotting scripts to be sourced
  #********************************************************************************************************************
  
  Keep_WQDwnld_Script <- grep("WQ_Download_Srce", dir("./Scripts"), value = TRUE) %>%                                 
    dplyr::last(.)
  
  Keep_WQProc_Script <- grep("WQ_Proc_Srce", dir("./Scripts"), value = TRUE) %>%                                 
    dplyr::last(.)
  
  Keep_WQDlyStat_Proc_Script <- grep("WQ_DlyStat_Proc_Srce", dir("./Scripts"), value = TRUE) %>%                                 
    dplyr::last(.)
  
  Keep_WQDlyStat_Plot_Script <- grep("WQ_DlyStat_PlotTS_Srce", dir("./Scripts"), value = TRUE) %>%                                 
    dplyr::last(.)
  
  Keep_WQPlot_Script <- grep("WQ_PlotTS_Srce", dir("./Scripts"), value = TRUE) %>%                                 
    dplyr::last(.)
 
  #********************************************************************************************************************
  #                            Sources the download script and cleans up environment
  #********************************************************************************************************************
  
  source(paste("./Scripts/", Keep_WQDwnld_Script, sep = ""))
  source("./Scripts/Cleanup_Environ.R")
  
  #********************************************************************************************************************
  #                    If statement to process/plot data only if there are data available at the gages
  #                     Will throw an error if there are no NWIS records for selected gages, Pcodes, and dates
  #********************************************************************************************************************
  
  if (nrow(Keep_Merged_WQ_Raw) == 0) {
    message ("Selected gages had no data records for the date range and water quality parameters selected; No water quality data were downloaded or processed")                                    ## Returns an error when data available for gages
  } else {
    
    #********************************************************************************************************************
    #                            Sources the water quality data processing script and cleans up environment
    #********************************************************************************************************************
    
    source(paste("./Scripts/", Keep_WQProc_Script, sep = ""))
    source("./Scripts/Cleanup_Environ.R")
    
    #********************************************************************************************************************
    #                     Sources the water quality daily stats processing and plotting script and cleans up environment
    #********************************************************************************************************************
    
    source(paste("./Scripts/", Keep_WQDlyStat_Proc_Script, sep = ""))
    
    source(paste("./Scripts/", Keep_WQDlyStat_Plot_Script, sep = ""))
    source("./Scripts/Cleanup_Environ.R")
    
    #********************************************************************************************************************
    #                            Sources the time series plotting script and cleans up environment
    #********************************************************************************************************************
    
    source(paste("./Scripts/", Keep_WQPlot_Script, sep = ""))
    source("./Scripts/Cleanup_Environ.R")
  }
}


print("**************************************************************************************************************")
print("            Scripts successfully completed.  Downloaded, processed, and plotted all NWIS data")
print("**************************************************************************************************************")

