
#********************************************************************************************************************
#
#                    Includes spatial viewer scripts and arguments (including HTML vs. SAVE argument to either save figures or create spatial map)
#                                  
#       SOURCE FUNCTION Script to Generate Multiple Panel Calibration Figure (Time Series, 1:1, Prob Dist, Goodness of Fit Statistics)
#       
#                     Updated scripts no longer have the GoF scripts in the multipanel calibration figure
#
#                         Input data needs to be preprocessed based on Proc_Source Script
#                              Designed to Run as SOURCE Script Called in Master_Script v6.0 or later
#
#           Requires purrr and nested dataframe (NestedData_Multi)  and will exclude segments that do not exceed the Min_Obs cutoff 
#           
#                                         Includes PDF vs. JPG ggsave argument   
#                                         
#                                         J. Davis v2021-07-14                                                   
#
#********************************************************************************************************************

#********************************************************************************************************************
#                                   Create custom function to create multipanel figure
#                                   
#                      OUTPUT argument defaults to SAVE, but can be set to HTML (allowing HTML maps to be displayed)
#                                   
#                           Includes Fig_Type argument that allows figure to be saved as PDF or JPG
#                                             Default is to save it as a JPG
#                                             
#                          Includes Fig_Numb argument that allows Figure # to be included in the figure title
#                                     Default is to exclude the figure # from the figure title
#                                      
#********************************************************************************************************************



PLOT_SAVE_NESTED_MULTIPANEL <- function(MODEL_DF, MEAS_DF, OUTPUT = "SAVE", FIG_TYPE = "JPG", FIG_NUMB = NA) {
  
  #********************************************************************************************************************
  #                                          Creates Watershed Map that includes Monitoring Station location
  #********************************************************************************************************************
  
  if ("MAP" %in% Plot_Types) {                                                                      ## IF statement when Map selected on Lookup
    
    Stat_temp <- Station_Order_Lookup %>%                                                             ## Extracts lat/longs for monitoring station
      dplyr::filter(Segment_ID == unique(MEAS_DF$Segment_ID)) %>%
      st_as_sf(coords = c("Long", "Lat"), crs = as.numeric(Shape_CRS))                                ## Converts lat/longs to shapefile (w/ reprojection)
    
    Map_temp <- Base_Map + 
      geom_sf(data = Stat_temp, size = 2, color = "red", shape = 19) +                                ## Plots WASP network
      coord_sf(datum = NA) +                                                                          ## Sets axis to coordinate units
      theme_nothing()
    
  }
  
  
  
  #********************************************************************************************************************
  #                                  Start Plotting and Calculating Statistics
  #********************************************************************************************************************
  
  #********************************************************************************************************************
  #                                   Loop Over Data to Generate Figure Scales
  #********************************************************************************************************************
  if(str_detect(str_to_upper(unique(MEAS_DF$PCode)), "PH|WSE")) {                                                     ##  Uses IFELSE statement to generate vectors to set figure axes scales; WSE and PH zoomed to data range
    TS_Scale <- range(c(MEAS_DF$Result_Obs, MODEL_DF$Result_Sim_Full), na.rm = TRUE)                                  ##  Sets range based on combined FULL Sim and paired Obs data
    Reg_Scale_Y <- range(c(MEAS_DF$Result_Obs, MEAS_DF$Result_Sim), na.rm = TRUE)                                     ##  Sets Y-scale of regression based on range of paired OBS and SIM data 
    Reg_Scale_X <- range(c(MEAS_DF$Result_Obs, MEAS_DF$Result_Sim), na.rm = TRUE)                                     ##  Sets X-scale of regression based on range of paired OBS and SIM data
    Prob_Scale <- range(c(MEAS_DF$Result_Obs, MEAS_DF$Result_Sim))                                                    ##  Sets Y-scale of prob plot based on range of paired OBS and SIM data
  } else {                                                                                                            ##  As above, but sets the min value to 0 for all figures and max based on max
    TS_Scale <- c(0, max(c(MEAS_DF$Result_Obs, MODEL_DF$Result_Sim_Full), na.rm = TRUE))                 
    Reg_Scale_Y <- c(0, max(c(MEAS_DF$Result_Obs, MEAS_DF$Result_Sim), na.rm = TRUE))
    Reg_Scale_X <- c(0, max(c(MEAS_DF$Result_Obs, MEAS_DF$Result_Sim), na.rm = TRUE)) 
    Prob_Scale <- c(0, max(c(MEAS_DF$Result_Obs, MEAS_DF$Result_Sim), na.rm = TRUE))
  }
  #********************************************************************************************************************
  #                                                Time Series Plot
  #           Plots simulated data as a line plot, with the measured data overlaid as scatterplot (color based on Flagged vs Accepted)
  #********************************************************************************************************************
  
  #********************************************************************************************************************
  #                              IF Statement to include Figure # in Plot Title or not
  #                                       Default is to exclude the Figure #
  #********************************************************************************************************************
  
  
  if(is.na(FIG_NUMB)) {
    
    Main_Title <- paste(paste(unique(MEAS_DF$Plot_Title), collapse = " / "),                                                 ## Create plot title based on WASP Seg and Station Name
                        " \nParameter: ", unique(MODEL_DF$PCode), sep = "")  
  } else {
    
    Main_Title <- paste(paste0("Figure ", FIG_NUMB, ": ", unique(MEAS_DF$Plot_Title), collapse = " / "),                    ## Create plot title based on WASP Seg and Station Name
                        " \nParameter: ", unique(MODEL_DF$PCode), sep = "")  
  }
  
  #********************************************************************************************************************
  #                              Create footnote and filenames
  #********************************************************************************************************************
  
  
  FootNote <- paste("\n(Calib Station: ",                                                                                    ## Adds a footnote to figure indicating segment and sampling station
                    gsub("_", " / ", unique(MODEL_DF$Station_ID_Mrgd)), 
                    "; ", Model_Type, " Seg: ", unique(MEAS_DF$Segment_ID), ")", 
                    "\nDate Plotted: ", Sys.Date(), sep = "")
  
  File_Title <- paste(unique(MEAS_DF$Plot_Title), collapse = " / ") %>%                                                    ## Creates filename
    str_trim(side = "both") %>%
    str_trunc(., width = 30, ellipsis = "") %>%
    gsub(" / ", "_", . ) %>%
    gsub("\\.|\\. ", " ", .) %>%
    gsub(" ", "_", .)
  
  Punit <- unique(MEAS_DF$Units)                                                                                           ## Extracts unit for Plotting
  
  TS <- ggplot() +                                                                                                                                                      ## Calls up ggplot
    geom_line(data = MODEL_DF, aes(x = Date_Time, y = Result_Sim_Full), linetype = 1, color = "black",  linewidth = 0.5) +                                                 ## Creates time series of simulated values
    geom_point(data = MEAS_DF, aes(x = Date_Time, y = Result_Obs, color = factor(RCode, levels = c("Accepted", "Flagged"))), size = 1.75, alpha = 0.8, show.legend = TRUE) +            ## Adds the measured observations to the time series (Colored based on Flagged vs Accepted)
    ##geom_line(data = MODEL_DF, aes(x = Date_Time, y = Result_Sim_Full), linetype = 1, color = "black",  linewidth = 0.5) +                                                 ## Creates time series of simulated values
    scale_color_manual(values = c("Accepted" = "dodgerblue", "Flagged" = "red"), drop = FALSE, name = "Remark Code") +                                                  ## Manually maps colors of measured data
    theme_bw() +                                                                                                             
    coord_cartesian(ylim = TS_Scale) +                                                                    ## Rescales  axes
    scale_x_datetime(date_breaks = "2 years", date_labels = "%Y") +                                                          ## Sets the breakpoints on the x-axis
    geom_hline(aes(yintercept = 0), color = "black", linetype = 3) +                                                         ## Adds a horizontal reference line at 0
    labs(y = paste0(unique(MEAS_DF$PCode)," (", unique(MEAS_DF$Units), ")", sep = ""),                                                                            ## Provides a y-axis label
         x = "Date") +                                                                                                       ## Provides a x-axis label
    theme(
      plot.title= element_text(size = 10, hjust = 0.5, color = 'blue'),                                                      ## Justifies and changes font size of title
      #panel.background = element_rect(fill = "white", color = "black"),                                                      ## Change the panel background and creates a black border
      text = element_text(size = 8),                                                                                         ## Resizes the text
      legend.position = "bottom",
      plot.margin = margin(unit(c(1, 1, 1, 5), "cm")),
      legend.margin = margin(unit(c(-10, 0, -1, 0), "cm")))                                                                  ## Positions the legend
  
  #********************************************************************************************************************
  #                                                       Regression Plot
  #                           Plots simulated data vs observed data with a liner regression line and 1:1 line
  #********************************************************************************************************************
  
  Reg <- ggplot(data = MEAS_DF, aes(x = Result_Obs, y = Result_Sim)) +
    geom_abline(aes(intercept = 0, slope = 1, color = "1:1", linetype = "1:1"), show.legend = FALSE) +                                ## Adds 1:1 line with constant color, size, and type (allows legend mapping)
    geom_smooth(aes(color = "Linear Regression", linetype = "Linear Regression"), method = "lm", se = FALSE, formula = "y ~ x", show.legend = TRUE) +          ## Adds linear regression line with constant color, size, and type (allows legend mapping)
    geom_point(aes(fill = factor(RCode, levels = c("Accepted", "Flagged"))), shape = 21, size = 2.5, alpha = 0.80, stroke = 0, color = "white", show.legend = TRUE) +  ## Plots Simulated vs Observed data
    coord_cartesian(ylim = Reg_Scale_Y, xlim = Reg_Scale_X) +                                           ## Rescales axes
    #scale_size_manual(name = "Lines", values = c("Linear Regression" = 1, "1:1" = 1)) +                                                             ## Manually sets size legend based on crosswalk
    scale_color_manual(name = "Lines", values = c("Linear Regression" = "red", "1:1" = "blue"), drop = FALSE) +                                                   ## Manually sets color legend based on crosswalk   
    scale_fill_manual(values = c("Accepted" = "dodgerblue", "Flagged" = "red"), drop = FALSE, name = "Remark Code") +                               ## Manually rescales fill color of datapoints (border is white)
    scale_linetype_manual(name = "Lines", values = c("Linear Regression" = 2, "1:1" = 1)) +                                                         ## Manually sets linetype legend based on crosswalk
    labs(x = paste0("Observed ", unique(MEAS_DF$PCode)," (", unique(MEAS_DF$Units), ")"),                                                                                 ## Provides x-axis label
         y = paste0("Simulated ", unique(MEAS_DF$PCode), " (", unique(MEAS_DF$Units), ")")) +                                                                              ## Provides a y-axis label
    theme_bw() +
    #guides(color = guide_legend(override.aes = list(linetype = c(1,2)))) +
    theme(
      plot.title = element_text(size = 20, hjust = 0.5),                                                       ## Justifies and changes font size of title
      #panel.background = element_rect(fill = "white", color = "black"),                                        ## Change the panel background and creates a black border
      text = element_text(size = 8),                                                                           ## Resizes the text
      legend.position = "bottom",
      plot.margin = margin(unit(c(1, 5, 1, 5), "cm")),
      legend.margin = margin(unit(c(-10, 0, -1, 0), "cm")),                                                    ## Positions the legend
      legend.title = element_blank(),                                                                          ## Drops legend title
      legend.key.width = unit(1, "cm"))                                                                        ## Widens legend key box to make lines easier to see
  
  #********************************************************************************************************************
  #                                                       Probability Plot
  #             Plots cumulative distribution function line for simulated and observed data (Axes flipped: x-axis is the probability)
  #********************************************************************************************************************
  temp_ECDF <- MEAS_DF %>%
    select(Date_Time, PCode, Result_Obs, Result_Sim, Units, Depth) %>%
    gather(-Date_Time, -PCode, -Units, -Depth, key = "Type", value = "Result")%>%                                       ## Casts the DF into a long format with a Sim vs Obs identifier that will be used to group the data
    dplyr::mutate(Type = gsub("^Result_", "", Type))
  
  prob_brks <- c(0, 0.01, 0.05, 0.10, 0.20, 0.25, 0.40, 0.50, 0.60, 0.75, 0.80, 0.90, 0.95, 1)                  ## Sets up the percentile labels
  
  Punit <- unique(temp_ECDF$Units)
  
  Prob <- ggplot(temp_ECDF, aes(Result, colour = Type)) +
    theme_bw() +                                                                                                ## sets up the initial plot
    stat_ecdf(geom = "line", linewidth = 0.8, pad = FALSE, na.rm = TRUE) +                                           ## Plots the cumulative distribution curve
    #scale_y_continuous(breaks = prob_brks, labels = prob_brks*100) +                                            ## Sets the range and breakpoints for teh Y-axis (bottom) using a probability normal distribution
    {if(str_detect(str_to_upper(unique(MEAS_DF$PCode)), "FLOW|FLW|FLOWCMS|FLOW_CMS")) scale_x_log10(labels = comma)} +                               ## For FLOW, plots values based on log10scale
    coord_flip() +                                                                                              ## Flips X and Y axis
    scale_y_continuous(breaks = prob_brks, labels = prob_brks*100) +                                            ## Sets the range and breakpoints for teh Y-axis (bottom) using a probability normal distribution
    #expand_limits(x = Prob_Scale) +                                                                             ## Sets scale of flipped x-axis
    scale_colour_manual(values = c("Obs" = "black", "Sim" = "red"))+                                            ## Creates a manual legend based on color
    labs(y = "Percent",
         x = paste(unique(MEAS_DF$PCode), " (", unique(MEAS_DF$Units), ")", sep = "")) +                                                            ## Labels the axes
    theme(
      panel.border = (element_rect(fill = NA, color = "black")),
      #panel.background = element_rect(fill = "white"),
      text = element_text(size = 8),                                                                            ## Resizes the text
      legend.position = "top",
      plot.margin = margin(unit(c(-0.5, 1, 1, 5), "cm")),
      legend.margin = margin(unit(c(-1, 0, -10, 0), "cm")),
      legend.title = element_blank()) 
  
  #********************************************************************************************************************
  #                                      Extract Summary Statistics and GoF Statistics
  #                                  Uses the output from the Calc_GOF_Stats subscript
  #                                 Stats based only on records with 'ACCEPTED' measurement RCodes
  #             If Segment/PCode had < 2 records or SD == 0, then GOF stats reported as NA and only report summary stats
  #                          If no records were 'ACCEPTED' all stats are reported as NA
  #********************************************************************************************************************
  
  Stats_temp <- Plot_SumStats %>%                                                                                         ## Uses the summary stats table to extract relevant stats for Segment/PCode
    dplyr::filter(PCode == unique(MEAS_DF$PCode) & Segment_ID == unique(MEAS_DF$Segment_ID)) %>%
    dplyr::select(-PCode, -Segment_ID, -Station_ID_Mrgd, -Plot_Title) %>%
    as.data.frame()
  
  GoF_temp <- Plot_GOF %>%                                                                                               ## Uses goodness of fit stats table to extract relevant stats for Segment/PCode
    dplyr::filter(PCode == unique(MEAS_DF$PCode) & Segment_ID == unique(MEAS_DF$Segment_ID)) %>%
    dplyr::select(-PCode, -Segment_ID, -Station_ID_Mrgd, -Plot_Title) %>%
    rename('GoF Metric' = GoF_Metric) %>%
    as.data.frame() 
  
  #********************************************************************************************************************
  #                                   Assembles multipanel plot
  #********************************************************************************************************************
  
  Stats_1 <- tableGrob(GoF_temp, theme = ttheme_minimal(base_size = 8, padding = unit(c(0.25,0.25), "cm")), rows = NULL)                             ## Converts the GoF DF to a figure object
  
  Stats_2 <- tableGrob(Stats_temp, theme = ttheme_minimal(base_size = 8, padding = unit(c(0.25,0.25), "cm")), rows = NULL)                           ## Converts the Summary Stats DF to a figure object
  
  #-------------------------- If statement to assemble figure w/ or w/o watershed map        
  if ("MAP" %in% Plot_Types) {                                                                                                                       ## Formats figure, if watershed map included
    
    Stats_Lay <- rbind(c(1, 1, NA, 3), c(2, NA, 3, 3))                                                                                                 ## Formats merging of Stats and map for panel, includes blank panels to avoid overplotting
    
    Stats_Figs <- arrangeGrob(Stats_2, Stats_1, Map_temp, layout_matrix = Stats_Lay, heights = c(0.5, 0.75), widths = c(1, 0.50, 0.50, 0.75))          ## Combines the two stats DF and map into a single panel
    
    GridOut <- arrangeGrob(TS, Reg, Prob, Stats_Figs, ncol = 2, top = text_grob(Main_Title, color = 'blue'),
                           bottom = text_grob(FootNote, color = "black", just = "center", size = 8))                                                    ## Combines 3 figures, stats and map into a single figure
    
  } else {                                                                                                                                           ## If no map included, combines two stats DF only
    
    Stats_Figs <- arrangeGrob(Stats_2, Stats_1, ncol = 1, nrow = 2)
    
    GridOut <- arrangeGrob(TS, Reg, Prob, Stats_Figs, ncol = 2, top = text_grob(Main_Title, color = 'blue'),                                         ## If no map selected, combines 3 figs and stats
                           bottom = text_grob(FootNote, color = "black", just = "center", size = 8))
    
  }
  
  
  #********************************************************************************************************************
  #                              IF Statement to save figure as PDF or JPG
  #                                             Default is JPG
  #********************************************************************************************************************
  
  
  if(str_to_upper(FIG_TYPE) == "PDF") {
    
    PlotFile <- paste(Path_Data_Out, "/Plot_Calib/",  FIG_NUMB, "_", unique(unique(MEAS_DF$PCode)), "_", File_Title, ".pdf", sep = "")
    
  } else {
    
    PlotFile <- paste(Path_Data_Out, "/Plot_Calib/", unique(unique(MEAS_DF$PCode)), "_", File_Title, ".jpg", sep = "")
  }
  
  #********************************************************************************************************************
  #                          IF Statement to either save figure or render as map for spatial comparisons
  #                              
  #                         The scripts do no allow both HTML and Static JPGS to be created simultaneously
  #                         
  #                         This optimizes script run times due to lengthy processing time for saving .JPG
  #********************************************************************************************************************
  
  if(OUTPUT == "HTML") {
    as_ggplot(GridOut)

    } else {
    
    ggsave(PlotFile, GridOut, width = 12, height = 6, dpi = 300)
  }
  
  
}


#********************************************************************************************************************
#                         FUNCTION to create single multipanel plot based on targeted Pcodes and WASP segment ID
#********************************************************************************************************************

PLOT_SINGLE_MULTI <- function(PCODE_VEC, SEGMENT_VEC) {
  
  NestData_Multi %>%                                                                                                   ## Filters NestedDF based on vector of PCodes
    dplyr::filter(PCode %in% PCODE_VEC & Segment_ID %in% SEGMENT_VEC) %>%
    {pwalk(.l = list(.$Model_Filt, .$Meas_Filt), PLOT_SAVE_NESTED_MULTIPANEL)}                                                ## Feeds filtered dataframe into boxplot function
  
}



#********************************************************************************************************************
#                         FUNCTION to render multipanel calibration figure with interactive watershed HTML map
#                   
#********************************************************************************************************************


HTML_CALIB <- function(PCODE) {
  
  #********************************************************************************************************************
  #                         Preprocesses DF for HTML mapping
  #********************************************************************************************************************
  
  temp <- NestData_Multi %>%                                                                                                                          ## Filters NestedDF based on vector of PCodes
    dplyr::filter(PCode == PCODE) %>%
    mutate(plot = pmap(.l = list(.$Model_Filt, .$Meas_Filt, rep("HTML", times = length(.$Meas_Filt))), PLOT_SAVE_NESTED_MULTIPANEL)) %>%               ## Generates multipanel plot and appends DF
    mutate(Long = Station_Order_Lookup$Long[match(Segment_ID, Station_Order_Lookup$Segment_ID)],                                                      ## Adds Lat/Long/Plot_Title metadata for spatial plotting
           Lat = Station_Order_Lookup$Lat[match(Segment_ID, Station_Order_Lookup$Segment_ID)],
           Plot_Title = Station_Order_Lookup$Plot_Title[match(Segment_ID, Station_Order_Lookup$Segment_ID)])
  
  #********************************************************************************************************************
  #                         Creates spatial HTML map of multipanel figures w/ pop out maps
  #********************************************************************************************************************
  
  if(nrow(temp) != 0) {                                                                                                                              ## Will only generate plot if PCODE is in teh DF
    leaflet() %>%
      addTiles() %>%                                                                                                                                   ## Adds basemap, which defaults to OpenStreets
      addPolygons(data = st_as_sf(Wtrshd_Map), color = "Blue", opacity = 1) %>%                                                                        ## Adds watershed map
      addPolylines(data = st_as_sf(Stream_Map), color = "Black", opacity = 1, weight = 0.75, fill = TRUE, fillColor = "Black", fillOpacity = 1) %>%    ## Adds stream network map
      addMarkers(data = temp, lng = ~Long, lat = ~Lat, label = ~Plot_Title, group = "pt", labelOptions = labelOptions(textsize = "14px")) %>%          ## Adds sampling location pins and labels
      addPopupGraphs(temp$plot, group = "pt", width = 1200, height = 600)                                                                              ## Adds multipanel plots to each pin
  }
}



#********************************************************************************************************************
#                         Calls functions using WALK2
#********************************************************************************************************************

# PLOT_SINGLE_MULTI(c("DO", "FLOWCMS"), c("1", "16"))                                                                                          ## Calls multipanel function based on selected PCODES/SEGMENTS

# 
# walk2(.x = NestData_Multi$Model_Filt, .y = NestData_Multi$Meas_Filt, PLOT_SAVE_NESTED_MULTIPANEL)                                 ## Calls the nested multipanel function for ALL PCODES

# pwalk(.l = list(MODEL_DF = NestData_Multi$Model_Filt, MEAS_DF = NestData_Multi$Meas_Filt, FIG_TYPE = "PDF", FIG_NUMB = NestData_Multi$Fig_Numb), PLOT_SAVE_NESTED_MULTIPANEL)                                 ## Calls the nested multipanel function for ALL PCODES

# pwalk(.l = list(NestData_Multi$Model_Filt, NestData_Multi$Meas_Filt), PLOT_SAVE_NESTED_MULTIPANEL)                                 ## Calls the nested multipanel function for ALL PCODES



