
#********************************************************************************************************************
#
#                     SOURCE FUNCTION Script to Generate Geomean for CHLA, TN, TP by Station
#
#                           Input data needs to be preprocessed based on Proc_Source Script
#                         Designed to Run as SOURCE Script Called in Master_Script v6.0 or later
#                              
#          Requires purrr and nested dataframe (NestedData_Box) and will exclude segments that do not exceed the Min_Obs_BoxDens cutoff                         
#
#********************************************************************************************************************


#********************************************************************************************************************
#                                Create function that will populate the Excel workbook
#                                
#                                Requires vector of DFs and vector of sheet names
#********************************************************************************************************************


POPULATE_EXCEL_WORKBOOK <- function(INPUT_DATA, WORKSHEET_NAME) {
  
  negStyle <- createStyle(fontColour = "#9C0006", bgFill = "#FFC7CE")                                                                                                ## Creates cell style for negative values
  
  addWorksheet(wb_Replc_Blnk, WORKSHEET_NAME)                                                                                                                        ## Adds worksheet based on vector of worksheet names
  writeData(wb_Replc_Blnk, sheet = WORKSHEET_NAME, INPUT_DATA)                                                                                                       ## Writes R DF to new worksheet
  setColWidths(wb_Replc_Blnk, sheet = WORKSHEET_NAME, cols = 1:ncol(INPUT_DATA), widths = "auto")                                                                    ## Sets widths of columns    
  
  conditionalFormatting(wb_Replc_Blnk, WORKSHEET_NAME, cols = 1:ncol(INPUT_DATA), rows = 2:(nrow(INPUT_DATA)+1), type = "expression", rule ="<0", style = negStyle)  ## Adds conditional formatting to highlight negative values
  
}


#********************************************************************************************************************
#                                      QA: Verify that Measured Data has TN, TP, CHLA Records and
#                                                  Geomean selected for plotting
#********************************************************************************************************************

if ("GEOMEAN" %in% Plot_Types) {                                                                                                                         ## Only Calculates Geomeans if GEOMEANS selected in Plot_Types
 
  #********************************************************************************************************************
  #                                                Create Vectors to Filter Data
  #********************************************************************************************************************
  
  PCode_Geom <- intersect(c("TN", "TP", grep("^CHLA", unique(Measured_Paired$PCode), value = TRUE)), unique(Measured_Paired$PCode))          ## Creates the vector of PCodes for Geomean calculation
  
   
  if (length(intersect(PCode_Geom, c("TN", "TP", grep("^CHLA", unique(Measured_Paired$PCode), value = TRUE)))) == 0) {                                   ## Ends script if no measured TN, TP, or CHLA to plot
    print("Geomean selected, but measured data does not contain records for TN, TP, or CHLA; No geomeans can be plotted")
  } else {                                                                                                                                               ## If Measured Data has Pcodes, then runs rest of script
    
    
    #********************************************************************************************************************
    #                                            Create Geomean Functions to be called by GGPLOT
    #********************************************************************************************************************
    G_mean <- function(y) {                                                                              ## Creates geomean function for table
      y = exp(mean(log(y)))
    }
    
    Geo_CI_Filt <- function(y){                                                                          ## Create FUNX to create DF w/ Geomean and CI for GGPLOT
      if(length(y) >= 3){
        Plot_CI <- data.frame(                                                                           ## Extracts the following variables using pluck
          y = exp(Rmisc::CI(log(y), ci = 0.95)) %>% pluck(.,"mean"),
          ymin = exp(Rmisc::CI(log(y), ci = 0.95)) %>% pluck(.,"lower"),
          ymax = exp(Rmisc::CI(log(y), ci = 0.95)) %>% pluck(.,"upper"))
      }else if(length(y) < 3) {                        ## Calculate Geomean and Geomean CI
        Plot_CI <- data.frame(                                                                           ## Extracts the following variables using pluck
          y = exp(mean(log(y), na.rm = TRUE)),
          ymin = NA,
          ymax = NA)
        return(Plot_CI)
      }
    }
    
    
    #********************************************************************************************************************
    #                                         Objects for File and Plot Naming
    #********************************************************************************************************************
    
    if (length(setdiff(c(1:12), Month_Geom)) == 0) {
      Period <- "Simulation Period"
    } else (Period <- str_c(range(lubridate::month(Month_Geom, label = TRUE, abbr = TRUE)), collapse = " to "))                         ## IFELSE to indicate whether the plot data are filtered by month
    
    File_Ext <- ifelse(str_detect(Period, "Sim"), "SimPer", "Filt")
    Year_Vec <- seq(min(Model_Intrp$Year), max(Model_Intrp$Year), by = 2)                                                                  ## Creates the vector of years that will name figures
    #********************************************************************************************************************
    #                               Simulation Period: Calculate Table of Geomean and Conf Intervals
    #********************************************************************************************************************
    
    Geom_Model <- Model_Intrp %>%
      dplyr::filter(PCode %in% PCode_Geom,
                    Month %in% Month_Geom) %>%                                                                                                 ## Filters data based on PCode
      gather(Result_Sim_Full, key = "Obs_Sim", value = "Result") %>%                                                                    ## Reshapes data
      group_by(Segment_ID, PCode, Obs_Sim) %>%                                                                                        ## Group data for Geomean calc
      summarize(Numb_Rec = n(),
                Geomean = exp(Rmisc::CI(log(Result), ci = 0.95)) %>% pluck(.,"mean"),                                                   ## Extracts Geomean
                Lower_95CI = exp(Rmisc::CI(log(Result), ci = 0.95)) %>% pluck(.,"lower"),                                               ## Extracts Lower 95% CI for Geomean
                Upper_95CI = exp(Rmisc::CI(log(Result), ci = 0.95)) %>% pluck(.,"upper")) %>%                                           ## Extracts Upper 95% CI for Geomean   
      dplyr::mutate(Lower_95CI = case_when(Numb_Rec < 3 ~ as.numeric(NA),
                                           TRUE ~ Lower_95CI),
                    Upper_95CI = case_when(Numb_Rec < 3 ~ as.numeric(NA),
                                           TRUE ~ Upper_95CI)) 
    
    Geom_Meas <- Measured_Paired %>%
      dplyr::filter(PCode %in% PCode_Geom,
                    Month %in% Month_Geom) %>%                                                                                                 ## Filters data based on PCode
      select(Segment_ID, PCode, Date_Time, Result_Obs, Result_Sim) %>%                                                                  ## Drops columns and reshapes data
      gather(Result_Obs, Result_Sim, key = "Obs_Sim", value = "Result") %>%                                                             ## Groups data for Geomean calc
      group_by(Segment_ID, PCode, Obs_Sim) %>%                                                                                          ## Groups data for Geomean calc
      summarize(Numb_Rec = n(),
                Geomean = exp(Rmisc::CI(log(Result), ci = 0.95)) %>% pluck(.,"mean"),                                                   ## Extracts Geomean
                Lower_95CI = exp(Rmisc::CI(log(Result), ci = 0.95)) %>% pluck(.,"lower"),                                               ## Extracts Lower 95% CI for Geomean
                Upper_95CI = exp(Rmisc::CI(log(Result), ci = 0.95)) %>% pluck(.,"upper")) %>%                                           ## Extracts Upper 95% CI for Geomean   
      dplyr::mutate(Lower_95CI = case_when(Numb_Rec < 3 ~ as.numeric(NA),
                                           TRUE ~ Lower_95CI),
                    Upper_95CI = case_when(Numb_Rec < 3 ~ as.numeric(NA),
                                           TRUE ~ Upper_95CI))                                                                          ## Converts u to mu microgram symbol
    
    Geom_Summary <- bind_rows(Geom_Model, Geom_Meas) %>%                                                                                ## Merges Geomean calcs for Measured and Modeled DF
      select(-Numb_Rec) %>%
      dplyr::mutate_if(is.numeric, round, 3) %>%                                                                                               ## Rounds calculations
      dplyr::mutate(Units = Measured_Paired$Units[match(PCode, Measured_Paired$PCode)],                                                              ## Adds units and Station_Name column back in based on Measured DF
                    Station_ID_Mrgd = Station_Order_Lookup$Station_ID_Mrgd[match(Segment_ID, Station_Order_Lookup$Segment_ID)], 
                    Plot_Title = Station_Order_Lookup$Plot_Title[match(Segment_ID, Station_Order_Lookup$Segment_ID)]) %>%
      select(PCode, Segment_ID, Plot_Title, Station_ID_Mrgd, Units, Obs_Sim, everything()) %>%                                                         ## Reorders columns
      as.data.frame() %>%                                                                                                               ## Converts to DF
      arrange(PCode, Plot_Title, Obs_Sim) %>%                                                                                         ## Sorts DF
      gather(grep("mean", names(.)), grep("Lower", names(.)), grep("Upper", names(.)), key = "Stat", value = "Result") %>%              ## Reshapes data
      dplyr::mutate(Obs_Sim = case_when(Obs_Sim == "Result_Obs" ~ "Observed-Paired",                                                           ## Remaps grouping variables
                                        Obs_Sim == "Result_Sim" ~ "Simulated-Paired",
                                        Obs_Sim == "Result_Sim_Full" ~ "Simulated-Entire Period",
                                        TRUE ~ as.character(NA))) %>%
      unite(temp, Obs_Sim, Stat, sep = ": ") %>%                                                                                        ## Combines columns for reshaping
      spread(temp, Result)  %>%                                                                                                         ## Casts DF wide
      dplyr::mutate(Station_ID_Mrgd = Station_Order_Lookup$Station_ID_Mrgd[match(Segment_ID, Station_Order_Lookup$Segment_ID)]) %>%                  ## Adds Merged Name to the DF
      select("PCode", "Segment_ID", "Plot_Title", "Station_ID_Mrgd", "Units", matches("^Obs"), 
             matches("^Simulated-Paired"), matches("^Simulated-Entire"))                                                               ## Reorders columns
    
    Geom_Sum_Sub <- Geom_Summary %>%                                                                                                    ## Drops stations with no calibration data
      dplyr::filter(!is.na(`Observed-Paired: Geomean`))
    
    #********************************************************************************************************************
    #                                Annual: Calculate Table of Geomean and Conf Intervals
    #********************************************************************************************************************
    Yr_Model <- Model_Intrp %>%
      dplyr::filter(PCode %in% PCode_Geom,
                    Month %in% Month_Geom) %>%                                                                                                 ## Filters data based on PCode
      gather(Result_Sim_Full, key = "Obs_Sim", value = "Result") %>%                                                                     ## Reshapes data
      group_by(Segment_ID, PCode, Obs_Sim, Year) %>%                                                                                    ## Group data for Geomean calc
      summarize(Geomean = exp(mean(log(Result), na.rm = TRUE)))
    
    Yr_Meas <- Measured_Paired %>%
      dplyr::filter(PCode %in% PCode_Geom,
                    Month %in% Month_Geom) %>%                                                                                                 ## Filters data based on PCode
      gather(Result_Obs, Result_Sim, key = "Obs_Sim", value = "Result") %>%                                                             ## Groups data for Geomean calc
      group_by(Segment_ID, PCode, Obs_Sim, Year) %>%                                                                                    ## Groups data for Geomean calc
      summarize(Geomean = exp(mean(log(Result), na.rm = TRUE)))
    
    Yr_Summary <- bind_rows(Yr_Model, Yr_Meas) %>%                                                                                      ## Merges Geomean calcs for Measured and Modeled DF
      dplyr::mutate_if(is.numeric, round, 3) %>%                                                                                               ## Rounds the calculations
      dplyr::mutate(Units = Measured_Paired$Units[match(PCode, Measured_Paired$PCode)],                                                              ## Adds the units and Plot_Title column back in based on Measured DF
                    Station_ID_Mrgd = Station_Order_Lookup$Station_ID_Mrgd[match(Segment_ID, Station_Order_Lookup$Segment_ID)],
                    Plot_Title = Measured_Paired$Plot_Title[match(Segment_ID, Measured_Paired$Segment_ID)]) %>% 
      select(PCode, Segment_ID, Plot_Title, Year, Units, Obs_Sim, matches("mean")) %>%                                                   ## Reorders columns
      as.data.frame() %>%                                                                                                               ## Converts to DF
      arrange(PCode, Segment_ID, Year, Obs_Sim) %>%                                                                                   ## Sorts DF
      gather(Geomean, key = "Stat", value = "Result") %>%                                                                               ## Reshapes data
      dplyr::mutate(Obs_Sim = case_when(Obs_Sim == "Result_Obs" ~ "Observed-Paired",                                                           ## Remaps the grouping variables
                                        Obs_Sim == "Result_Sim" ~ "Simulated-Paired",
                                        Obs_Sim == "Result_Sim_Full" ~ "Simulated-Entire Period",
                                        TRUE ~ as.character(NA))) %>% 
      unite(temp, Obs_Sim, Stat, sep = ": ") %>%                                                                                        ## Combines columns for reshaping
      spread(temp, Result)  %>%                                                                                                         ## Casts the DF wide
      dplyr::mutate(Station_ID_Mrgd = Station_Order_Lookup$Station_ID_Mrgd[match(Segment_ID, Station_Order_Lookup$Segment_ID)]) %>%                  ## Adds Merged Name to the DF
      select("PCode", "Segment_ID", "Plot_Title", "Station_ID_Mrgd", "Year", "Units",                                                     ## Reorders columns
             matches("^Obs"), matches("Simulated-Paired"), matches("Simulated-Entire Period"))
    
    #********************************************************************************************************************
    #                               Write out Excel File that Reports Geomean Summaries
    #********************************************************************************************************************
    
    wb_Replc_Blnk <- createWorkbook()    
    walk2(.x = list(Geom_Summary, Geom_Sum_Sub, Yr_Summary), .y = list("SimPerd_AllStat", "SimPerd_woMissng", "Annual_AllStat"), ~POPULATE_EXCEL_WORKBOOK(.x, .y))
    
    openxlsx::saveWorkbook(wb_Replc_Blnk, paste(Path_Data_Out, "/Stats/Gmean_", File_Ext, "_", Model_Run_Code,  ".xlsx", sep = ""), overwrite = TRUE)   
    
    print(paste("Geomean statistics calculated based on script", Geomean_Script, sep = " "))
    
    
  }
}


#********************************************************************************************************************
#                                                Custom function to plot geomeans across all years
#********************************************************************************************************************

PLOT_SAVE_NESTED_GEOMEAN <- function(MODEL_DF, MEAS_DF) {
  
    if(nrow(MEAS_DF) > 0) {                                                                                                                 ## If statement so skip PCodes that had no measured data at any segment
    #********************************************************************************************************************
    #                                    Uses geomean calculations to set upper limit
    #********************************************************************************************************************
    
    Geom_Scale <- Geom_Summary %>%                                                                                          ##  Sets Y-scale for across year plot
      dplyr::filter(PCode == unique(MEAS_DF$PCode)) %>%
      select(contains("Upper")) %>%
      max(.) %>% 
      c(0, .)                                                                                                               ## Range set to 0, maximum geomean of OBS and SIM
    
    #********************************************************************************************************************
    #                                                Filter Data Prior to Plotting
    #********************************************************************************************************************
    Temp_Meas <- MEAS_DF %>%                                                                                           ## Selects only months in month vector
      dplyr::filter(Month %in% Month_Geom) %>%
      dplyr::mutate(Units = str_replace(Units, "^u", "\u03BC"))
    
    Temp_Model <- MODEL_DF %>%                                                                                             ## Filters the model data for across year plot
      dplyr::filter(Month %in% Month_Geom) %>%
      dplyr::filter(Segment_ID %in% unique(Temp_Meas$Segment_ID))                                                               ## For plot across all years, removes stations w/ calib data
    
    
    #********************************************************************************************************************
    #                           Across Year Scatterplot w/ GeoMean for OBS-Paired, SIM-Paired, SIM-Entire Period
    #********************************************************************************************************************
    Plot_Geom <- ggplot() +
      stat_summary(data = Temp_Model,                                                                                                                                 ## Calls FUNX to add error bars for SIM data from entire simulation period 
                   aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)),  y = Result_Sim_Full, color = "Result_Sim_Full", shape = "Result_Sim_Full"), 
                   fun.data = Geo_CI_Filt, na.rm = TRUE, geom = "pointrange", size = 0.75, position  = position_nudge(x = 0.1), show.legend = TRUE, alpha = 0.75) +
      stat_summary(data = Temp_Meas,                                                                                                                                  ## Calls FUNX to add error bars for SIM data from paired DF 
                   aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)),  y = Result_Sim, color = "Result_Sim", shape = "Result_Sim"),               
                   fun.data = Geo_CI_Filt, na.rm = TRUE, geom = "pointrange", size = 0.75, position  = position_nudge(x = -0.1), show.legend = TRUE, alpha = 0.75) +            
      stat_summary(data = Temp_Meas,                                                                                                                                  ## Calls FUNX to add error bars for OBS data from paired DF 
                   aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)),  y = Result_Obs, color = "Result_Obs", shape = "Result_Obs"),                                                            
                   fun.data = Geo_CI_Filt, na.rm = TRUE, geom = "pointrange", size = 0.75, show.legend = TRUE, alpha = 0.75) +
      coord_cartesian(ylim = c(Geom_Scale[1], Geom_Scale[2])) +
      #expand_limits(y = c(0, max(c(Temp_Model$Result_Sim_Full, Temp_Meas$Result_Sim, Temp_Meas$Result_Obs)))) +                                                                                                            ## Rescales y-axis to same range 
      scale_color_manual(name = NULL, values = c("Result_Obs" = "black", "Result_Sim" = "red", "Result_Sim_Full" = "red"), 
                         labels = c("Result_Obs" = "Observed: \nPaired", "Result_Sim" = "Simulated: \nPaired", "Result_Sim_Full" = "Simulated: \nAll Records")) +           ## Overrides color scheme for manual legend
      scale_shape_manual(name = NULL, values = c("Result_Obs" = 15, "Result_Sim" = 19, "Result_Sim_Full" = 17), 
                         labels = c("Result_Obs" = "Observed: \nPaired", "Result_Sim" = "Simulated: \nPaired", "Result_Sim_Full" = "Simulated: \nAll Records")) +
      labs(title = paste(unique(MEAS_DF$PCode), "\n (Geometric Mean +/- 95% Conf. Int.)", "\n", Period, sep = " "),                                                                       ## Labels the plot and axes 
           x = "Stations \n Downstream to Upstream",
           y = paste(unique(MEAS_DF$PCode), " (", unique(MEAS_DF$Units), ")", sep = ""),
           caption = paste0("Date Plotted: ", Sys.Date())) +
      theme_bw() +
      theme(plot.title = element_text(hjust = 0.5),                                                                                                                   ## Formats the plot        
            #panel.background = element_rect(fill = "white", color = "black"),                                                                                         ## Change  panel background and creates black border
            axis.text = element_text(size = 10),
            axis.text.x = element_text(angle = 80, hjust = 1,face = "bold"),
            legend.position = "bottom",
            legend.text = element_text("hjust"))
    
    if(n_distinct(Temp_Model$Segment_ID) == 1) {                                                                                                                    ## Uses IF statement to change jpg format
      ggsave(paste(Path_Data_Out, "/Plot_Geom/", unique(MEAS_DF$PCode), "_Geo.jpg", sep = ""), Plot_Geom, width = 5, height = 8, dpi = 300)
    } else (ggsave(paste(Path_Data_Out, "/Plot_Geom/", unique(MEAS_DF$PCode), "_Geo.jpg", sep = ""), Plot_Geom, width = 5, height = 8, dpi = 300))
  }
}


#********************************************************************************************************************
#                          Custom Function to Plot Annual Scatterplot w/ GeoMean for OBS-Paired, SIM-Paired, SIM-Entire Period
#********************************************************************************************************************


PLOT_ANNUAL_NESTED_GEOMEAN <- function(MODEL_DF, MEAS_DF) {
  
  if(nrow(MEAS_DF) > 0) {                                                                                                                 ## If statement so skip PCodes that had no measured data at any segment
    #********************************************************************************************************************
    #                                    Uses geomean calculations to set upper limit
    #********************************************************************************************************************
    
    Geom_Scale_Yr <- bind_rows(Yr_Summary) %>%                                                                              ##  Sets Y-scale for annual plots
      as.data.frame() %>%
      dplyr::filter(PCode == unique(MEAS_DF$PCode)) %>%
      select(contains("mean")) %>%
      max(., na.rm = TRUE) %>%
      c(0, .)                                                                                                               ## Range set to 0, maximum geomean of OBS and SIM
    
    
    
    #********************************************************************************************************************
    #                                                Filter Data Prior to Plotting
    #********************************************************************************************************************
    Temp_Meas <- MEAS_DF %>%                                                                                               ## Selects only months in month vector
      dplyr::filter(Month %in% Month_Geom) %>%
      dplyr::mutate(Units = str_replace(Units, "^u", "\u03BC"))
    
    Temp_Model_Yr <- MODEL_DF %>%                                                                                          ## Filters model data for annual compare
      dplyr::filter(Month %in% Month_Geom)                                                                                      
    
    ## If statement to skip annual plots, if not selected on Lookup
    
    ggplot() +
      stat_summary(data = Temp_Model_Yr,                                                                                                                                      ## Calls FUNX SIM data from entire simulation period 
                   aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)),  y = Result_Sim_Full, color = "Result_Sim_Full", shape = "Result_Sim_Full"),          
                   fun = G_mean, na.rm = TRUE, geom = "point", size = 4, position  = position_nudge(x = 0.1), show.legend = TRUE, alpha = 0.75) +
      stat_summary(data = Temp_Meas,                                                                                                                                          ## Calls FUNX for SIM data from paired DF
                   aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)),  y = Result_Sim, color = "Result_Sim", shape = "Result_Sim"),                       
                   fun = G_mean, na.rm = TRUE, geom = "point", size = 4, position  = position_nudge(x = -0.1), show.legend = TRUE, alpha = 0.75) +           
      stat_summary(data = Temp_Meas,                                                                                                                                          ## Calls FUNX for OBS data from paired DF
                   aes(x = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)),  y = Result_Obs, color = "Result_Obs", shape = "Result_Obs"),                       
                   fun = G_mean, na.rm = TRUE, geom = "point", size = 4,  show.legend = TRUE, alpha = 0.75) +
      coord_cartesian(ylim = c(Geom_Scale_Yr[1], Geom_Scale_Yr[2])) +
      #expand_limits(y = c(0, Geom_Scale[2])) +                                                                                                                               ## Rescales y-axis to same range 
      scale_color_manual(name = NULL, values = c("Result_Obs" = "black", "Result_Sim" = "red", "Result_Sim_Full" = "red"), 
                         labels = c("Result_Obs" = "Observed: Paired", "Result_Sim" = "Simulated: Paired", "Result_Sim_Full" = "Simulated: All Records")) +                   ## Overrides color scheme for manual legend
      scale_shape_manual(name = NULL, values = c("Result_Obs" = 15, "Result_Sim" = 19, "Result_Sim_Full" = 17), 
                         labels = c("Result_Obs" = "Observed: Paired", "Result_Sim" = "Simulated: Paired", "Result_Sim_Full" = "Simulated: All Records")) +
      labs(title = paste(unique(MEAS_DF$PCode), "\n (Geometric Mean)", "\n", Period, sep = " "),                                                                              ## Labels the plot and axes 
           x = "Stations \n Downstream to Upstream",
           y = paste(unique(Temp_Meas$PCode), " (", unique(Temp_Meas$Units), ")", sep = ""),
           caption = paste0("Date Plotted: ", Sys.Date())) +
      theme_bw() +
      theme(plot.title = element_text(hjust = 0.5),                                                                                                                           ## Formats the plot        
            #panel.background = element_rect(fill = "white", color = "black"),                                                                                                 ## Change  panel background and creates black border
            axis.text = element_text(size = 10),
            axis.text.x = element_text(angle = 80, hjust = 1, face = "bold"),
            legend.position = "bottom") +
      facet_wrap_paginate(~Year, ncol = 1, nrow = 2, page = 1, labeller = label_both)  
    
    pages <- n_pages(ggplot2::last_plot())                                                                                                                                    ## Calculate number of pages needed to capture all facet pages
    
    for (j in 1:pages) {                                                                                                                                                      ## For loop to iteratively plot and save each page of facet wrap paginate
      ggsave(paste(str_remove(Path_Data_Out, "/Proc_Out"), "/Plot_Geom/Year/", unique(unique(MEAS_DF$PCode)), "_", j, ".jpg", sep = ""), ggplot2::last_plot() + 
               facet_wrap_paginate(~Year, ncol = 1, nrow = 2, page = j, labeller = label_both), width = 8, height = 10, dpi = 300)
      
    }
  }
}
  
 

