
#********************************************************************************************************************
#
#                    Includes spatial viewer scripts and arguments (including MAP vs. SAVE argument to either save figures or create spatial map)
#       
#       Standalone FUNCTION Script to Generate single Time Series figure that is comparable to the TS panel in the Calibration Multipanel
#                  
#                        Script will not be called by Master Script and has to be run independently
#                      
#                        Allows creation of larger time series figure that is identical to the multipanel figure
#                      
#                         Input data needs to be preprocessed based on Proc_Source Script
#                              Designed to Run as SOURCE Script Called in Master_Script v6.0 or later
#
#           Requires purrr and nested dataframe (NestedData_Multi)  and will exclude segments that do not exceed the Min_Obs cutoff 
#           
#                                         Includes PDF vs. JPG ggsave argument 
#                                         
#                                         J. Davis v2021-07-14                                               
#
#********************************************************************************************************************

#********************************************************************************************************************
#                                   Calls custom function to create Plot_TS subfolder if missing
#                                      
#********************************************************************************************************************


CREATE_SUBFOLDER("TIME_SERIES")                                                   ## Calls custom function to create Time Series subfolder


#********************************************************************************************************************
#                                   Create custom function to create time series figure
#                                   
#                      OUTPUT argument defaults to SAVE, but can be set to MAP (allowing maps to be displayed)
#                                   
#                           Includes Fig_Type argument that allows figure to be saved as PDF or JPG
#                                             Default is to save it as a JPG
#                                             
#                          Includes Fig_Numb argument that allows Figure # to be included in the figure title
#                                     Default is to exclude the figure # from the figure title
#                                      
#********************************************************************************************************************



PLOT_TIME_SERIES <- function(MODEL_DF, MEAS_DF, OUTPUT = "SAVE", FIG_TYPE = "JPG", FIG_NUMB = NA) {
  
  
  #********************************************************************************************************************
  #                                   Loop Over Data to Generate Figure Scales
  #********************************************************************************************************************
  if(str_detect(str_to_upper(unique(MEAS_DF$PCode)), "PH|WSE")) {                                                     ##  Uses IFELSE statement to generate vectors to set figure axes scales; WSE and PH zoomed to data range
    TS_Scale <- range(c(MEAS_DF$Result_Obs, MODEL_DF$Result_Sim_Full), na.rm = TRUE)                                  ##  Sets range based on combined FULL Sim and paired Obs data
    Reg_Scale_Y <- range(c(MEAS_DF$Result_Obs, MEAS_DF$Result_Sim), na.rm = TRUE)                                     ##  Sets Y-scale of regression based on range of paired OBS and SIM data 
    Reg_Scale_X <- range(c(MEAS_DF$Result_Obs, MEAS_DF$Result_Sim), na.rm = TRUE)                                     ##  Sets X-scale of regression based on range of paired OBS and SIM data
    Prob_Scale <- range(c(MEAS_DF$Result_Obs, MEAS_DF$Result_Sim))                                                    ##  Sets Y-scale of prob plot based on range of paired OBS and SIM data
  } else {                                                                                                            ##  As above, but sets the min value to 0 for all figures and max based on max
    TS_Scale <- c(0, max(c(MEAS_DF$Result_Obs, MODEL_DF$Result_Sim_Full), na.rm = TRUE))                 
    Reg_Scale_Y <- c(0, max(c(MEAS_DF$Result_Obs, MEAS_DF$Result_Sim), na.rm = TRUE))
    Reg_Scale_X <- c(0, max(c(MEAS_DF$Result_Obs, MEAS_DF$Result_Sim), na.rm = TRUE)) 
    Prob_Scale <- c(0, max(c(MEAS_DF$Result_Obs, MEAS_DF$Result_Sim), na.rm = TRUE))
  }
  #********************************************************************************************************************
  #                                                Time Series Plot
  #           Plots simulated data as a line plot, with the measured data overlaid as scatterplot (color based on Flagged vs Accepted)
  #********************************************************************************************************************
  
  #********************************************************************************************************************
  #                              IF Statement to include Figure # in Plot Title or not
  #                                       Default is to exclude the Figure #
  #********************************************************************************************************************
  
  
  if(is.na(FIG_NUMB)) {
    
    Main_Title <- paste(paste(unique(MEAS_DF$Plot_Title), collapse = " / "),                                                 ## Create plot title based on WASP Seg and Station Name
                        " \nParameter: ", unique(MODEL_DF$PCode), sep = "")  
  } else {
    
    Main_Title <- paste(paste0("Fig. ", FIG_NUMB, ": ", unique(MEAS_DF$Plot_Title), collapse = " / "),                    ## Create plot title based on WASP Seg and Station Name
                        " \nParameter: ", unique(MODEL_DF$PCode), sep = "")  
  }
  
  #********************************************************************************************************************
  #                              Create footnote and filenames
  #********************************************************************************************************************
  
  
  FootNote <- paste("\n(Calib Station: ",                                                                                    ## Adds a footnote to figure indicating segment and sampling station
                    gsub("_", " / ", unique(MODEL_DF$Station_ID_Mrgd)), 
                    "; ", Model_Type, " Seg: ", unique(MEAS_DF$Segment_ID), ")", "\n Date Plotted: ", Sys.Date(), sep = "")
  
  File_Title <- paste(unique(MEAS_DF$Plot_Title), collapse = " / ") %>%                                                    ## Creates filename
    str_trim(side = "both") %>%
    str_trunc(., width = 20, ellipsis = "") %>%
    gsub(" / ", "_", . ) %>%
    gsub("\\.|\\. ", " ", .) %>%
    gsub(" ", "_", .)
  
  
  TS <- ggplot() +                                                                                                                                                      ## Calls up ggplot
    geom_line(data = MODEL_DF, aes(x = Date_Time, y = Result_Sim_Full), linetype = 1, color = "black",  linewidth = 0.5) +                                                 ## Creates time series of simulated values
    geom_point(data = MEAS_DF, aes(x = Date_Time, y = Result_Obs, color = factor(RCode, levels = c("Accepted", "Flagged"))), size = 1.75, alpha = 0.8, show.legend = TRUE ) +            ## Adds the measured observations to the time series (Colored based on Flagged vs Accepted)
    scale_color_manual(values = c("Accepted" = "dodgerblue", "Flagged" = "red"), drop = FALSE, name = "Remark Code") +                                                  ## Manually maps colors of measured data
    theme_bw() +                                                                                                             
    coord_cartesian(ylim = TS_Scale) +                                                                    ## Rescales  axes
    scale_x_datetime(date_breaks = "2 years", date_labels = "%Y") +                                                          ## Sets the breakpoints on the x-axis
    geom_hline(aes(yintercept = 0), color = "black", linetype = 3) +                                                         ## Adds a horizontal reference line at 0
    labs(title = Main_Title,
         caption = FootNote,
         y = paste0(unique(MEAS_DF$PCode)," (", unique(MEAS_DF$Units), ")", sep = ""),                                                                            ## Provides a y-axis label
         x = "Date") +                                                                                                       ## Provides a x-axis label
    theme(
      text = element_text(size = 8),                                                                                         ## Resizes the text
      plot.title= element_text(size = 12, hjust = 0.5, color = 'blue'),                                                      ## Justifies and changes font size of title
      plot.caption = element_text(hjust = 0.5, size = 8),
      #panel.background = element_rect(fill = "white", color = "black"),                                                      ## Change the panel background and creates a black border
      legend.position = "bottom",
      plot.margin = margin(unit(c(1, 1, 1, 5), "cm")),
      legend.margin = margin(unit(c(-10, 0, -1, 0), "cm")))                                                                  ## Positions the legend
  
  #********************************************************************************************************************
  #                              IF Statement to either save figure or render as map for spatial comparisons
  #********************************************************************************************************************
  
  if(OUTPUT == "HTML") {
    ggplot2::last_plot()
  } else {
    
    #********************************************************************************************************************
    #                              IF Statement to allow generation of PDFs
    #********************************************************************************************************************
    
    
    if(str_to_upper(FIG_TYPE) == "PDF") {
      
      PlotFile <- paste(Path_Data_Out, "/Plot_TS/",  FIG_NUMB, "_", unique(unique(MEAS_DF$PCode)), "_", File_Title, ".pdf", sep = "")
      
    } else {
      
      PlotFile <- paste(Path_Data_Out, "/Plot_TS/", unique(unique(MEAS_DF$PCode)), "_", File_Title, ".jpg", sep = "")
    }
    
    ggsave(PlotFile, TS, width = 12, height = 6, dpi = 300)
  }
  
  
}


#********************************************************************************************************************
#                         FUNCTION to render Time Series figured with interactive watershed map
#                   
#********************************************************************************************************************

HTML_TS <- function(PCODE) {
  
  #********************************************************************************************************************
  #                         Preprocesses DF for mapping
  #********************************************************************************************************************
  temp <- NestData_Multi %>%                                                                                                     ## Filters NestedDF based on vector of PCodes
    dplyr::filter(PCode == PCODE) %>%                                                                                            
    mutate(plot = pmap(.l = list(.$Model_Filt, .$Meas_Filt, rep("HTML", times = length(.$Meas_Filt))), PLOT_TIME_SERIES)) %>%     ## Generates the time series plot and appends DF
    mutate(Long = Station_Order_Lookup$Long[match(Segment_ID, Station_Order_Lookup$Segment_ID)],                                 ## Adds Lat/Long/Plot_Title metadata for spatial plotting
           Lat = Station_Order_Lookup$Lat[match(Segment_ID, Station_Order_Lookup$Segment_ID)],
           Plot_Title = Station_Order_Lookup$Plot_Title[match(Segment_ID, Station_Order_Lookup$Segment_ID)])
  
  #********************************************************************************************************************
  #                         Creates spatial map of time series figures w/ pop out maps
  #********************************************************************************************************************
  
  leaflet() %>%
    addTiles() %>%                                                                                                                                    ## Adds basemap, which defaults to OpenStreets
    addPolygons(data = st_as_sf(Wtrshd_Map), color = "Blue", opacity = 1) %>%                                                                         ## Adds watershed map
    addPolylines(data = st_as_sf(Stream_Map), color = "Black", opacity = 1, weight = 0.75, fill = TRUE, fillColor = "Black", fillOpacity = 1) %>%     ## Adds stream network map
    addMarkers(data = temp, lng = ~Long, lat = ~Lat, label = ~Plot_Title, group = "pt", labelOptions = labelOptions(textsize = "14px")) %>%           ## Adds sampling location pins and labels
    addPopupGraphs(temp$plot, group = "pt", width = 1200, height = 600)                                                                               ## Adds time series plot to each pin
  
}


#********************************************************************************************************************
#                        Calls time series plotting function to generate static plots
#********************************************************************************************************************

walk2(.x = NestData_Multi$Model_Filt, .y = NestData_Multi$Meas_Filt, PLOT_TIME_SERIES)                         ## Calls the nested time series function for ALL PCODES

#********************************************************************************************************************
#                       Calls time series plotting function to generate HTML interactive plots
#********************************************************************************************************************

#CREATE_SUBFOLDER("HTML")                                                                                       ## Calls custom function to create HTML subfolder
#walk(.x = unique(PCode_Lookup$PCode), ~withr::with_dir(paste0(Path_Data_Out, "/Plot_HTML/"), saveWidget(HTML_TS(.x), title = paste0(.x, "_TS"), file = paste0(.x, "_TS.html"))))
#HTML_TS("TN")
